@@ -28,6 +28,7 @@
 #include "ev_MacToolbar.h"
 #include "xap_MacApp.h"
 #include "xap_MacFrame.h"
+#include "xap_MacToolbar_Control.h"
 #include "ev_Toolbar_Actions.h"
 #include "ev_Toolbar_Layouts.h"
 #include "ev_Toolbar_Labels.h"
@@ -52,6 +53,7 @@ EV_MacToolbar::EV_MacToolbar(XAP_MacApp * pMacApp, XAP_MacFrame * pMacFrame,
 	m_pViewListener = NULL;
 	m_lid = 0;							// view listener id
 	m_MacWindow = pMacFrame->getMacWindow ();
+	m_hMainControl = NULL;
 }
 
 EV_MacToolbar::~EV_MacToolbar(void)
@@ -116,14 +118,19 @@ bool EV_MacToolbar::synthesize(void)
 {
 	OSErr err;
 	ControlHandle rootControl;
-	ControlHandle toolbarControl;
+	XAP_MacToolbar_Control *appWideControl;
+	Rect toolbarRect;
+	const UInt16 BTN_WIDTH = XAP_MacToolbar_Control::getButtonWidth ();
+	const UInt16 BTN_HEIGHT = XAP_MacToolbar_Control::getButtonHeight ();
+	const UInt16 BTN_SPACE = XAP_MacToolbar_Control::getButtonSpace ();
 
-    short btnX = 8;
-    WindowPtr owningWin = m_pMacFrame->getMacWindow ();
+    short btnX = BTN_SPACE;
+	appWideControl = ((XAP_MacApp*)m_pMacFrame->getApp())->getToolbarControl();
+    WindowPtr owningWin = appWideControl->getWindow();
     UT_ASSERT (owningWin);
 	// create a toolbar from the info provided.
 
-	xxx_UT_DEBUGMSG(("EV_MacToolbar::synthesize() called !\n"));
+	UT_DEBUGMSG(("EV_MacToolbar::synthesize() called !\n"));
 	const EV_Toolbar_ActionSet * pToolbarActionSet = m_pMacApp->getToolbarActionSet();
 	UT_ASSERT(pToolbarActionSet);
 	
@@ -136,17 +143,16 @@ bool EV_MacToolbar::synthesize(void)
     Rect btnRect;
 
     /* create the toolbar */
-	m_toolbarRect.top = m_pMacFrame->_getVisibleRgnTop();
-    _calcToolbarRect ();
-	m_pMacFrame->_setVisibleRgnTop (m_toolbarRect.bottom);
-	err = ::GetRootControl(m_MacWindow, &rootControl);
+	appWideControl->requestToolbarRect (toolbarRect);
+	err = ::GetRootControl(owningWin, &rootControl);
 	UT_ASSERT (err == noErr);
 	if (err) {
 		UT_DEBUGMSG(("GetRootControl failed: %d\n", err));
 	}
-	::CreateWindowHeaderControl (m_MacWindow, &m_toolbarRect, false, &toolbarControl);
-	UT_ASSERT (toolbarControl);
-	err = ::EmbedControl (toolbarControl, rootControl);
+	::CreateWindowHeaderControl (owningWin, &toolbarRect, false, &m_hMainControl);
+	UT_ASSERT (m_hMainControl);
+	::HideControl (m_hMainControl);
+	err = ::EmbedControl (m_hMainControl, rootControl);
 	UT_ASSERT (err == noErr);
 	if (err) {
 		UT_DEBUGMSG(("EmbedControl failed: %d\n", err));
@@ -175,8 +181,11 @@ bool EV_MacToolbar::synthesize(void)
 	for (UT_uint32 k=0; (k < nrLabelItemsInLayout); k++)
 	{
         ControlHandle control;
+		ControlButtonContentInfo info;
+		PicHandle iconHandle;
         const char * szToolTip;
         Rect btnRect;
+		bool	bFoundIcon;
         
 		EV_Toolbar_LayoutItem * pLayoutItem = m_pToolbarLayout->getLayoutItem(k);
 		UT_ASSERT(pLayoutItem);
@@ -204,42 +213,57 @@ bool EV_MacToolbar::synthesize(void)
 
 			case EV_TBIT_PushButton:
                 // get pixmap
-
-				//				m_pMacToolbarIcons->getPixmapForIcon(wTLW->window,
-				//								  &wTLW->style->bg[GTK_STATE_NORMAL],
-				//								  pLabel->getIconName(),
-				//								  &wPixmap);
-				//UT_ASSERT(bFoundIcon);
-
-
+				bFoundIcon = m_pMacToolbarIcons->getBitmapForIcon(pLabel->getIconName(),
+												  &iconHandle);
+				UT_ASSERT(bFoundIcon);
                 // build control
-                btnRect.top = m_toolbarRect.top + 8;
+                btnRect.top = toolbarRect.top + BTN_SPACE;
                 btnRect.left = btnX;
-                btnRect.bottom = btnRect.top + 24;
-                btnRect.right = btnX + 24;
+                btnRect.bottom = btnRect.top + BTN_HEIGHT;
+                btnRect.right = btnX + BTN_WIDTH;
                 
-				UT_DEBUGMSG (("Toolbar: new push button at %d, %d, %d, %d\n", btnRect.top, btnRect.left, btnRect.bottom, btnRect.right));
-                control = ::NewControl (owningWin, &btnRect, "\p", true, 0, 0, 1, kControlBevelButtonSmallBevelProc, 0);
+				xxx_UT_DEBUGMSG (("Toolbar: new push button at %d, %d, %d, %d\n", btnRect.top, btnRect.left, btnRect.bottom, btnRect.right));
+                control = ::NewControl (owningWin, &btnRect, "\p", true, 0, 0, 1, kControlBevelButtonNormalBevelProc, 0);
 				UT_ASSERT (control);
-				err = ::EmbedControl (control, toolbarControl);
+				err = ::EmbedControl (control, m_hMainControl);
 				UT_ASSERT (err == noErr);
-                btnX += 32;
+				// set its icon
+				info.contentType = kControlContentPictHandle;
+				info.u.picture = iconHandle;
+				// find why the iconHandle is bogus and makes Carbon crashing
+				err = ::SetControlData (control, kControlEntireControl, kControlBevelButtonContentTag, 
+				                      sizeof (info), &info);
+				UT_ASSERT (err == noErr);
+				err = ::SetControlCommandID (control, id);
+				UT_ASSERT (err == noErr);
+				btnX += BTN_WIDTH + BTN_SPACE;
                 break;
 			case EV_TBIT_ToggleButton:
 			case EV_TBIT_GroupButton:
                 // get pixmap
-                
+				bFoundIcon = m_pMacToolbarIcons->getBitmapForIcon(pLabel->getIconName(),
+												  &iconHandle);
+				UT_ASSERT(bFoundIcon);
                 // build control
-                btnRect.top = m_toolbarRect.top + 8;
+                btnRect.top = toolbarRect.top + BTN_SPACE;
                 btnRect.left = btnX;
-                btnRect.bottom = btnRect.top + 24;
-                btnRect.right = btnX + 24;
+                btnRect.bottom = btnRect.top + BTN_HEIGHT;
+                btnRect.right = btnX + BTN_WIDTH;
                 
-				UT_DEBUGMSG (("Toolbar: new group button at %d, %d, %d, %d\n", btnRect.top, btnRect.left, btnRect.bottom, btnRect.right));
+				xxx_UT_DEBUGMSG (("Toolbar: new group button at %d, %d, %d, %d\n", btnRect.top, btnRect.left, btnRect.bottom, btnRect.right));
                 control = ::NewControl (owningWin, &btnRect, "\p", true, 0, 0, 1, kControlBevelButtonNormalBevelProc, 0);
-				err = ::EmbedControl (control, toolbarControl);
+				err = ::EmbedControl (control, m_hMainControl);
 				UT_ASSERT (err == noErr);
-                btnX += 32;
+               				// set its icon
+				info.contentType = kControlContentPictHandle;
+				info.u.picture = iconHandle;
+				// find why the iconHandle is bogus and makes Carbon crashing
+				err = SetControlData (control, kControlEntireControl, kControlBevelButtonContentTag, 
+				                      sizeof (info), &info);
+				UT_ASSERT (err == noErr);
+				err = ::SetControlCommandID (control, id);
+				UT_ASSERT (err == noErr);
+				btnX += BTN_WIDTH + BTN_SPACE;
                 break;
 			case EV_TBIT_EditText:
 				break;
@@ -261,9 +285,9 @@ bool EV_MacToolbar::synthesize(void)
                 //
                 // Really special as Combo Box does NOT exists in MacOS.
                 // Use popup menu instead. This will be the same as combo boxes are not editable in AbiWord
-                btnRect.top = m_toolbarRect.top + 8;
+                btnRect.top = toolbarRect.top + BTN_SPACE;
                 btnRect.left = btnX;
-                btnRect.bottom = btnRect.top + 24;
+                btnRect.bottom = btnRect.top + BTN_HEIGHT;
                 btnRect.right = btnX + iWidth;
                 
 				static short menuID = 10000;
@@ -275,9 +299,11 @@ bool EV_MacToolbar::synthesize(void)
 							kControlBehaviorCommandMenu + kControlContentTextOnly, 0,
 							kControlBevelButtonNormalBevelProc, 0);
                 // control = ::NewControl (owningWin, &btnRect, "\p", true, 0, 0, 1, kControlBevelButtonSmallBevelProc, 0);
-				err = ::EmbedControl (control, toolbarControl);
+				err = ::EmbedControl (control, m_hMainControl);
 				UT_ASSERT (err == noErr);
-                btnX += iWidth + 8;
+				err = ::SetControlCommandID (control, id);
+				UT_ASSERT (err == noErr);
+				btnX += iWidth + BTN_SPACE;
 				if (pControl)
 				{
 					pControl->populate();
@@ -305,8 +331,9 @@ bool EV_MacToolbar::synthesize(void)
 				break;
 					
 			case EV_TBIT_Spacer:
-				UT_DEBUGMSG (("Putting spacer\n"));
-				btnX += 8;
+				// TODO really put some visual spacer. Purely cosmetic.
+				xxx_UT_DEBUGMSG (("Putting spacer\n"));
+				btnX += BTN_SPACE * 2;
 				break;
 					
 			case EV_TBIT_BOGUS:
@@ -338,7 +365,8 @@ bool EV_MacToolbar::bindListenerToView(AV_View * pView)
 	bool bResult = pView->addListener(static_cast<AV_Listener *>(m_pViewListener),&m_lid);
 	UT_ASSERT(bResult);
 
-	refreshToolbar(pView, AV_CHG_ALL);
+	if (pView->isDocumentPresent())
+		refreshToolbar(pView, AV_CHG_ALL);
     return bResult;
 }
 
@@ -519,24 +547,3 @@ bool EV_MacToolbar::getToolTip(long lParam)
 	return true;
 }
 
-
-void EV_MacToolbar::_calcToolbarRect ()
-{
-    Rect rect;
-    GrafPtr	macWindowPort;
-#if TARGET_API_MAC_CARBON
-    macWindowPort = ::GetWindowPort (m_MacWindow);
-    ::GetPortBounds (macWindowPort, &rect);
-#else
-    /* don't do this in Carbon !! */
-    macWindowPort = (GrafPtr)m_MacWindow;
-    rect = macWindowPort->portRect;
-#endif
-    // Draw the window header where toolbar reside
-	short top = m_toolbarRect.top;
-    m_toolbarRect = rect;
-    ::InsetRect (&m_toolbarRect, -1, -1 );
-	// get the previous toolbar...
-	m_toolbarRect.top = top;
-    m_toolbarRect.bottom = m_toolbarRect.top + 40;
-}
