@@ -462,7 +462,8 @@ bool EV_QNXToolbar::bindListenerToView(AV_View * pView)
 	bResult = pView->addListener(static_cast<AV_Listener *>(m_pViewListener),&m_lid);
 	UT_ASSERT(bResult);
 
-	refreshToolbar(pView, AV_CHG_ALL);
+        if (pView->isDocumentPresent())
+            refreshToolbar(pView, AV_CHG_ALL);
 
 	return true;
 }
@@ -499,7 +500,7 @@ bool EV_QNXToolbar::refreshToolbar(AV_View * pView, AV_ChangeMask mask)
 		{
 		case EV_TLF_Normal:
 			{
-				const char * szState = 0;
+				const char * szState = NULL;
 				EV_Toolbar_ItemState tis = pAction->getToolbarItemState(pView,&szState);
 
 				switch (pAction->getItemType())
@@ -587,7 +588,7 @@ bool EV_QNXToolbar::refreshToolbar(AV_View * pView, AV_ChangeMask mask)
 					//printf("State [%s] \n", (szState) ? szState : "NULL");
 					if (szState && !(top = PtListItemPos(tcb->m_widget, szState))) {
 						//Assume this is the case of Times New Roman not being found
-						FontID *id = PfFindFont((const uchar_t *)szState, 0, 10);
+						FontID *id = PfFindFont((const char *)szState, 0, 10);
 						if (id) {
 							top = PtListItemPos(tcb->m_widget, (char *)PfFontDescription(id));
 							PfFreeFont(id);
@@ -606,6 +607,24 @@ bool EV_QNXToolbar::refreshToolbar(AV_View * pView, AV_ChangeMask mask)
 				}
 				break;
 
+
+				case EV_TBIT_ColorFore:
+				case EV_TBIT_ColorBack:
+				{
+					bool bGrayed = EV_TIS_ShouldBeGray(tis);
+
+					tcb = (struct _cb_data *) m_vecToolbarWidgets.getNthItem(k);
+					UT_ASSERT(tcb);
+					PtWidget_t *item;
+					item = tcb->m_widget;
+					UT_ASSERT(item);
+						
+					// Disable/enable toolbar item
+					PtSetArg(&args[n++], Pt_ARG_FLAGS, 
+							 (bGrayed) ? (Pt_BLOCKED | Pt_GHOST) : Pt_SELECTABLE, 
+							 Pt_BLOCKED | Pt_GHOST | Pt_SELECTABLE);
+					PtSetResources(tcb->m_widget, n, args);
+				}break;
 				case EV_TBIT_StaticLabel:
 					break;
 				case EV_TBIT_Spacer:
