@@ -29,17 +29,32 @@
 #include "gr_UnixImage.h"
 #include "ut_sleep.h"
 #include "xap_UnixFrame.h"
+#include "xap_Strings.h"
 
 #ifdef HAVE_GNOME
-#include <gnome.h>
-#include <gdk-pixbuf/gdk-pixbuf.h>
 #include "gr_UnixGnomeImage.h"
 #endif
 
+#include "ut_debugmsg.h"
+#include "ut_assert.h"
+#include "ut_misc.h"
+#include "ut_string.h"
+#include "xap_UnixDialogHelper.h"
+#include "ut_wctomb.h"
+#include "xap_EncodingManager.h"
+#include "ut_OverstrikingChars.h"
+
+
 #if 1
 #include <gdk/gdkprivate.h>
 static bool isFontUnicode(GdkFont *font)
 {
+	if(!font)
+	{
+		UT_DEBUGMSG(("gr_UnixGraphics: isFontUnicode: font is NULL !!!\n"));
+		return false;
+	}
+	
 	GdkFontPrivate *font_private = (GdkFontPrivate*) font;
 	XFontStruct *xfont = (XFontStruct *) font_private->xfont;
 	
@@ -47,19 +62,12 @@ static bool isFontUnicode(GdkFont *font)
 }
 #endif
 
-#include "ut_debugmsg.h"
-#include "ut_assert.h"
-#include "ut_misc.h"
-#include "ut_string.h"
-#include "ut_dialogHelper.h"
-#include "ut_wctomb.h"
-#include "xap_EncodingManager.h"
-#include "ut_OverstrikingChars.h"
 
 //
 // Below this size we use GDK fonts. Above it we use metric info.
 //
 #define MAX_ABI_GDK_FONT_SIZE 200
+#define FALLBACK_FONT_SIZE 12
 
 XAP_UnixFontHandle *	GR_UnixGraphics::s_pFontGUI = NULL;
 UT_uint32 				GR_UnixGraphics::s_iInstanceCount = 0;
@@ -108,6 +116,8 @@ GR_UnixGraphics::GR_UnixGraphics(GdkWindow * win, XAP_UnixFontManager * fontMana
 	m_cs = GR_Graphics::GR_COLORSPACE_COLOR;
 	m_cursor = GR_CURSOR_INVALID;
 	setCursor(GR_CURSOR_DEFAULT);
+	
+	m_pFallBackFontHandle = new XAP_UnixFontHandle(m_pFontManager->getDefaultFont(), FALLBACK_FONT_SIZE);
 }
 
 GR_UnixGraphics::~GR_UnixGraphics()
@@ -115,6 +125,8 @@ GR_UnixGraphics::~GR_UnixGraphics()
 	s_iInstanceCount--;
 	if(!s_iInstanceCount)
 		DELETEP(s_pFontGUI);
+	
+	delete m_pFallBackFontHandle;
 }
 
 bool GR_UnixGraphics::queryProperties(GR_Graphics::Properties gp) const
@@ -122,6 +134,7 @@ bool GR_UnixGraphics::queryProperties(GR_Graphics::Properties gp) const
 	switch (gp)
 	{
 	case DGP_SCREEN:
+	case DGP_OPAQUEOVERLAY:
 		return true;
 	case DGP_PAPER:
 		return false;
@@ -162,6 +175,9 @@ static bool fallback_used;
 void GR_UnixGraphics::drawChar(UT_UCSChar Char, UT_sint32 xoff, UT_sint32 yoff)
 {
 	UT_UCSChar Wide_char = remapGlyph(Char, false);
+	if(Wide_char == 0x200B || Wide_char == 0xFEFF) //zero width spaces
+		return;
+		
 	GdkFont *font = XAP_EncodingManager::get_instance()->is_cjk_letter(Wide_char) ? m_pMultiByteFont : m_pSingleByteFont;
 
 	if(XAP_EncodingManager::get_instance()->isUnicodeLocale())
@@ -200,6 +216,8 @@ void GR_UnixGraphics::drawChars(const UT_UCSChar* pChars, int iCharOffset,
 	WCTOMB_DECLS;
 	GdkFont *font;
 	UT_sint32 x;
+	
+	static bool bFontSizeWarning = true;
 
 #ifdef BIDI_ENABLED	
 	// to be able to handle overstriking characters, we have to remember the width
@@ -215,7 +233,46 @@ void GR_UnixGraphics::drawChars(const UT_UCSChar* pChars, int iCharOffset,
   	for(pC=pChars+iCharOffset, x=xoff; pC<pChars+iCharOffset+iLength; ++pC)
 	{
 		UT_UCSChar actual = remapGlyph(*pC,false);
+		if(actual == 0x200B || actual == 0xFEFF) //zero width spaces
+			continue;
+			
 		font=XAP_EncodingManager::get_instance()->is_cjk_letter(actual)? m_pMultiByteFont: m_pSingleByteFont;
+
+		if(!font)
+		{
+			// what now? this happens for instance when you set font size to 72
+			// and zoom to 200; obviously gtk cannot create font that big
+			// we do get the width right though, since we use the afm file for
+			// such big sizes, so we should just use the default font here,
+			// this will look really weird, but it is better then not drawing anything
+			// and much better than crashing
+			if(bFontSizeWarning)
+			{
+				XAP_App * pApp = XAP_App::getApp();
+				UT_ASSERT(pApp);
+				const XML_Char * msg = pApp->getStringSet()->getValue(XAP_STRING_ID_MSG_UnixFontSizeWarning);
+				UT_ASSERT(msg);
+				bFontSizeWarning = false;
+				messageBoxOK(msg);
+			}
+			
+			UT_DEBUGMSG(("gr_UnixGraphics::drawChars: no font to draw with, using default !!!\n"));
+			UT_sint32 iSize = m_pFallBackFontHandle ? m_pFallBackFontHandle->getSize() : 0;
+			UT_sint32 iMySize = FALLBACK_FONT_SIZE * getZoomPercentage() / 100;
+				
+			if(iSize != iMySize)
+			{
+				delete m_pFallBackFontHandle;
+				m_pFallBackFontHandle = new XAP_UnixFontHandle(m_pFontManager->getDefaultFont(),iMySize);
+			}
+			
+			setFont(m_pFallBackFontHandle);
+			font=XAP_EncodingManager::get_instance()->is_cjk_letter(actual)? m_pMultiByteFont: m_pSingleByteFont;
+			
+			UT_ASSERT(font);
+			if(!font)
+				return;
+		}
 		
 		if(XAP_EncodingManager::get_instance()->isUnicodeLocale())
 		{
@@ -338,6 +395,10 @@ UT_uint32 GR_UnixGraphics::measureUnRemappedChar(const UT_UCSChar c)
 	// but its not (for presumed performance reasons).  Also, a difference
 	// is that measureString() uses remapping to get past zero-width
 	// character cells.
+	
+	if(c == 0x200B || c == 0xFEFF) // 0-with spaces
+		return 0;
+		
 	if (!m_pFontManager)
 		return 0;
 
@@ -443,6 +504,9 @@ UT_uint32 GR_UnixGraphics::measureString(const UT_UCSChar* s, int iOffset,
 	for (int i = 0; i < num; i++)
     {
 		cChar = remapGlyph(s[i + iOffset], true);
+		if(cChar == 0x200B || cChar == 0xFEFF)
+			continue;
+			
 		width = gdk_char_width_wc (pFont, cChar);
 		charWidth += width;
 		if (pWidths)
@@ -554,9 +618,30 @@ GR_Font * GR_UnixGraphics::findFont(const char* pszFontFamily,
 	XAP_UnixFont * unixfont = m_pFontManager->getFont(pszFontFamily, s);	
 	if (!unixfont)
 	{
-		// Oops!  We don't have that font here.  substitute something
-		// we know we have (get smarter about this later)
+		// Oops!  We don't have that font here.
+		// first try "Times New Roman", which should be sensible, and should
+		// be there unless the user fidled with the installation
 		unixfont = m_pFontManager->getFont("Times New Roman", s);
+		
+		// Oh well, see if there are any fonts at all, and if so
+		// just take the first one ...
+		if(!unixfont)
+		{
+				UT_Vector *	pVec = m_pFontManager->getAllFonts();
+				if(pVec && pVec->getItemCount() > 0)
+				{
+					// get the first font we have
+					unixfont = static_cast<XAP_UnixFont *>(pVec->getNthItem(0));
+				}
+
+				// free the returned vector
+				DELETEP(pVec);
+		
+		}
+			
+		// this is really desperate, we do not seem to have any fonts
+		// we cannot be blamed if we just give up
+		
 		if (!unixfont)
 		{
 			char message[1024];
@@ -568,6 +653,7 @@ GR_Font * GR_UnixGraphics::findFont(const char* pszFontFamily,
 					   "\n"
 					   "AbiWord cannot continue without this font.", pszFontFamily);
 			messageBoxOK(message);
+			
 			exit(1);
 		}
 		
@@ -599,6 +685,16 @@ UT_uint32 GR_UnixGraphics::getFontAscent(GR_Font * fnt)
 	{
 		GdkFont* pFont = hndl->getGdkFont();
 		GdkFont* pMatchFont=hndl->getMatchGdkFont();
+
+		// with the incremental loader in place, this happens to be
+		// the first place which tries to load fonts (because we need
+		// to compute font ascents).  If they're not present, these
+		// GdkFont guys will be NULL, so we would segfault.  Let's
+		// quietly abort instead.
+
+		if (!pFont || !pMatchFont)
+			abort();
+
 		return MAX(pFont->ascent, pMatchFont->ascent);
 	}
 //
@@ -609,6 +705,11 @@ UT_uint32 GR_UnixGraphics::getFontAscent(GR_Font * fnt)
 		XAP_UnixFont * pSingleByte = NULL;
 		XAP_UnixFont * pMultiByte = NULL;
 		hndl->explodeUnixFonts(&pSingleByte,&pMultiByte);
+
+		// Some more crash protection.  Why not?
+		if (!pSingleByte || !pSingleByte->getMetricsData())
+			abort();
+
 		GlobalFontInfo * gfsi = pSingleByte->getMetricsData()->gfi;
 		UT_ASSERT(gfsi);
 		UT_uint32 ascsingle = (UT_uint32) ( (double) gfsi->fontBBox.ury * (double) hndl->getSize() /1000.);
@@ -619,6 +720,7 @@ UT_uint32 GR_UnixGraphics::getFontAscent(GR_Font * fnt)
 
 UT_uint32 GR_UnixGraphics::getFontAscent()
 {
+
 	return getFontAscent(m_pFont);
 }
 
@@ -1144,6 +1246,14 @@ void GR_UnixGraphics::setCursor(GR_Graphics::Cursor c)
 	case GR_CURSOR_GRAB:
 		cursor_number = GDK_HAND1;
 		break;
+
+	case GR_CURSOR_LINK:
+		cursor_number = GDK_HAND2;
+		break;
+
+	case GR_CURSOR_WAIT:
+		cursor_number = GDK_WATCH;
+		break;
 	}
 
 	GdkCursor * cursor = gdk_cursor_new(cursor_number);
