@@ -27,6 +27,9 @@
 
 #include "gr_Win32Graphics.h"
 #include "gr_Win32Image.h"
+#include <xap_Win32App.h>
+
+#include <xap_Win32Res_Cursors.rc2>
 
 #include "ut_debugmsg.h"
 #include "ut_assert.h"
@@ -34,6 +37,7 @@
 #include "ut_Win32OS.h"
 
 /*****************************************************************/
+UT_sint32 GR_Win32Graphics::s_iScreenResolution = 0;
 
 // A small helper class
 class private_FontReverter
@@ -90,6 +94,16 @@ void GR_Win32Graphics::_constructorCommonCode(HDC hdc)
 
 	m_remapBuffer = NULL;
 	m_remapBufferSize = 0;
+#ifdef BIDI_ENABLED
+	m_remapIndices = NULL;
+#endif
+	if(!s_iScreenResolution)
+	{
+		s_iScreenResolution = _getResolution();
+		setStaticScreenResolution(s_iScreenResolution);
+	}
+	
+	UT_DEBUGMSG(("GR_Win32Graphics: screen resolution %d\n", s_iScreenResolution));
 }
 
 GR_Win32Graphics::GR_Win32Graphics(HDC hdc, HWND hwnd, XAP_App * app)
@@ -115,6 +129,9 @@ GR_Win32Graphics::~GR_Win32Graphics()
 	}
 
 	delete [] m_remapBuffer;
+#ifdef BIDI_ENABLED
+	delete [] m_remapIndices;
+#endif
 }
 
 bool GR_Win32Graphics::queryProperties(GR_Graphics::Properties gp) const
@@ -125,6 +142,8 @@ bool GR_Win32Graphics::queryProperties(GR_Graphics::Properties gp) const
 		return !m_bPrint;
 	case DGP_PAPER:
 		return m_bPrint;
+	case DGP_OPAQUEOVERLAY:
+		return true;
 	default:
 		UT_ASSERT(0);
 		return false;
@@ -244,6 +263,8 @@ void GR_Win32Graphics::drawChar(UT_UCSChar Char, UT_sint32 xoff, UT_sint32 yoff)
 
 	UT_UCSChar currentChar = remapGlyph(Char, false);
 
+	if(currentChar == 0x200B || currentChar == 0xFEFF)
+		return;
 	// Windows NT and Windows 95 support the Unicode Font file. 
 	// All of the Unicode glyphs can be rendered if the glyph is found in
 	// the font file. However, Windows 95 does  not support the Unicode 
@@ -266,16 +287,17 @@ void GR_Win32Graphics::drawChar(UT_UCSChar Char, UT_sint32 xoff, UT_sint32 yoff)
 	else
 	{
 		// Unicode font and default character set handling for WinNT and Win9x
-		ExtTextOutW(m_hdc, xoff, yoff, 0, NULL, &currentChar, 1, NULL);
+		ExtTextOutW(m_hdc, xoff, yoff, 0/*ETO_GLYPH_INDEX*/, NULL, &currentChar, 1, NULL);
 	}
 }
 
 void GR_Win32Graphics::drawChars(const UT_UCSChar* pChars,
-								 int iCharOffset, int iLength,
+								 int iCharOffset, int iLengthOrig,
 								 UT_sint32 xoff, UT_sint32 yoff)
 {
 	UT_ASSERT(pChars);
-
+	// iLength can be modified by _remapGlyphs
+	int iLength = iLengthOrig;
 	HFONT hFont = GR_Win32Font::Acq::getHFONT(*m_pFont);
 	SelectObject(m_hdc, hFont);
 	SetTextAlign(m_hdc, TA_LEFT | TA_TOP);
@@ -293,6 +315,7 @@ void GR_Win32Graphics::drawChars(const UT_UCSChar* pChars,
 	LOGFONT lf;
 	int iRes = GetObject(hFont, sizeof(LOGFONT), &lf);
 	UT_ASSERT(iRes);
+
 	if (UT_IsWinNT() == false && lf.lfCharSet == SYMBOL_CHARSET)
 	{
 		// Symbol character handling for Win9x
@@ -306,25 +329,69 @@ void GR_Win32Graphics::drawChars(const UT_UCSChar* pChars,
 	else
 	{
 		// Unicode font and default character set handling for WinNT and Win9x
-		ExtTextOutW(m_hdc, xoff, yoff, 0, NULL, currentChars, iLength, NULL);
+#ifdef BIDI_ENABLED
+		if(XAP_App::getApp()->theOSHasBidiSupport())
+		{
+			UT_ASSERT(m_remapIndices);
+			GCP_RESULTSW gcpResult;
+			gcpResult.lStructSize = sizeof(GCP_RESULTS);
+			gcpResult.lpOutString = NULL;			// Output string
+			gcpResult.lpOrder = NULL;				// Ordering indices
+			gcpResult.lpDx = NULL;					// Distances between character cells
+			gcpResult.lpCaretPos = NULL;			// Caret positions
+			gcpResult.lpClass = NULL;				// Character classifications
+			gcpResult.lpGlyphs = m_remapIndices;	// Character glyphs
+			gcpResult.nGlyphs = m_remapBufferSize;  // Array size
+ 
+			if(GetCharacterPlacementW(m_hdc, currentChars, iLength, 0, &gcpResult, GCP_REORDER))
+			{
+				ExtTextOutW(m_hdc, xoff, yoff, ETO_GLYPH_INDEX, NULL, m_remapIndices, gcpResult.nGlyphs, NULL);
+			}
+			else
+			{
+				UT_ASSERT(UT_SHOULD_NOT_HAPPEN);
+				goto simple_exttextout;
+			}
+		}
+		else
+#endif
+		{
+#ifdef BIDI_ENABLED
+simple_exttextout:
+#endif
+			ExtTextOutW(m_hdc, xoff, yoff, 0, NULL, currentChars, iLength, NULL);
+		}
 	}
 
 }
 
-UT_UCSChar*	GR_Win32Graphics::_remapGlyphs(const UT_UCSChar* pChars, int iCharOffset, int iLength)
+UT_UCSChar*	GR_Win32Graphics::_remapGlyphs(const UT_UCSChar* pChars, int iCharOffset, int &iLength)
 {
 
 	if (iLength > (int)m_remapBufferSize)
 	{
 		delete [] m_remapBuffer;
+#ifdef BIDI_ENABLED
+		if(XAP_App::getApp()->theOSHasBidiSupport())
+		{
+			delete [] m_remapIndices;
+			m_remapIndices = new UT_UCSChar[iLength];
+		}
+#endif
 		m_remapBuffer = new UT_UCSChar[iLength];
 		m_remapBufferSize = iLength;
 	}
 
-	for (int i = 0; i < iLength; ++i)
+    // Need to handle zero-width spaces correctly
+	int i, j;
+	for (i = 0, j = 0; i < iLength; ++i, ++j)
 	{
-		m_remapBuffer[i] = remapGlyph(pChars[iCharOffset + i], false);
+		m_remapBuffer[j] = remapGlyph(pChars[iCharOffset + i], false);
+		if(m_remapBuffer[j] == 0x200B || m_remapBuffer[j] == 0xFEFF)
+			j--;
 	}
+ 
+	iLength -= (i - j);
 
 	return m_remapBuffer;
 }
@@ -523,7 +590,7 @@ void GR_Win32Graphics::polyLine(UT_Point * pts, UT_uint32 nPoints)
 	HPEN hPen = CreatePen(PS_SOLID, 1, m_clrCurrent);
 	HPEN hOldPen = (HPEN) SelectObject(m_hdc, hPen);
 
-	POINT * points = (POINT *)calloc(nPoints, sizeof(POINT));
+	POINT * points = (POINT *)UT_calloc(nPoints, sizeof(POINT));
 	UT_ASSERT(points);
 
 	for (UT_uint32 i = 0; i < nPoints; i++)
@@ -585,6 +652,14 @@ bool GR_Win32Graphics::startPage(const char * szPageLabel, UT_uint32 pageNumber,
 		EndPage(m_hdc);
 	}
 
+	// Correct for Portrait vs Lanscape mode
+	DEVMODE pDevMode;
+	pDevMode.dmSize = sizeof(DEVMODE);
+	pDevMode.dmDriverExtra = 0;
+	pDevMode.dmFields = DM_ORIENTATION;
+	pDevMode.dmOrientation = (bPortrait) ? DMORIENT_PORTRAIT : DMORIENT_LANDSCAPE;
+	ResetDC( m_hdc, &pDevMode );
+
 	const int iRet = StartPage(m_hdc);
 
 	m_bStartPage = iRet > 0;
@@ -641,6 +716,7 @@ void GR_Win32Graphics::scroll(UT_sint32 x_dest, UT_sint32 y_dest,
 
 void GR_Win32Graphics::clearArea(UT_sint32 x, UT_sint32 y, UT_sint32 width, UT_sint32 height)
 {
+
 //	UT_ASSERT((x + width) < 800);
 	
 	HBRUSH hBrush = (HBRUSH) GetStockObject(WHITE_BRUSH);
@@ -667,12 +743,10 @@ void GR_Win32Graphics::invertRect(const UT_Rect* pRect)
 
 void GR_Win32Graphics::setClipRect(const UT_Rect* pRect)
 {
-	if (queryProperties(GR_Graphics::DGP_PAPER))
-	{
-		// FIXME: Currently it seems the cliprect doesn't work
-		// when printing images, clipping out large portions of them.
-		return;
-	}
+	// This causes a lot of drawing and screen refresh problems.  
+	// It can be removed from here without problems, but may
+	// not work for other things.  For now leave code in place.
+	// return;
 
 	int res;
 	m_pRect = pRect;
@@ -723,6 +797,14 @@ void GR_Win32Graphics::drawImage(GR_Image* pImg, UT_sint32 xDest, UT_sint32 yDes
 		return;
 	}
 
+	// When Printing yDest must be within Page Height for each
+	// Page.   
+	if (queryProperties(GR_Graphics::DGP_PAPER))
+	{
+		UT_sint32 nPageHeight = GetDeviceCaps(m_hdc, PHYSICALHEIGHT);
+		yDest = ( yDest % nPageHeight );
+	}
+
 	GR_Win32Image* pWin32Img = static_cast<GR_Win32Image*>(pImg);
 
 	BITMAPINFO* pDIB = pWin32Img->getDIB();
@@ -776,6 +858,8 @@ void GR_Win32Graphics::handleSetCursorMessage(void)
 {
 	// deal with WM_SETCURSOR message.
 
+	XAP_Win32App * pWin32App = static_cast<XAP_Win32App *>(m_pApp);
+	HINSTANCE hinst = pWin32App->getInstance();
 	LPCTSTR cursor_name;
 	
 	switch (m_cursor)
@@ -785,50 +869,78 @@ void GR_Win32Graphics::handleSetCursorMessage(void)
 		/*FALLTHRU*/
 	case GR_CURSOR_DEFAULT:
 		cursor_name = IDC_ARROW;		// top-left arrow
+		hinst = NULL;
 		break;
-		
+	
+	case GR_CURSOR_LINK:	
+	case GR_CURSOR_GRAB:
+		cursor_name = MAKEINTRESOURCE(IDC_ABIHAND);
+		break;
+
+	case GR_CURSOR_EXCHANGE:
+		cursor_name = MAKEINTRESOURCE(IDC_EXCHANGE);
+		break;
+
 	case GR_CURSOR_IBEAM:
 		cursor_name = IDC_IBEAM;
+		hinst = NULL;
 		break;
 
 	case GR_CURSOR_RIGHTARROW:
 		cursor_name = IDC_ARROW;		// TODO change this
+		hinst = NULL;
 		break;
 		
 #ifdef BIDI_ENABLED
 //#error choose a suitable cursor; this is just a placeholder !!!		
 	case GR_CURSOR_LEFTARROW:
 		cursor_name = IDC_ARROW;		// TODO change this
+		hinst = NULL;
 		break;
 #endif
 
 	case GR_CURSOR_IMAGE:
 		cursor_name = IDC_SIZEALL;
+		hinst = NULL;
 		break;
 		
 	case GR_CURSOR_IMAGESIZE_NW:
 	case GR_CURSOR_IMAGESIZE_SE:
 		cursor_name = IDC_SIZENWSE;
+		hinst = NULL;
 		break;
 		
+	case GR_CURSOR_UPDOWN:
 	case GR_CURSOR_IMAGESIZE_N:
 	case GR_CURSOR_IMAGESIZE_S:
 		cursor_name = IDC_SIZENS;
+		hinst = NULL;
 		break;
 		
 	case GR_CURSOR_IMAGESIZE_NE:
 	case GR_CURSOR_IMAGESIZE_SW:
 		cursor_name = IDC_SIZENESW;
+		hinst = NULL;
 		break;
 		
+	case GR_CURSOR_LEFTRIGHT:
 	case GR_CURSOR_IMAGESIZE_E:
 	case GR_CURSOR_IMAGESIZE_W:
 		cursor_name = IDC_SIZEWE;
+		hinst = NULL;
+		break;
+
+	case GR_CURSOR_WAIT:
+		cursor_name = IDC_WAIT;
+		hinst = NULL;
 		break;
 	}
 
-	HCURSOR hCursor = LoadCursor(NULL,cursor_name);
-	SetCursor(hCursor);
+	HCURSOR hCursor = LoadCursor(hinst,cursor_name);
+	if (hCursor != NULL)
+	{
+		SetCursor(hCursor);
+	}
 }
 
 void GR_Win32Graphics::setColor3D(GR_Color3D c)
