@@ -28,7 +28,7 @@
 
 #include "ut_debugmsg.h"
 
-void _UT_OutputMessage(char *s, ...)
+void _UT_OutputMessage(const char *s, ...)
 {
 	va_list marker;
 
