@@ -20,7 +20,7 @@
 /*
 	Platform: MacOS Classic & Carbon
 	
-	$Id: xap_MacApp.cpp,v 1.19 2001/08/30 23:54:33 sam Exp $	
+	$Id: xap_MacApp.cpp,v 1.21 2001/11/05 22:37:29 hub Exp $	
 */
 
 #include <string.h>
@@ -53,6 +53,8 @@
 #include "xap_MacFontManager.h"
 #include "xap_MacTlbr_Icons.h"
 #include "xap_MacTlbr_ControlFactory.h"
+#include "xap_MacToolbar_Control.h"
+
 #include "ev_MacMenu.h"
 
 
@@ -70,16 +72,20 @@ XAP_MacApp::XAP_MacApp(XAP_Args * pArgs, const char * szAppName)
 	m_finished = false;
 	m_pMacToolbarIcons = 0;
 	m_pMacFontManager = NULL;
+	m_pToolbarControl = new XAP_MacToolbar_Control ();
 }
 
 XAP_MacApp::~XAP_MacApp(void)
 {
 	DELETEP(m_pMacFontManager);
 	DELETEP(m_pMacToolbarIcons);
+	DELETEP(m_pToolbarControl);
 }
 
 bool XAP_MacApp::initialize(void)
 {
+	OSStatus err;
+
 	// let our base class do it's thing.
 	
 	XAP_App::initialize();
@@ -89,12 +95,23 @@ bool XAP_MacApp::initialize(void)
 
 	// load only one copy of the platform-specific icons.
 
-//	m_pMacToolbarIcons = new AP_MacToolbar_Icons();
+	m_pMacToolbarIcons = new AP_MacToolbar_Icons();
 	
 	// do anything else we need here...
 
 	// _pClipboard = new AP_MacClipboard();
-	
+
+#if defined(USE_CARBON_EVENTS)
+	EventTypeSpec eventTypes[1];
+	eventTypes[0].eventClass = kEventClassCommand;
+	eventTypes[0].eventKind = kEventCommandProcess;
+	EventHandlerRef handlerRef;
+	EventHandlerUPP handlerUPP = NewEventHandlerUPP (HandleCarbonMenus);
+	err = ::InstallApplicationEventHandler (handlerUPP, 1, eventTypes, (void*)this, &handlerRef);
+#endif
+
+	m_pToolbarControl->show ();
+
 	return true;
 }
 
@@ -122,7 +139,8 @@ XAP_Toolbar_ControlFactory * XAP_MacApp::getControlFactory(void)
 	return &m_controlFactory;
 }
 
-const char * XAP_MacApp::getUserPrivateDirectory(void) {
+const char * XAP_MacApp::getUserPrivateDirectory(void) 
+{
         /* return a pointer to a static buffer */
 	short vRefNum;
 	long dirID;
@@ -212,7 +230,45 @@ void XAP_MacApp::InitializeMacToolbox ()
 	}
 }
 
+#if defined(USE_CARBON_EVENTS)
+/*!
+	Carbon event handler
+ */
+pascal OSStatus XAP_MacApp::HandleCarbonMenus (EventHandlerCallRef nextHandler, 
+											EventRef theEvent, void* userData)
+{
+	OSStatus err = noErr;
+	XAP_MacApp *	myApp = (XAP_MacApp *)userData;
+	UT_ASSERT (myApp);
+	UInt32			eventKind;
+	HICommand		cmd;
+	
+	eventKind = ::GetEventKind (theEvent);
+	if (eventKind == kEventCommandProcess) {
+		XAP_Menu_Id xapId;
+		GetEventParameter (theEvent, kEventParamDirectObject, typeHICommand, NULL, sizeof (HICommand), NULL, &cmd);
 
+		xapId = cmd.commandID;
+		
+		XAP_MacFrame *theFrame = dynamic_cast<XAP_MacFrame *>(myApp->getLastFocussedFrame ());
+		UT_ASSERT (theFrame);
+		
+		EV_MacMenu *menu = theFrame->getMenu();
+		UT_ASSERT (menu);
+
+		menu->onCommand (xapId);
+	}
+	
+	err = CallNextEventHandler (nextHandler, theEvent);
+	return err;
+}
+
+
+#else
+/*!
+	Dispatch the event.
+	Classic events only 
+*/
 void XAP_MacApp::DispatchEvent (const EventRecord & theEvent)
 {
 	WindowPtr	targetWin;
@@ -322,7 +378,7 @@ void XAP_MacApp::DispatchEvent (const EventRecord & theEvent)
 	} 
 }
 
-
+/* Classic events only */
 void XAP_MacApp::HandleMenus (long menuSelection)
 {
 	short id, item;
@@ -342,7 +398,7 @@ void XAP_MacApp::HandleMenus (long menuSelection)
 	}	
 	HiliteMenu(0);
 }
-
+#endif
 
 
 /*
@@ -353,10 +409,14 @@ void XAP_MacApp::run ()
 	unsigned short mask = everyEvent;
 	EventRecord theEvent;
 	unsigned long delay = ::GetCaretTime();
+#if defined(USE_CARBON_EVENTS)
+	RunApplicationEventLoop ();
+#else
 	while (!m_finished) {
 		while (::WaitNextEvent(mask, &theEvent, delay, NULL))
 		{
 			DispatchEvent (theEvent);
 		}
 	}
+#endif
 }
