@@ -27,7 +27,7 @@
 #include <unistd.h>
 #include "ut_string.h"
 #include "ut_assert.h"
-#include "ut_dialogHelper.h"
+#include "xap_UnixDialogHelper.h"
 #include "xap_Dialog_Id.h"
 #include "xap_Dlg_MessageBox.h"
 #include "xap_UnixDlg_FileOpenSaveAs.h"
@@ -41,6 +41,8 @@
 #include "ut_svg.h"
 #include "ut_misc.h"
 #include "gr_UnixGraphics.h"
+#include "fg_Graphic.h"
+#include "fg_GraphicRaster.h"
 
 #if defined(HAVE_GNOME)
 #include "gr_UnixGnomeImage.h"
@@ -109,6 +111,20 @@ static gint s_preview_exposed(GtkWidget * /* widget */,
 	return FALSE;
 }
 
+static gint
+fsel_key_event (GtkWidget *widget, GdkEventKey *event, XAP_Dialog_FileOpenSaveAs::tAnswer * answer)
+{
+#ifdef GDK_Escape
+	if (event->keyval == GDK_Escape) {
+		gtk_signal_emit_stop_by_name (GTK_OBJECT (widget), "key_press_event");
+		s_cancel_clicked ( widget, answer ) ;
+		return TRUE;
+	}
+#endif
+
+	return FALSE;
+}
+
 static gint s_filename_select (GtkCList * w,
 			       gint row, gint col, GdkEvent * evt,
 			       gpointer ptr)
@@ -176,7 +192,7 @@ bool XAP_UnixDialog_FileOpenSaveAs::_run_gtk_main(XAP_Frame * pFrame,
 			UT_cloneString(szDialogFilename, gtk_file_selection_get_filename(pFS));
 			UT_ASSERT(szDialogFilename);
 
-			err = stat(szFinalPathname, &buf);
+			err = stat(szDialogFilename, &buf);
 			UT_ASSERT(err == 0 || err == -1);
 			
 			// Check for a directory entered as filename.  When true,
@@ -254,6 +270,11 @@ bool XAP_UnixDialog_FileOpenSaveAs::_run_gtk_main(XAP_Frame * pFrame,
 
 			UT_DEBUGMSG(("UseSuffix: %d\n", wantSuffix));
 
+			// do not want suffix for directory names
+			err = stat(szDialogFilename, &buf);
+			if (S_ISDIR(buf.st_mode))
+				wantSuffix = false;
+
 			// if the file doesn't have a suffix already, and the file type
 			// is normal (special types are negative, like auto detect),
 			// and the user wants extensions, slap a suffix on it.   
@@ -265,7 +286,7 @@ bool XAP_UnixDialog_FileOpenSaveAs::_run_gtk_main(XAP_Frame * pFrame,
 					UT_ASSERT(szSuffix);                            
 					UT_uint32 length = strlen(szDialogFilename) + strlen(szSuffix) + 1;
 					
-					szFinalPathname = (char *)calloc(length,sizeof(char));
+					szFinalPathname = (char *)UT_calloc(length,sizeof(char));
 					
 					if (szFinalPathname)                            						
 						{                                               
@@ -449,13 +470,6 @@ void XAP_UnixDialog_FileOpenSaveAs::runModal(XAP_Frame * pFrame)
 		bCheckWritePermission = false;
 		break;
 	}
-	case XAP_DIALOG_ID_FILE_SAVEAS:
-	{
-		szTitle = pSS->getValue(XAP_STRING_ID_DLG_FOSA_SaveAsTitle);
-		szFileTypeLabel = pSS->getValue(XAP_STRING_ID_DLG_FOSA_FileSaveTypeLabel);
-		bCheckWritePermission = true;
-		break;
-	}
 	case XAP_DIALOG_ID_FILE_IMPORT:
 	  {
 		szTitle = pSS->getValue(XAP_STRING_ID_DLG_FOSA_ImportTitle);
@@ -463,6 +477,20 @@ void XAP_UnixDialog_FileOpenSaveAs::runModal(XAP_Frame * pFrame)
 		bCheckWritePermission = false;
 	    break;
 	  }
+	case XAP_DIALOG_ID_INSERT_FILE:
+	  {
+		szTitle = pSS->getValue(XAP_STRING_ID_DLG_FOSA_InsertTitle);
+		szFileTypeLabel = pSS->getValue(XAP_STRING_ID_DLG_FOSA_FileOpenTypeLabel);
+		bCheckWritePermission = false;
+		break;
+	  }
+	case XAP_DIALOG_ID_FILE_SAVEAS:
+	{
+		szTitle = pSS->getValue(XAP_STRING_ID_DLG_FOSA_SaveAsTitle);
+		szFileTypeLabel = pSS->getValue(XAP_STRING_ID_DLG_FOSA_FileSaveTypeLabel);
+		bCheckWritePermission = true;
+		break;
+	}
 	case XAP_DIALOG_ID_FILE_EXPORT:
 	  {
 		szTitle = pSS->getValue(XAP_STRING_ID_DLG_FOSA_ExportTitle);
@@ -491,7 +519,10 @@ void XAP_UnixDialog_FileOpenSaveAs::runModal(XAP_Frame * pFrame)
 	m_FS = pFS;
 	
 	connectFocus(GTK_WIDGET(pFS),pFrame);
-
+	gtk_window_set_modal (GTK_WINDOW (pFS), TRUE);
+	gtk_widget_show_all (GTK_WIDGET (pFS));
+	gtk_grab_add (GTK_WIDGET (pFS));
+ 
 	GtkWidget * filetypes_pulldown = NULL;
 	
 	/*
@@ -632,12 +663,15 @@ void XAP_UnixDialog_FileOpenSaveAs::runModal(XAP_Frame * pFrame)
 							 GTK_SIGNAL_FUNC(s_delete_clicked),
 							 &m_answer);
 
+	gtk_signal_connect (GTK_OBJECT (pFS),
+			    "key_press_event",
+			    GTK_SIGNAL_FUNC(fsel_key_event), &m_answer);
 	gtk_signal_connect(GTK_OBJECT(pFS->ok_button), "clicked",
 					   GTK_SIGNAL_FUNC(s_ok_clicked), &m_answer);
 	gtk_signal_connect(GTK_OBJECT(pFS->cancel_button), "clicked",
 					   GTK_SIGNAL_FUNC(s_cancel_clicked), &m_answer);
 
-	if (m_id == XAP_DIALOG_ID_FILE_OPEN || m_id == XAP_DIALOG_ID_INSERT_PICTURE) // only hide the buttons if we're opening a file/picture
+	if (m_id == XAP_DIALOG_ID_FILE_OPEN || m_id == XAP_DIALOG_ID_INSERT_PICTURE || m_id == XAP_DIALOG_ID_FILE_EXPORT || m_id == XAP_DIALOG_ID_INSERT_FILE) // only hide the buttons if we're opening a file/picture
 	  gtk_file_selection_hide_fileop_buttons(pFS);
 
 	// use the persistence info and/or the suggested filename
@@ -702,8 +736,9 @@ void XAP_UnixDialog_FileOpenSaveAs::runModal(XAP_Frame * pFrame)
 	UT_ASSERT(frame);
 	GtkWidget * parent = frame->getTopLevelWindow();
 	UT_ASSERT(parent);
+
 	// center it
-    centerDialog(parent, GTK_WIDGET(pFS));
+	centerDialog(parent, GTK_WIDGET(pFS));
 	
 	gtk_widget_show(GTK_WIDGET(pFS));
 	gtk_grab_add(GTK_WIDGET(pFS));
@@ -760,14 +795,14 @@ gint XAP_UnixDialog_FileOpenSaveAs::previewPicture (void)
 	int answer = 0;
 
 	UT_ByteBuf *pBB = NULL;
-	IEGraphicFileType iegft = IEGFT_Unknown;
+	FG_Graphic * pGraphic = 0;
 	IE_ImpGraphic* pIEG = NULL;
 	UT_Error errorCode = UT_OK;
 	GR_Image *pImage = NULL;
 
 	double		scale_factor = 0.0;
-	UT_sint32	scaled_width,scaled_height;
-	UT_sint32	iImageWidth,iImageHeight;
+	UT_sint32     scaled_width,scaled_height;
+	UT_sint32     iImageWidth,iImageHeight;
 
 	if (!buf)
 	  {
@@ -793,88 +828,61 @@ gint XAP_UnixDialog_FileOpenSaveAs::previewPicture (void)
 	pBB->insertFromFile(0, buf);
 
 	// Build an Import Graphic based on file type
-	errorCode = IE_ImpGraphic::constructImporter(buf, iegft, &pIEG);
-	if (errorCode)
+	errorCode = IE_ImpGraphic::constructImporter(buf, IEGFT_Unknown, &pIEG);
+	if ((errorCode != UT_OK) || !pIEG)
 	{
 		DELETEP(pBB);
 		pGr->drawChars (ucstext, 0, len, 12, 35);
 		goto Cleanup;
 	}
-	iegft = pIEG->fileTypeForContents( (const char *) pBB->getPointer(0), pBB->getLength());
 
-	// Skip import if PNG or SVG file
-	if (iegft != IEGFT_PNG && iegft != IEGFT_SVG)
-	{
-		// Convert to PNG or SVG (pBB Memoried freed in function
-		UT_ByteBuf *pTempBB = NULL; 
-		errorCode = pIEG->convertGraphic(pBB, &pTempBB);  
-		pBB = pTempBB;
-		if (errorCode)
-		{
-			DELETEP(pIEG);
-			DELETEP(pBB);
-			pGr->drawChars (ucstext, 0, len, 12, 35);
-			goto Cleanup;
-		}
-	}
-	// Reset file type based on conversion
-	iegft = pIEG->fileTypeForContents( (const char *) pBB->getPointer(0), pBB->getLength());
-	DELETEP(pIEG);
+	errorCode = pIEG->importGraphic (pBB, &pGraphic);
 
-	if (iegft == IEGFT_PNG)
-	{
-		UT_PNG_getDimensions(pBB, iImageWidth, iImageHeight);
-	}
-	else if (iegft == IEGFT_SVG)
-	{
-		UT_sint32 layoutWidth;
-		UT_sint32 layoutHeight;
-		UT_SVG_getDimensions(pBB, pGr, iImageWidth, iImageHeight, layoutWidth, layoutHeight);
-	}
-	else
-	{
-		UT_ASSERT(UT_SHOULD_NOT_HAPPEN); // But it could, I think.
-	}
+	if ((errorCode != UT_OK) || !pGraphic)
+	  {
+	    pGr->drawChars (ucstext, 0, len, 12, 35);
+	    goto Cleanup;
+	  }
 
-	if (m_preview->allocation.width >= iImageWidth && m_preview->allocation.height >= iImageHeight)
-		scale_factor = 1.0;
-	else
-		scale_factor = MIN( (double) m_preview->allocation.width/iImageWidth,
-				    (double) m_preview->allocation.height/iImageHeight);
-
-	scaled_width  = (int) (scale_factor * iImageWidth);
-	scaled_height = (int) (scale_factor * iImageHeight);
-
-	if (iegft == IEGFT_PNG)
+	if ( FGT_Raster == pGraphic->getType () )
 	{
 #if defined(HAVE_GNOME)
 		pImage = new GR_UnixGnomeImage(NULL,false);
 #else
 		pImage = new GR_UnixImage(NULL);
 #endif
+		UT_ByteBuf * png = static_cast<FG_GraphicRaster*>(pGraphic)->getRaster_PNG();
+		UT_PNG_getDimensions (png, iImageWidth, iImageHeight);
+
+		if (m_preview->allocation.width >= iImageWidth && m_preview->allocation.height >= iImageHeight)
+		  scale_factor = 1.0;
+		else
+		  scale_factor = MIN( (double) m_preview->allocation.width/iImageWidth,
+				      (double) m_preview->allocation.height/iImageHeight);
+		
+		scaled_width  = (int)(scale_factor * iImageWidth);
+		scaled_height = (int)(scale_factor * iImageHeight);
+
+		pImage->convertFromBuffer(png, scaled_width, scaled_height);
+		
+		pGr->drawImage(pImage,
+			       (int)((m_preview->allocation.width  - scaled_width ) / 2),
+			       (int)((m_preview->allocation.height - scaled_height) / 2));
+		
+		answer = 1;
 	}
-	else // if (iegft == IEGFT_SVG)
+	else // if ( FGT_Vector == pGraphic->getType () )
 	{
-		pImage = new GR_VectorImage(NULL);
+	  //pImage = new GR_VectorImage(NULL);
 	}
 
-	if ( iegft == IEGFT_PNG ) // disable png for now
-	  {
-
-	    pImage->convertFromBuffer(pBB, scaled_width, scaled_height);
-	    
-	    pGr->drawImage(pImage,
-			   (m_preview->allocation.width  - scaled_width ) / 2,
-			   (m_preview->allocation.height - scaled_height) / 2);
-	    
-	    answer = 1;
-	  }
  Cleanup:
-	DELETEP(pBB);
 	DELETEP(pImage);
 	DELETEP(pGr);
 	DELETEP(fnt);
 	DELETEPV(ucstext);
+	DELETEP(pIEG);
+	DELETEP(pGraphic);
 
 	return answer;
 }
