@@ -225,7 +225,69 @@ gint XAP_UnixFrame::_fe::button_release_event(GtkWidget * w, GdkEventButton * e)
 	
 	return 1;
 }
-	
+
+/*!
+ * Background zoom updater. It updates the view zoom level after all configure
+ * events have been processed. This is 
+ */
+gint XAP_UnixFrame::_fe::do_ZoomUpdate(gpointer /* XAP_UnixFrame * */ p)
+{
+	XAP_UnixFrame * pUnixFrame = static_cast<XAP_UnixFrame *>(p);
+	AV_View * pView = pUnixFrame->getCurrentView();
+	if(!pView)
+	{
+		pUnixFrame->m_iZoomUpdateID = 0;
+		pUnixFrame->m_bDoZoomUpdate = false;
+		return FALSE;
+	}
+	if(pUnixFrame->m_bDoZoomUpdate && (pView->getWindowWidth() == pUnixFrame->m_iNewWidth) && (pView->getWindowHeight() == pUnixFrame->m_iNewHeight))
+	{
+		pUnixFrame->m_iZoomUpdateID = 0;
+		pUnixFrame->m_bDoZoomUpdate = false;
+		return FALSE;
+	}
+
+    pUnixFrame->m_bDoZoomUpdate = true;
+	UT_sint32 iNewWidth = 0;
+	UT_sint32 iNewHeight = 0;
+	do
+	{
+		AV_View * pView = pUnixFrame->getCurrentView();
+		if(!pView)
+		{
+			pUnixFrame->m_iZoomUpdateID = 0;
+			pUnixFrame->m_bDoZoomUpdate = false;
+			return FALSE;
+		}
+		while(pView->isLayoutFilling())
+		{
+//
+// Comeback when it's finished.
+//
+			return TRUE;
+		}
+		iNewWidth = pUnixFrame->m_iNewWidth;
+		iNewHeight = pUnixFrame->m_iNewHeight;
+		pView = pUnixFrame->getCurrentView();
+		if(pView)
+		{
+			pUnixFrame->_startViewAutoUpdater();
+			pView->setWindowSize(iNewWidth, iNewHeight);
+			pUnixFrame->updateZoom();
+		}
+		else
+		{
+			pUnixFrame->m_iZoomUpdateID = 0;
+			pUnixFrame->m_bDoZoomUpdate = false;
+			return FALSE;
+		}
+	}
+	while((iNewWidth != pUnixFrame->m_iNewWidth) || (iNewHeight != pUnixFrame->m_iNewHeight));
+	pUnixFrame->m_iZoomUpdateID = 0;
+	pUnixFrame->m_bDoZoomUpdate = false;
+	return FALSE;
+}
+
 gint XAP_UnixFrame::_fe::configure_event(GtkWidget* w, GdkEventConfigure *e)
 {
 	// This is basically a resize event.
@@ -234,11 +296,15 @@ gint XAP_UnixFrame::_fe::configure_event(GtkWidget* w, GdkEventConfigure *e)
 	AV_View * pView = pUnixFrame->getCurrentView();
 
 	if (pView)
-		pView->setWindowSize(e->width, e->height);
-
-	// Dynamic Zoom Implimentation
-   pUnixFrame->updateZoom();
-
+	{
+		pUnixFrame->m_iNewWidth = e->width;
+		pUnixFrame->m_iNewHeight = e->height;
+		// Dynamic Zoom Implimentation
+		if(!pUnixFrame->m_bDoZoomUpdate && (pUnixFrame->m_iZoomUpdateID == 0))
+		{
+			pUnixFrame->m_iZoomUpdateID = gtk_idle_add((GtkFunction) do_ZoomUpdate, (gpointer) pUnixFrame);
+		}
+	}
 	return 1;
 }
 	
@@ -344,7 +410,7 @@ gint XAP_UnixFrame::_fe::expose(GtkWidget * w, GdkEventExpose* pExposeEvent)
 		GR_Graphics * pG = pView->getGraphics();
 		pG->doRepaint(&rClip);
 	}
-	return 0;
+	return 0; 
 }
 
 /*!
@@ -360,7 +426,7 @@ gint XAP_UnixFrame::_fe::abi_expose_repaint( gpointer p)
 	UT_Rect localCopy;
 	XAP_UnixFrame * pF = static_cast<XAP_UnixFrame *>(p);
 	FV_View * pV = (FV_View *) pF->getCurrentView();
-	if(!pV)
+	if(!pV || (pV->getPoint() == 0))
 	{ 
 		return TRUE;
 	}
@@ -387,7 +453,7 @@ gint XAP_UnixFrame::_fe::abi_expose_repaint( gpointer p)
 //
 		pG->setExposePending(false);
 		pG->setExposedAreaAccessed(false);
-//			UT_DEBUGMSG(("Painting area:  left=%d, top=%d, width=%d, height=%d\n", localCopy.left, localCopy.top, localCopy.width, localCopy.height));
+		xxx_UT_DEBUGMSG(("Painting area:  left=%d, top=%d, width=%d, height=%d\n", localCopy.left, localCopy.top, localCopy.width, localCopy.height));
 		xxx_UT_DEBUGMSG(("SEVIOR: Repaint now \n"));
 		pV->draw(&localCopy);
 	}
@@ -443,6 +509,9 @@ XAP_UnixFrame::XAP_UnixFrame(XAP_UnixApp * app)
 	m_pUnixPopup = NULL;
 	m_pView = NULL;
 	m_iAbiRepaintID = 0;
+	m_bDoZoomUpdate = false;
+	m_iZoomUpdateID = 0;
+	m_wTopLevelWindow = NULL;
 }
 
 // TODO when cloning a new frame from an existing one
@@ -458,6 +527,8 @@ XAP_UnixFrame::XAP_UnixFrame(XAP_UnixFrame * f)
 	m_pUnixPopup = NULL;
 	m_pView = NULL;
 	m_iAbiRepaintID = 0;
+	m_bDoZoomUpdate = false;
+	m_iZoomUpdateID = 0;
 }
 
 XAP_UnixFrame::~XAP_UnixFrame(void)
@@ -467,6 +538,10 @@ XAP_UnixFrame::~XAP_UnixFrame(void)
 	{
 		gtk_timeout_remove(m_iAbiRepaintID);
 	}
+	if(m_bDoZoomUpdate)
+	{
+		gtk_timeout_remove(m_iZoomUpdateID);
+	}
 	DELETEP(m_pUnixMenu);
 	DELETEP(m_pUnixPopup);
 }
@@ -515,6 +590,125 @@ bool XAP_UnixFrame::initialize(const char * szKeyBindingsKey, const char * szKey
 	return true;
 }
 
+
+void XAP_UnixFrame::setCursor(GR_Graphics::Cursor c)
+{
+//	if (m_cursor == c)
+//		return;
+//	m_cursor = c;
+	FV_View * pView = (FV_View *) getCurrentView();
+	if(pView)
+	{
+		GR_Graphics * pG = pView->getGraphics();
+		if(pG && pG->queryProperties( GR_Graphics::DGP_PAPER))
+		{
+			return;
+		}
+	}
+	if(getTopLevelWindow() == NULL || (m_iFrameMode != XAP_NormalFrame))
+	{
+		return;
+	}
+	GdkCursorType cursor_number;
+	
+	switch (c)
+	{
+	default:
+		UT_ASSERT(UT_NOT_IMPLEMENTED);
+		/*FALLTHRU*/
+	case GR_Graphics::GR_CURSOR_DEFAULT:
+		cursor_number = GDK_TOP_LEFT_ARROW;
+		break;
+		
+	case GR_Graphics::GR_CURSOR_IBEAM:
+		cursor_number = GDK_XTERM;
+		break;
+
+	//I have changed the shape of the arrow so get a consistent
+	//behaviour in the bidi build; I think the new arrow is better
+	//for the purpose anyway
+	
+	case GR_Graphics::GR_CURSOR_RIGHTARROW:
+		cursor_number = GDK_SB_RIGHT_ARROW; //GDK_ARROW;
+		break;
+
+#ifdef BIDI_ENABLED
+//#error choose a suitable cursor; this is just a placeholder !!!		
+	case GR_Graphics::GR_CURSOR_LEFTARROW:
+		cursor_number = GDK_SB_LEFT_ARROW; //GDK_LEFT_PTR;
+		break;
+#endif		
+
+	case GR_Graphics::GR_CURSOR_IMAGE:
+		cursor_number = GDK_FLEUR;
+		break;
+		
+	case GR_Graphics::GR_CURSOR_IMAGESIZE_NW:
+		cursor_number = GDK_TOP_LEFT_CORNER;
+		break;
+		
+	case GR_Graphics::GR_CURSOR_IMAGESIZE_N:
+		cursor_number = GDK_TOP_SIDE;
+		break;
+		
+	case GR_Graphics::GR_CURSOR_IMAGESIZE_NE:
+		cursor_number = GDK_TOP_RIGHT_CORNER;
+		break;
+		
+	case GR_Graphics::GR_CURSOR_IMAGESIZE_E:
+		cursor_number = GDK_RIGHT_SIDE;
+		break;
+		
+	case GR_Graphics::GR_CURSOR_IMAGESIZE_SE:
+		cursor_number = GDK_BOTTOM_RIGHT_CORNER;
+		break;
+		
+	case GR_Graphics::GR_CURSOR_IMAGESIZE_S:
+		cursor_number = GDK_BOTTOM_SIDE;
+		break;
+		
+	case GR_Graphics::GR_CURSOR_IMAGESIZE_SW:
+		cursor_number = GDK_BOTTOM_LEFT_CORNER;
+		break;
+		
+	case GR_Graphics::GR_CURSOR_IMAGESIZE_W:
+		cursor_number = GDK_LEFT_SIDE;
+		break;
+		
+	case GR_Graphics::GR_CURSOR_LEFTRIGHT:
+		cursor_number = GDK_SB_H_DOUBLE_ARROW;
+		break;
+
+	case GR_Graphics::GR_CURSOR_UPDOWN:
+		cursor_number = GDK_SB_V_DOUBLE_ARROW;
+		break;
+
+	case GR_Graphics::GR_CURSOR_EXCHANGE:
+		cursor_number = GDK_EXCHANGE;
+		break;
+
+	case GR_Graphics::GR_CURSOR_GRAB:
+		cursor_number = GDK_HAND1;
+		break;
+
+	case GR_Graphics::GR_CURSOR_LINK:
+		cursor_number = GDK_HAND2;
+		break;
+
+	case GR_Graphics::GR_CURSOR_WAIT:
+		cursor_number = GDK_WATCH;
+		break;
+	}
+
+	GdkCursor * cursor = gdk_cursor_new(cursor_number);
+	gdk_window_set_cursor(getTopLevelWindow()->window, cursor);
+	gdk_window_set_cursor(getVBoxWidget()->window, cursor);
+	gdk_window_set_cursor(m_wSunkenBox->window, cursor);
+	gdk_window_set_cursor(m_wStatusBar->window, cursor);
+	gdk_cursor_destroy(cursor);
+
+}
+
 UT_sint32 XAP_UnixFrame::setInputMode(const char * szName)
 {
 	UT_sint32 result = XAP_Frame::setInputMode(szName);
@@ -549,29 +743,37 @@ XAP_DialogFactory * XAP_UnixFrame::getDialogFactory(void)
 
 void XAP_UnixFrame::nullUpdate() const
 {
-  gtk_main_iteration ();
+	UT_uint32 i =0;
+	while(gtk_events_pending() && (i < 5))
+	{
+		gtk_main_iteration ();
+		i++;
+	}
 }
 
+
 void XAP_UnixFrame::_createTopLevelWindow(void)
 {
 	// create a top-level window for us.
 
 	bool bResult;
-
-	m_wTopLevelWindow = gtk_window_new(GTK_WINDOW_TOPLEVEL);
-	gtk_object_set_data(GTK_OBJECT(m_wTopLevelWindow), "ic_attr", NULL);
-	gtk_object_set_data(GTK_OBJECT(m_wTopLevelWindow), "ic", NULL);
+	if(m_iFrameMode == XAP_NormalFrame)
+	{
+		m_wTopLevelWindow = gtk_window_new(GTK_WINDOW_TOPLEVEL);
+		gtk_object_set_data(GTK_OBJECT(m_wTopLevelWindow), "ic_attr", NULL);
+		gtk_object_set_data(GTK_OBJECT(m_wTopLevelWindow), "ic", NULL);
+		gtk_window_set_title(GTK_WINDOW(m_wTopLevelWindow),
+							 m_pUnixApp->getApplicationTitleForTitleBar());
+		gtk_window_set_policy(GTK_WINDOW(m_wTopLevelWindow), TRUE, TRUE, FALSE);
+		gtk_window_set_wmclass(GTK_WINDOW(m_wTopLevelWindow),
+							   m_pUnixApp->getApplicationName(),
+							   m_pUnixApp->getApplicationName());
+	}
 	gtk_object_set_data(GTK_OBJECT(m_wTopLevelWindow), "toplevelWindow",
 						m_wTopLevelWindow);
 	gtk_object_set_data(GTK_OBJECT(m_wTopLevelWindow), "toplevelWindowFocus",
 						GINT_TO_POINTER(FALSE));
 	gtk_object_set_user_data(GTK_OBJECT(m_wTopLevelWindow),this);
-	gtk_window_set_title(GTK_WINDOW(m_wTopLevelWindow),
-						 m_pUnixApp->getApplicationTitleForTitleBar());
-	gtk_window_set_policy(GTK_WINDOW(m_wTopLevelWindow), TRUE, TRUE, FALSE);
-	gtk_window_set_wmclass(GTK_WINDOW(m_wTopLevelWindow),
-						   m_pUnixApp->getApplicationName(),
-						   m_pUnixApp->getApplicationName());
 
 	// This is now done with --geometry parsing.
 	//gtk_widget_set_usize(GTK_WIDGET(m_wTopLevelWindow), 700, 650);
@@ -620,8 +822,11 @@ void XAP_UnixFrame::_createTopLevelWindow(void)
 	// TODO for some reason, the toolbar functions require the TLW to be
 	// TODO realized (they reference m_wTopLevelWindow->window) before we call them.
 
-	gtk_widget_realize(m_wTopLevelWindow);
 
+	if(m_iFrameMode == XAP_NormalFrame)
+	{
+		gtk_widget_realize(m_wTopLevelWindow);
+	}
 
 
 	gtk_signal_connect(GTK_OBJECT(m_wTopLevelWindow), "key_press_event",
@@ -642,8 +847,9 @@ void XAP_UnixFrame::_createTopLevelWindow(void)
 	// if it wants to.  we will put it below the document
 	// window (a peer with toolbars and the overall sunkenbox)
 	// so that it will appear outside of the scrollbars.
-
+	m_wStatusBar = NULL;
 	m_wStatusBar = _createStatusBarWindow();
+
 	if (m_wStatusBar)
 	{
 		gtk_widget_show(m_wStatusBar);
@@ -690,6 +896,68 @@ void XAP_UnixFrame::_createTopLevelWindow(void)
 	return;
 }
 
+/*!
+ * This code is used by the dynamic menu API to rebuild the menus after a
+ * a change in the menu structure.
+ */
+void XAP_UnixFrame::rebuildMenus(void)
+{
+//
+// Destroy the old menu bar
+//
+	m_pUnixMenu->destroy();
+//
+// Delete the old class
+//
+	DELETEP(m_pUnixMenu);
+//
+// Build a new one.
+//
+	m_pUnixMenu = new EV_UnixMenuBar(m_pUnixApp,this,
+									 m_szMenuLayoutName,
+									 m_szMenuLabelSetName);
+	UT_ASSERT(m_pUnixMenu);
+	bool bResult = m_pUnixMenu->rebuildMenuBar();
+	UT_ASSERT(bResult);
+
+}
+
+
+/*!
+ * This code is used by the dynamic toolbar API to rebuild a toolbar after a
+ * a change in the toolbar structure.
+ */
+void XAP_UnixFrame::rebuildToolbar(UT_uint32 ibar)
+{
+//
+// Destroy the old toolbar
+//
+	EV_Toolbar * pToolbar = (EV_Toolbar *) m_vecToolbars.getNthItem(ibar);
+	const char * szTBName = (const char *) m_vecToolbarLayoutNames.getNthItem(ibar);
+	EV_UnixToolbar * pUTB = static_cast<EV_UnixToolbar *>( pToolbar);
+	UT_sint32 oldpos = pUTB->destroy();
+//
+// Delete the old class
+//
+	delete pToolbar;
+	if(oldpos < 0)
+	{
+		return;
+	}
+//
+// Build a new one.
+//
+	pToolbar = _newToolbar(m_app, (XAP_Frame *) this,szTBName,
+						   (const char *) m_szToolbarLabelSetName);
+	static_cast<EV_UnixToolbar *>(pToolbar)->rebuildToolbar(oldpos);
+	m_vecToolbars.setNthItem(ibar, (void *) pToolbar, NULL);
+//
+// Refill the framedata pointers
+//
+	refillToolbarsInFrameData();
+	repopulateCombos();
+}
+
 bool XAP_UnixFrame::close()
 {
 	gtk_widget_destroy(getTopLevelWindow());
@@ -708,8 +976,10 @@ bool XAP_UnixFrame::raise()
 
 bool XAP_UnixFrame::show()
 {
-	gtk_widget_show(m_wTopLevelWindow);
-
+	if(m_wTopLevelWindow)
+	{
+		gtk_widget_show(m_wTopLevelWindow);
+	}
 	return true;
 }
 
@@ -739,63 +1009,85 @@ bool XAP_UnixFrame::openURL(const char * szURL)
 	// TODO: or text/html handler in a global registry
 
 	// ORDER:
+	// Use value of environment variable BROWSER, if valid; otherwise:
 	// 1) konqueror
 	// 2) mozilla
 	// 3) netscape
 	// 4) kdehelp
 	// 5) lynx in an xterm
-
-	if(progExists("konqueror"))
+  	char *env_browser = getenv ("BROWSER");
+  	if (env_browser)
+  	{
+		if(progExists(env_browser))
+		{
+			if (strstr (env_browser, "netscape"))
+			{
+				fmtstring = g_strdup_printf("%s -remote openURL\\('%%s'\\) || %s '%%s' &", env_browser, env_browser);
+				if (fmtstring) execstring = g_strdup_printf(fmtstring, szURL, szURL);
+			}
+			else
+			{
+				fmtstring = g_strdup_printf("%s '%%s' &", env_browser);
+				if (fmtstring) execstring = g_strdup_printf(fmtstring, szURL);
+			}
+		}
+  	}
+	if (fmtstring == 0)
 	{
-		fmtstring = "konqueror %s &";
-		execstring = g_strdup_printf(fmtstring, szURL);
+		if(progExists("konqueror"))
+		{
+			fmtstring = "konqueror '%s' &";
+			execstring = g_strdup_printf(fmtstring, szURL);
+		}
+		else if(progExists("galeon"))
+		{
+		  	fmtstring = "galeon '%s' &";
+			execstring = g_strdup_printf(fmtstring, szURL);
+		}
+		// Anyone know how to find out where it might be, regardless?
+		else if(progExists("mozilla"))
+		{
+		        fmtstring = "mozilla '%s' &";
+			execstring = g_strdup_printf(fmtstring, szURL);
+		}
+		else if(progExists("netscape"))
+		{
+			// Try to connect to a running Netscape, if not, start new one
+			fmtstring = "netscape -remote openURL\\('%s'\\) || netscape '%s' &";
+			execstring = g_strdup_printf(fmtstring, szURL, szURL);
+		}
+		else if(progExists("khelpcenter"))
+		{
+			fmtstring = "khelpcenter '%s' &";
+			execstring = g_strdup_printf(fmtstring, szURL);
+		}
+		else if(progExists("gnome-help-browser"))
+		{
+			fmtstring = "gnome-help-browser '%s' &";
+			execstring = g_strdup_printf(fmtstring, szURL);
+		}
+		else if(progExists("lynx"))
+		{
+			fmtstring = "xterm -e lynx '%s' &";
+			execstring = g_strdup_printf(fmtstring, szURL);
+		}
+		else if(progExists("w3m"))
+		{
+			fmtstring = "xterm -e w3m '%s' &";
+	 		execstring = g_strdup_printf(fmtstring, szURL);
+	 	}
 	}
-	else if(progExists("galeon"))
+	if (execstring)
 	{
-	  	fmtstring = "galeon %s &";
-		execstring = g_strdup_printf(fmtstring, szURL);
+		system (execstring);
+		g_free (execstring);
 	}
-	// Anyone know how to find out where it might be, regardless?
-	else if(progExists("mozilla"))
-	{
-	        fmtstring = "mozilla %s &";
-		execstring = g_strdup_printf(fmtstring, szURL);
-	}
-	else if(progExists("netscape"))
-	{
-		// Try to connect to a running Netscape, if not, start new one
-		fmtstring = "netscape -remote openURL\\(%s\\) || netscape %s &";
-		execstring = g_strdup_printf(fmtstring, szURL, szURL);
-	}
-	else if(progExists("khelpcenter"))
-	{
-		fmtstring = "khelpcenter %s &";
-		execstring = g_strdup_printf(fmtstring, szURL);
-	}
-	else if(progExists("gnome-help-browser"))
-	{
-		fmtstring = "gnome-help-browser %s &";
-		execstring = g_strdup_printf(fmtstring, szURL);
-	}
-	else if(progExists("lynx"))
-	{
-		fmtstring = "xterm -e lynx %s &";
-		execstring = g_strdup_printf(fmtstring, szURL);
-	}
-	else if(progExists("w3m"))
-	{
-		fmtstring = "xterm -e w3m %s &";
- 		execstring = g_strdup_printf(fmtstring, szURL);
- 	}
-
-	system(execstring);
-	g_free (execstring);
 	return false;
 }
 
 bool XAP_UnixFrame::updateTitle()
 {
-	if (!XAP_Frame::updateTitle())
+	if (!XAP_Frame::updateTitle() || (m_wTopLevelWindow== NULL) || (m_iFrameMode != XAP_NormalFrame))
 	{
 		// no relevant change, so skip it
 		return false;
@@ -869,7 +1161,11 @@ bool XAP_UnixFrame::runModalContextMenu(AV_View * /* pView */, const char * szMe
 		// gtk_main_quit() when they're done.
 		gtk_main();
 	}
-
+	XAP_Frame * pFrame = (XAP_Frame *) this;
+	if (pFrame->getCurrentView())
+	{
+		pFrame->getCurrentView()->focusChange( AV_FOCUS_HERE);
+	}
 	DELETEP(m_pUnixPopup);
 	return bResult;
 }
