@@ -61,6 +61,7 @@ XAP_Win32Dialog_FileOpenSaveAs::XAP_Win32Dialog_FileOpenSaveAs(XAP_DialogFactory
 															 XAP_Dialog_Id id)
 	: XAP_Dialog_FileOpenSaveAs(pDlgFactory,id)
 {
+	m_szDefaultExtension[0] = 0;
 }
 
 XAP_Win32Dialog_FileOpenSaveAs::~XAP_Win32Dialog_FileOpenSaveAs(void)
@@ -85,6 +86,32 @@ static bool SuffixInList(const char *haystack, const char *needle)
 	return false;
 }
 
+/*!
+ Retrieve the default extension to use if user does not provide one
+
+ \param indx -- index into the filter list
+ */
+char * XAP_Win32Dialog_FileOpenSaveAs::_getDefaultExtension(UT_uint32 indx)
+{
+	static char abw_sfx[] = "abw";
+	
+	UT_uint32 end = UT_pointerArrayLength((void **) m_szDescriptions);
+	if(indx >= end)
+		return abw_sfx;
+	
+	// copy at most DEFAULT_EXT_SIZE characters from the suffix;
+	strncpy(m_szDefaultExtension, m_szSuffixes[indx] + 2, DEFAULT_EXT_SIZE);
+	
+	m_szDefaultExtension[DEFAULT_EXT_SIZE] = 0;
+	
+	// make sure that we get rid off the semicolon if it got copied
+	char * semicolon = strchr(m_szDefaultExtension, ';');
+	if(semicolon)
+		*semicolon = 0;
+	UT_DEBUGMSG(("Default sfx [%s], (from [%s]\n", m_szDefaultExtension,m_szSuffixes[indx]));
+	return m_szDefaultExtension;
+}
+
 /*!
  Build a Windows filter list from Abi's filetypes
 
@@ -94,14 +121,14 @@ void XAP_Win32Dialog_FileOpenSaveAs::_buildFilterList(UT_String& sFilter)
 {
 	UT_String sAllSuffixes;
 
-	const XAP_StringSet*  pSS   = XAP_App::getApp()->getStringSet();
+	const XAP_StringSet*  pSS	= XAP_App::getApp()->getStringSet();
 
 	UT_ASSERT(UT_pointerArrayLength((void **) m_szSuffixes) ==
 			  UT_pointerArrayLength((void **) m_szDescriptions));
 
 	// measure one list, they should all be the same length
 	UT_uint32 end = UT_pointerArrayLength((void **) m_szDescriptions);
-	
+
 	for (UT_uint32 i = 0; i < end; i++)
 	{
 		sFilter += m_szDescriptions[i];
@@ -147,6 +174,48 @@ void XAP_Win32Dialog_FileOpenSaveAs::_buildFilterList(UT_String& sFilter)
 	sFilter += '\0';				// double 0 at the end
 }
 
+/*****************************************************************/
+
+/*!  
+  Gets the Windows File/Save common dialog box. To be able to get
+  the places bar containing icons for commonly-used folders when we
+  use hooking we need to use the new OPENFILENAME structure as defined
+  in WINNT5 or better. To keep backward compability, first try
+  with the new size of the structure, if it does not work it means the
+  the common dialog box DLL installed in the system does not support
+  the new features, and then we use the old one. In this situation, we
+  user does not have a places bar capable common dialog box DLL
+  anyway.
+*/
+BOOL  XAP_Win32Dialog_FileOpenSaveAs::GetSaveFileName_Hooked(OPENFILENAME_WIN50* lpofn)
+{
+		BOOL  bRslt;	
+		
+		lpofn->lStructSize = sizeof(OPENFILENAME_WIN50);		// Size of the new structure
+		lpofn->pvReserved = NULL;
+		lpofn->dwReserved = 0;
+		lpofn->FlagsEx = 0;		
+		
+				
+		bRslt = GetSaveFileName((OPENFILENAME *)lpofn);
+		
+		if (!bRslt) // Error
+		{			
+			DWORD dwError = CommDlgExtendedError();
+			
+			if (dwError==CDERR_STRUCTSIZE)	// This system does not support the place bar
+			{								
+					//  Try with the old one
+					lpofn->lStructSize = sizeof(OPENFILENAME);	
+					bRslt = GetSaveFileName((OPENFILENAME *)lpofn);
+			}
+				
+		}	
+		
+		return bRslt;
+}
+
+
 /*****************************************************************/
 
 void XAP_Win32Dialog_FileOpenSaveAs::runModal(XAP_Frame * pFrame)
@@ -158,18 +227,18 @@ void XAP_Win32Dialog_FileOpenSaveAs::runModal(XAP_Frame * pFrame)
 
 	HWND hFrame = m_pWin32Frame->getTopLevelWindow();
 
-	char szFile[MAX_DLG_INS_PICT_STRING];   // buffer for filename
+	char szFile[MAX_DLG_INS_PICT_STRING];	// buffer for filename
 	char szDir[MAX_DLG_INS_PICT_STRING];	// buffer for directory
 	UT_String sFilter;
-	OPENFILENAME ofn;						// common dialog box structure
+	OPENFILENAME_WIN50 ofn;						// common dialog box structure
 
 	ZeroMemory(szFile,sizeof(szFile));
 	ZeroMemory(szDir,sizeof(szDir));
-	ZeroMemory(&ofn, sizeof(OPENFILENAME));
+	ZeroMemory(&ofn, sizeof(OPENFILENAME_WIN50));
 
 	_buildFilterList(sFilter);
 
-	ofn.lStructSize = sizeof(OPENFILENAME);
+	ofn.lStructSize = sizeof(OPENFILENAME);		// Old size
 	ofn.hwndOwner = hFrame;
 	ofn.lpstrFile = szFile;
 	ofn.nMaxFile = sizeof(szFile);
@@ -179,16 +248,17 @@ void XAP_Win32Dialog_FileOpenSaveAs::runModal(XAP_Frame * pFrame)
 	ofn.nMaxFileTitle = 0;
 	ofn.lpstrInitialDir = NULL;
 	ofn.Flags = OFN_PATHMUSTEXIST | OFN_NOCHANGEDIR | OFN_HIDEREADONLY;
-			   
+	ofn.lpstrDefExt = _getDefaultExtension(0);
+	ofn.lCustData = (DWORD)this;
 	// use the persistence info and/or the suggested filename
 	// to properly seed the dialog.
 
 	if (!m_szInitialPathname || !*m_szInitialPathname)
 	{
 		// the caller did not supply initial pathname
-		// (or supplied an empty one).  see if we have
+		// (or supplied an empty one).	see if we have
 		// some persistent info.
-		
+
 		UT_ASSERT(!m_bSuggestName);
 		if (m_szPersistPathname)
 		{
@@ -217,8 +287,8 @@ void XAP_Win32Dialog_FileOpenSaveAs::runModal(XAP_Frame * pFrame)
 		// we have an initial pathname (the name of the document
 		// in the frame that we were invoked on).  if the caller
 		// wanted us to suggest a name, use the initial
-		// pathname as is.  if not, use the directory portion of
-		// it.  either way, we need to cut the pathname into two
+		// pathname as is.	if not, use the directory portion of
+		// it.	either way, we need to cut the pathname into two
 		// parts -- directory and file -- for the common dlg.
 
 		strcpy(szDir,m_szInitialPathname);
@@ -226,16 +296,25 @@ void XAP_Win32Dialog_FileOpenSaveAs::runModal(XAP_Frame * pFrame)
 		if (pLastSlash)
 			pLastSlash[1] = 0;
 		ofn.lpstrInitialDir = szDir;
-		
+
 		if (m_bSuggestName)
 		{
 			if (pLastSlash)
 				strcpy(szFile, m_szInitialPathname + (pLastSlash-szDir+1));
 			else
 				strcpy(szFile, m_szInitialPathname);
+
+			// if the file name has an extension, remove it
+			// (if we don't, and the document is of a different
+			// type than the one initially selected in the dialogue
+			// and the user just clicks OK, we get type - extension
+			// mismatch)
+			char * dot = strrchr(szFile, '.');
+			if(dot)
+				*dot = 0;
 		}
 	}
-		
+
 	// display the appropriate dialog box.
 
 	BOOL bDialogResult;
@@ -262,48 +341,59 @@ void XAP_Win32Dialog_FileOpenSaveAs::runModal(XAP_Frame * pFrame)
 	case XAP_DIALOG_ID_FILE_OPEN:
 		ofn.lpstrTitle = pSS->getValue(XAP_STRING_ID_DLG_FOSA_OpenTitle);
 		ofn.Flags |= OFN_FILEMUSTEXIST;
-		bDialogResult = GetOpenFileName(&ofn);
+		ofn.nFilterIndex = UT_pointerArrayLength((void **) m_szDescriptions) + 1;
+		bDialogResult = GetOpenFileName((OPENFILENAME *)&ofn);
 		break;
 
 	case XAP_DIALOG_ID_PRINTTOFILE:
 		ofn.lpstrTitle = pSS->getValue(XAP_STRING_ID_DLG_FOSA_PrintToFileTitle);
 		ofn.Flags |= OFN_OVERWRITEPROMPT;
-		bDialogResult = GetSaveFileName(&ofn);
+		bDialogResult = GetSaveFileName((OPENFILENAME *)&ofn);
 		break;
 
 	case XAP_DIALOG_ID_FILE_SAVEAS:
 		ofn.lpstrTitle = pSS->getValue(XAP_STRING_ID_DLG_FOSA_SaveAsTitle);
-		ofn.lpfnHook       = (LPOFNHOOKPROC) s_hookSaveAsProc;
+		ofn.lpfnHook	   = (LPOFNHOOKPROC) s_hookSaveAsProc;
 		ofn.Flags |= OFN_OVERWRITEPROMPT;
 		ofn.Flags |= OFN_EXPLORER;
 		ofn.Flags |= OFN_ENABLEHOOK;
-		bDialogResult = GetSaveFileName(&ofn);
+				
+		bDialogResult = GetSaveFileName_Hooked(&ofn);
 		break;
 
 	case XAP_DIALOG_ID_INSERT_PICTURE:
-		ofn.lpstrTitle     = pSS->getValue(XAP_STRING_ID_DLG_IP_Title);
-		ofn.hInstance      = pWin32App->getInstance();
+		ofn.lpstrTitle	   = pSS->getValue(XAP_STRING_ID_DLG_IP_Title);
+		ofn.hInstance	   = pWin32App->getInstance();
 		ofn.lpTemplateName = MAKEINTRESOURCE(XAP_RID_DIALOG_INSERT_PICTURE);
-		ofn.lpfnHook       = (LPOFNHOOKPROC) s_hookInsertPicProc;
+		ofn.lpfnHook	   = (LPOFNHOOKPROC) s_hookInsertPicProc;
+		ofn.nFilterIndex   = UT_pointerArrayLength((void **) m_szDescriptions) + 1;
 		ofn.Flags |= OFN_EXPLORER;
 		ofn.Flags |= OFN_ENABLETEMPLATE;
 		ofn.Flags |= OFN_ENABLEHOOK;
-		bDialogResult = GetOpenFileName(&ofn);
+		bDialogResult = GetSaveFileName_Hooked(&ofn);
 		break;
 
 	case XAP_DIALOG_ID_FILE_IMPORT:
-		ofn.lpstrTitle = pSS->getValue(XAP_STRING_ID_DLG_FOSA_ImportTitle);
+		ofn.lpstrTitle	 = pSS->getValue(XAP_STRING_ID_DLG_FOSA_ImportTitle);
+		ofn.nFilterIndex = UT_pointerArrayLength((void **) m_szDescriptions) + 1;
 		ofn.Flags |= OFN_FILEMUSTEXIST;
-		bDialogResult = GetOpenFileName(&ofn);
+		bDialogResult = GetOpenFileName((OPENFILENAME *)&ofn);
 		break;
 
 	case XAP_DIALOG_ID_FILE_EXPORT:
 		ofn.lpstrTitle = pSS->getValue(XAP_STRING_ID_DLG_FOSA_ExportTitle);
-		ofn.lpfnHook       = (LPOFNHOOKPROC) s_hookSaveAsProc;
+		ofn.lpfnHook	   = (LPOFNHOOKPROC) s_hookSaveAsProc;
 		ofn.Flags |= OFN_OVERWRITEPROMPT;
 		ofn.Flags |= OFN_EXPLORER;
 		ofn.Flags |= OFN_ENABLEHOOK;
-		bDialogResult = GetSaveFileName(&ofn);
+		bDialogResult = GetSaveFileName_Hooked(&ofn);
+		break;
+
+	case XAP_DIALOG_ID_INSERT_FILE:
+		ofn.lpstrTitle = pSS->getValue(XAP_STRING_ID_DLG_FOSA_InsertTitle);
+		ofn.Flags |= OFN_FILEMUSTEXIST;
+		ofn.nFilterIndex = UT_pointerArrayLength((void **) m_szDescriptions) + 1;
+		bDialogResult = GetOpenFileName((OPENFILENAME *)&ofn);
 		break;
 
 	default:
@@ -313,23 +403,31 @@ void XAP_Win32Dialog_FileOpenSaveAs::runModal(XAP_Frame * pFrame)
 
 	// TODO how do cancels get reported...
 	// TODO verify that current-working-directory is not changed...
-	
+
 	if (bDialogResult != FALSE)
 	{
 		UT_uint32 end = UT_pointerArrayLength((void **) m_szSuffixes);
 
-		if ((m_id == XAP_DIALOG_ID_FILE_SAVEAS) && 
-			(!UT_pathSuffix(szFile)) &&
-			(ofn.nFilterIndex <= end))
+		if ((m_id == XAP_DIALOG_ID_FILE_SAVEAS) &&
+			(!UT_pathSuffix(szFile)))
 		{
 			// add suffix based on selected file type
+			// if selected file is "all documents" or "all"
+			// default to .abw, since that is how it will get saved
 			UT_ASSERT(ofn.nFilterIndex > 0);
 
-			const char * szSuffix = UT_pathSuffix(m_szSuffixes[ofn.nFilterIndex - 1]);
+			const char * szSuffix = NULL;
+			char abw_sfx[] = ".abw";
+			
+			if(ofn.nFilterIndex <= end)
+				szSuffix = UT_pathSuffix(m_szSuffixes[ofn.nFilterIndex - 1]);
+			else
+				szSuffix = abw_sfx;
+			
 			UT_ASSERT(szSuffix);
 
 			UT_uint32 length = strlen(szFile) + strlen(szSuffix) + 1;
-			m_szFinalPathname = (char *)calloc(length,sizeof(char));
+			m_szFinalPathname = (char *)UT_calloc(length,sizeof(char));
 			if (m_szFinalPathname)
 			{
 				char * p = m_szFinalPathname;
@@ -356,7 +454,7 @@ void XAP_Win32Dialog_FileOpenSaveAs::runModal(XAP_Frame * pFrame)
 		m_answer = a_CANCEL;
 		UT_DEBUGMSG(("Didn't get a file: reason=0x%x\n", CommDlgExtendedError()));
 	}
-	
+
 	m_pWin32Frame = NULL;
 	return;
 }
@@ -372,24 +470,58 @@ void XAP_Win32Dialog_FileOpenSaveAs::runModal(XAP_Frame * pFrame)
  */
 UINT CALLBACK XAP_Win32Dialog_FileOpenSaveAs::s_hookSaveAsProc(HWND hDlg, UINT msg, WPARAM wParam, LPARAM lParam)
 {
-	XAP_Win32Dialog_FileOpenSaveAs* pThis = (XAP_Win32Dialog_FileOpenSaveAs *) GetWindowLong(hDlg,GWL_USERDATA);
-
+	XAP_Win32Dialog_FileOpenSaveAs* pThis;
+	static char buff[MAX_DLG_INS_PICT_STRING];
 	switch(msg)
 	{
-	case WM_NOTIFY:
-		{
-			const OFNOTIFY * pNotify = reinterpret_cast<OFNOTIFY *>(lParam);
-			switch (pNotify->hdr.code)
+		case WM_NOTIFY:
 			{
-			case CDN_TYPECHANGE:
-				UT_DEBUGMSG(("SaveAs filetype changed to %d\n", pNotify->lpOFN->nFilterIndex));
-				break;
-			}
-		}
-		break;
+				const OFNOTIFY * pNotify = reinterpret_cast<OFNOTIFY *>(lParam);
+				switch (pNotify->hdr.code)
+				{
+					case CDN_TYPECHANGE:
+						{
+							UT_DEBUGMSG(("SaveAs filetype changed to %d\n", pNotify->lpOFN->nFilterIndex));
+							pThis = (XAP_Win32Dialog_FileOpenSaveAs*)pNotify->lpOFN->lCustData;
+							char * ext = pThis->_getDefaultExtension(pNotify->lpOFN->nFilterIndex - 1);
+							// for some reason the  lpstrFile member of the struct will not be set properly
+							// so we have to retrieve the text directly from the control (I could swear that
+							// this used to work, and have no idea what changed)
+							GetDlgItemText(GetParent(hDlg), edt1, buff, MAX_DLG_INS_PICT_STRING);
+							//strcpy(buff,pNotify->lpOFN->lpstrFile);
+							char * dot = strchr(buff, '.');
+							if(dot)
+							{
+								*(dot+1) = 0;
+							}
+							else if(*buff)
+							{
+								UT_ASSERT(strlen(buff) < MAX_DLG_INS_PICT_STRING);
+								dot = buff + strlen(buff);
+								*dot = '.';
+								*(dot+1) = 0;
+							}
 
-	default:
-		return false;
+							if(dot)
+							{
+								UT_ASSERT(strlen(buff) + strlen(pNotify->lpOFN->lpstrDefExt) < MAX_DLG_INS_PICT_STRING);
+
+								if(ext)
+									strcat(buff,ext);
+								else
+									*dot = 0;
+						
+								CommDlg_OpenSave_SetControlText(GetParent(hDlg), edt1, buff);
+							}
+							CommDlg_OpenSave_SetDefExt(GetParent(hDlg), ext);
+						}
+						break;
+				}
+			}
+			break;
+
+		default:
+			return false;
 	}
 	return false;
 }
@@ -407,7 +539,7 @@ UINT CALLBACK XAP_Win32Dialog_FileOpenSaveAs::s_hookInsertPicProc(HWND hDlg, UIN
 {
 	XAP_Win32Dialog_FileOpenSaveAs* pThis = (XAP_Win32Dialog_FileOpenSaveAs *) GetWindowLong(hDlg,GWL_USERDATA);
 	bool bPreviewImage = ( IsDlgButtonChecked( hDlg, XAP_RID_DIALOG_INSERT_PICTURE_CHECK_ACTIVATE_PREVIEW )
-			                  == BST_CHECKED );
+							  == BST_CHECKED );
 
 	switch(msg)
 	{
@@ -445,22 +577,22 @@ UINT CALLBACK XAP_Win32Dialog_FileOpenSaveAs::s_hookInsertPicProc(HWND hDlg, UIN
 				if (bPreviewImage)
 				{
 					ShowWindow( GetDlgItem(hDlg,XAP_RID_DIALOG_INSERT_PICTURE_TEXT_HEIGHT),
-						         SW_SHOW );
+								 SW_SHOW );
 					ShowWindow( GetDlgItem(hDlg,XAP_RID_DIALOG_INSERT_PICTURE_TEXT_WIDTH),
-						         SW_SHOW );
+								 SW_SHOW );
 					pThis->_previewPicture(hDlg);
 					return true;
 				}
 				else
 				{
 					ShowWindow( GetDlgItem(hDlg,XAP_RID_DIALOG_INSERT_PICTURE_TEXT_HEIGHT),
-						         SW_HIDE );
+								 SW_HIDE );
 					ShowWindow( GetDlgItem(hDlg,XAP_RID_DIALOG_INSERT_PICTURE_TEXT_WIDTH),
-						         SW_HIDE );
+								 SW_HIDE );
 					return ( pThis->_initPreviewDlg(hDlg) );
 				}	
 			} 
-		}             
+		}			  
 		return false;
 		break;
 
@@ -479,12 +611,12 @@ UINT CALLBACK XAP_Win32Dialog_FileOpenSaveAs::s_hookInsertPicProc(HWND hDlg, UIN
 	
 UINT XAP_Win32Dialog_FileOpenSaveAs::_previewPicture(HWND hDlg)
 {
-	HWND hFOSADlg   = GetParent(hDlg);
-	HWND hFrame     = GetParent(hFOSADlg);
+	HWND hFOSADlg	= GetParent(hDlg);
+	HWND hFrame 	= GetParent(hFOSADlg);
 	HWND hThumbnail = GetDlgItem(hDlg,XAP_RID_DIALOG_INSERT_PICTURE_IMAGE_PREVIEW);
 
 	XAP_Win32Frame* pWin32Frame = (XAP_Win32Frame *) ( GetWindowLong(hFrame,GWL_USERDATA) );
-	const XAP_StringSet*  pSS   = pWin32Frame->getApp()->getStringSet();
+	const XAP_StringSet*  pSS	= pWin32Frame->getApp()->getStringSet();
 
 	// Check if File Name is for a file
 	char buf[MAX_DLG_INS_PICT_STRING];
@@ -498,7 +630,7 @@ UINT XAP_Win32Dialog_FileOpenSaveAs::_previewPicture(HWND hDlg)
 	UT_DEBUGMSG(("File Selected is %s\n", buf));
 
 	// Load File into memory
-	UT_ByteBuf* pBB     = new UT_ByteBuf(NULL);
+	UT_ByteBuf* pBB 	= new UT_ByteBuf(NULL);
 	UT_ByteBuf* pTempBB = new UT_ByteBuf(NULL);
 	pBB->insertFromFile(0, buf);
 
@@ -548,9 +680,9 @@ UINT XAP_Win32Dialog_FileOpenSaveAs::_previewPicture(HWND hDlg)
 	}
 	else
 	{
-		UT_sint32 layoutWidth;
-		UT_sint32 layoutHeight;
-		// TODO Fix this
+	//	TODO Fix this
+	//	UT_sint32 layoutWidth;
+	//	UT_sint32 layoutHeight;
 	//	UT_SVG_getDimensions(pBB, pGr, iImageWidth, iImageHeight, layoutWidth, layoutHeight);
 		iImageWidth = 0;
 		iImageHeight = 0;
@@ -558,15 +690,15 @@ UINT XAP_Win32Dialog_FileOpenSaveAs::_previewPicture(HWND hDlg)
 
 	// Update Height and Width Strings
 	sprintf( buf, 
-		     "%s %d",
-		      pSS->getValue(XAP_STRING_ID_DLG_IP_Height_Label), 
-		      iImageHeight );
+			 "%s %d",
+			  pSS->getValue(XAP_STRING_ID_DLG_IP_Height_Label), 
+			  iImageHeight );
 	SetDlgItemText( hDlg,
 					XAP_RID_DIALOG_INSERT_PICTURE_TEXT_HEIGHT,
 					buf );
 
 	sprintf( buf, 
-		     "%s %d",
+			 "%s %d",
 			 pSS->getValue(XAP_STRING_ID_DLG_IP_Width_Label),
 			 iImageWidth );
 	SetDlgItemText( hDlg,
@@ -591,8 +723,8 @@ UINT XAP_Win32Dialog_FileOpenSaveAs::_previewPicture(HWND hDlg)
 	FillRect(hdc, &r, GetSysColorBrush(COLOR_BTNFACE));
 	GR_Win32Graphics* pGr = new GR_Win32Graphics(hdc,hThumbnail,pWin32Frame->getApp());
 	pGr->drawImage(pImage,
-	 	          (r.right  - scaled_width ) / 2,
-	 			  (r.bottom - scaled_height) / 2);
+				  (r.right	- scaled_width ) / 2,
+				  (r.bottom - scaled_height) / 2);
 	EndPaint(hThumbnail,&ps);
 
 	DELETEP(pBB);
@@ -604,13 +736,13 @@ UINT XAP_Win32Dialog_FileOpenSaveAs::_previewPicture(HWND hDlg)
 
 UINT XAP_Win32Dialog_FileOpenSaveAs::_initPreviewDlg(HWND hDlg)
 {
-	HWND hFOSADlg   = GetParent(hDlg);
-	HWND hFrame     = GetParent(hFOSADlg);
+	HWND hFOSADlg	= GetParent(hDlg);
+	HWND hFrame 	= GetParent(hFOSADlg);
 	HWND hThumbnail = GetDlgItem(hDlg,XAP_RID_DIALOG_INSERT_PICTURE_IMAGE_PREVIEW);
 
 	XAP_Win32Frame* pWin32Frame = (XAP_Win32Frame *) ( GetWindowLong(hFrame,GWL_USERDATA) );
-	XAP_App*              pApp        = pWin32Frame->getApp();
-	const XAP_StringSet*  pSS         = pApp->getStringSet();
+	XAP_App*			  pApp		  = pWin32Frame->getApp();
+	const XAP_StringSet*  pSS		  = pApp->getStringSet();
 	
 	SetDlgItemText( hDlg,
 					XAP_RID_DIALOG_INSERT_PICTURE_IMAGE_PREVIEW,
