@@ -25,6 +25,7 @@
 #include "ut_vector.h"
 #include "ut_hash.h"
 #include "ut_xml.h"
+#include "ut_string_class.h"
 
 #define NUM_MODELESSID 39
 
@@ -45,6 +46,8 @@ class PD_DocumentRange;
 class AV_View;
 class XAP_EncodingManager;
 class UT_String;
+class XAP_Menu_Factory;
+class XAP_Toolbar_Factory;
 
 /*****************************************************************
 ******************************************************************
@@ -54,7 +57,7 @@ class UT_String;
 ******************************************************************
 *****************************************************************/
 
-class XAP_App
+class ABI_EXPORT XAP_App
 {
 public:									/* TODO these should be protected */
 	static const char* s_szBuild_ID;
@@ -64,6 +67,14 @@ public:									/* TODO these should be protected */
 	static const char* s_szBuild_CompileTime;
 	static const char* s_szBuild_CompileDate;
 
+ public:
+	static const char* getBuildId ();
+	static const char* getBuildVersion ();
+	static const char* getBuildOptions ();
+	static const char* getBuildTarget ();
+	static const char* getBuildCompileTime ();
+	static const char* getBuildCompileDate ();
+
 public:
 	XAP_App(XAP_Args * pArgs, const char * szAppName);
 	virtual ~XAP_App();
@@ -94,17 +105,17 @@ public:
 	EV_Menu_ActionSet *				getMenuActionSet();
 	EV_Toolbar_ActionSet *			getToolbarActionSet();
 
-	XAP_Args *						getArgs() const;
+	XAP_Args *			getArgs() const;
 
-	bool							addWordToDict(const UT_UCSChar * pWord, UT_uint32 len);
-	bool							isWordInDict(const UT_UCSChar * pWord, UT_uint32 len) const;
-
-	XAP_Prefs *						getPrefs() const;
+	bool				addWordToDict(const UT_UCSChar * pWord, UT_uint32 len);
+	bool				isWordInDict(const UT_UCSChar * pWord, UT_uint32 len) const;
+	void                            suggestWord(UT_Vector * pVecSuggestions, const UT_UCSChar * pWord, UT_uint32 lenWord);
+    XAP_Prefs *						getPrefs() const;
 	bool							getPrefsValue(const XML_Char * szKey, const XML_Char ** pszValue) const;
 	bool							getPrefsValue(const UT_String &stKey, UT_String &stValue) const;
 	bool							getPrefsValueBool(const XML_Char * szKey, bool * pbValue) const;
 
-	static XAP_App *						getApp() {return m_pApp;}
+	static XAP_App *				getApp();
 
 	virtual XAP_DialogFactory *				getDialogFactory() = 0;
 	virtual XAP_Toolbar_ControlFactory *	getControlFactory() = 0;
@@ -141,7 +152,16 @@ public:
 														UT_uint32 *width, UT_uint32 *height, UT_uint32 *flags = 0);
 	virtual void 							parseAndSetGeometry(const char *string);
 	virtual UT_sint32						makeDirectory(const char * szPath, const UT_sint32 mode ) const = 0;
-
+	XAP_Menu_Factory *                      getMenuFactory(void) const { return m_pMenuFactory;}
+	XAP_Toolbar_Factory *                   getToolbarFactory(void) const { return m_pToolbarFactory;}
+ 
+    virtual bool                            theOSHasBidiSupport() const {return false;}
+	bool                                    areToolbarsCustomizable(void) const 
+		                                             { return m_bAllowCustomizing;}
+    void                                    setToolbarsCustomizable(bool b);
+    
+	virtual bool 							getDisplayStatus(void) const;
+	virtual bool 							setDisplayStatus(bool b);
 
 protected:
 	void									_setAbiSuiteLibDir(const char * sz);
@@ -161,6 +181,11 @@ protected:
 	UT_Vector								m_vecFrames;
 	UT_StringPtrMap							m_hashClones;
 	XAP_Frame *								m_lastFocussedFrame;
+	XAP_Menu_Factory *                      m_pMenuFactory;
+	XAP_Toolbar_Factory *                   m_pToolbarFactory;
+
+
+	static bool 							s_bShowDisplay;
 
 	struct modeless_pair 
 	{ 
@@ -169,6 +194,7 @@ protected:
 	} m_IdTable[NUM_MODELESSID+1]; 
         
 	static XAP_App *						m_pApp;
+	bool                                    m_bAllowCustomizing;
 };
 
 #endif /* XAP_APP_H */
@@ -177,3 +203,7 @@ protected:
 
 
 
+
+
+
+
