@@ -219,9 +219,21 @@ const char* XAP_EncodingManager::strToNative(const char* in, const char* charset
 {
 	if (!charset || !*charset || !in || !*in || !buf)
 		return in; /*won't translate*/
-
+#if 0
+	// TODO this gets around the fact that gtk 1.2 cannot handle utf-8 input
+	// when we move to gtk 2 this original branch needs to be enabled
 	UT_iconv_t iconv_handle = UT_iconv_open(
 		bUseSysEncoding ? getNativeSystemEncodingName() : getNativeEncodingName(), charset);
+#else
+	UT_iconv_t iconv_handle;
+	const char * pNative =  bUseSysEncoding ? getNativeSystemEncodingName() : getNativeEncodingName();
+	
+	if(!strcmp(pNative, "utf-8") || !strcmp(pNative, "UTF-8"))
+		pNative = getNative8BitEncodingName();
+
+	iconv_handle = UT_iconv_open(pNative, charset);
+	xxx_UT_DEBUGMSG(("xap_EncodingManager::strToNative: pNative %s, iconv_handle 0x%x\n",pNative, iconv_handle));
+#endif
 
 	if (!UT_iconv_isValid(iconv_handle))
 		return in;
@@ -243,12 +255,14 @@ const char* XAP_EncodingManager::strToNative(const char* in, const char* charset
 	return retstr;
 };
 
-#ifndef	HAVE_LIBXML2
-	/*this is used by code that reads xml using expat*/
 int XAP_EncodingManager::XAP_XML_UnknownEncodingHandler(void* /*encodingHandlerData*/,
                                           const XML_Char *name,
                                           XML_Encoding *info)
 {
+#ifdef	HAVE_LIBXML2
+	return 0;
+#else
+	/*this is used by code that reads xml using expat*/
 	if (get_instance()->cjk_locale())
 	    return 0;/*this handler doesn't support multibyte encodings*/
 	UT_iconv_t iconv_handle = UT_iconv_open("UCS-2",name);
@@ -280,8 +294,8 @@ int XAP_EncodingManager::XAP_XML_UnknownEncodingHandler(void* /*encodingHandlerD
 	}
 	UT_iconv_close(iconv_handle);
 	return 1;
-};
 #endif
+};
 
 extern "C" { char *wvLIDToCodePageConverter(unsigned short lid); }
 static void init_values(const XAP_EncodingManager* that)
@@ -464,11 +478,11 @@ static const _map langcode_to_babelarg[]=
 	{NULL,NULL},
 	{"ru","english,russian"},
 	
-	/* I'm not sure that this is correct, but my TeTex 0.9.17 works only 
+	/* I'm not sure that this is correct, but my teTeX 0.9.17 works only 
 	   this way (i.e. only with "russian" in the middle) - hvv */
 	{"uk","english,russian,ukrainian"},
 	
-	/* I'm not sure again - my TeTex 0.9.17 doesn't know 'byelorussian' 
+	/* I'm not sure again - my teTeX 0.9.17 doesn't know 'byelorussian' 
 	   language - hvv */
 	{"be","english,russian"},
 	{NULL,NULL}
@@ -481,7 +495,7 @@ static const _map langcode_to_babelarg[]=
  RUSSIAN_CHARSET).
 */
 static const char* wincharsetcode_ru[]= /* russian charset */
-{ "ru","be", "uk" , NULL };
+{ "ru", "be", "uk" , NULL };
 static const char* wincharsetcode_el[]=  /* greek charset*/
 { "el", NULL };
 
@@ -498,10 +512,10 @@ static const char* wincharsetcode_th[]=  /* thai charset*/
   Tested with GB2312 only.  
 */
 static const char* wincharsetcode_zh_GB2312[]= /* chinese*/
-{ "zh_CN.GB2312", "zh_TW.GB2312", NULL };
+{ "zh_CN.GB2312", "zh_CN.GBK", "zh_CN.GB18030", NULL };
 
 static const char* wincharsetcode_zh_BIG5[]= /* chinese*/
-{ "zh_CN.BIG5", "zh_TW.BIG5", NULL };
+{ "zh_TW.BIG5", "zh_HK.BIG5-HKSCS", NULL };
 
 static const _rmap langcode_to_wincharsetcode[]=
 {
@@ -516,7 +530,7 @@ static const _rmap langcode_to_wincharsetcode[]=
 	{NULL}
 };
 
-static const UT_Bijection::pair_data zh_CN_big5[]=
+static const UT_Bijection::pair_data zh_TW_big5[]=
 {
 /*
     This data was constructed from the HJ's patch for support  of Big5 to 
@@ -529,20 +543,20 @@ static const UT_Bijection::pair_data zh_CN_big5[]=
     {NULL,NULL}
 };
 
-static const char* zh_CN_big5_keys[]=
-{  "zh_CN.BIG5", NULL };
+static const char* zh_TW_big5_keys[]=
+{  "zh_TW.BIG5", NULL };
 
 static const _rmap cjk_word_fontname_mapping_data[]=
 {
     {NULL},
-    {(char*)zh_CN_big5,zh_CN_big5_keys},
+    {(char*)zh_TW_big5,zh_TW_big5_keys},
     {NULL}
 };
 
 
 /*all CJK language codes should be listed here to be marked as CJK*/
 static const char* cjk_languages[]=
-{ "zh","ja","ko",NULL}; 
+{ "zh", "ja", "ko", NULL }; 
 
 static const _rmap langcode_to_cjk[]=
 {
@@ -573,16 +587,16 @@ static const _map MSCodepagename_to_charset_name_map[]=
     {NULL,NULL},
 	// libiconv also lists "SHIFT_JIS", "SHIFT-JIS", "MS_KANJI", "csShiftJIS"
 	{"CP932","SJIS"},
-    {"CP936","GB2312"},
+    {"CP936","GBK"},
     {"CP950","BIG5"},  
 	{"CP1361","JOHAB"},
     {NULL,NULL}
 };
 
 /*
- This table is only concern CJK RTF part.It is a reverse table of
- MSCodepagename_to_charset_name_map.Iconv doesn't know some cpNNNN,
- but M$Word know.
+ This table is only concern CJK RTF part.  It is a reverse table of
+ MSCodepagename_to_charset_name_map.  Iconv doesn't know some cpNNNN,
+ but M$Word knows.
 */
 static const _map charset_name_to_MSCodepagename_map[]=
 {
@@ -591,7 +605,10 @@ static const _map charset_name_to_MSCodepagename_map[]=
 	// libiconv also lists "SHIFT_JIS", "SHIFT-JIS", "MS_KANJI", "csShiftJIS"
 	{"SJIS","CP932"},
     {"GB2312","CP936"},
+    {"GBK","CP936"},
+    {"GB18030","CP936"},
     {"BIG5","CP950"},
+    {"BIG5-HKSCS","CP950"},
 	{"JOHAB","CP1361"},
     {NULL,NULL}
 };
@@ -601,10 +618,11 @@ static const _map langcode_to_winlangcode[]=
 {
 /*key, value*/
     {NULL},
-   {"zh_CN.BIG5",	"0x404"},  
    {"zh_CN.GB2312",	"0x804"},     
+   {"zh_CN.GBK",	"0x804"}, 
+   {"zh_CN.GB18030",	"0x804"}, 
+   {"zh_HK.BIG5-HKSCS",	"0x404"},  
    {"zh_TW.BIG5",	"0x404"},  
-   {"zh_TW.GB2312",	"0x804"}, 
     {NULL}
 };
 
@@ -930,11 +948,12 @@ void XAP_EncodingManager::initialize()
 	{
 	    fontsizes_mapping.clear();
 	    const char** fontsizes = cjk_locale() ? cjk_fontsizes : non_cjk_fontsizes;
-	    char buf[30];
 	    for(const char** cur=fontsizes; *cur; ++cur) 
 		{
-			snprintf(buf, sizeof(buf), " %s ",*cur);
-			fontsizes_mapping.add(*cur, buf);
+		  UT_String buf ( " " );
+		  buf += *cur;
+		  buf += " ";
+		  fontsizes_mapping.add(*cur, buf.c_str());
 	    }
 	}
 	
@@ -988,7 +1007,11 @@ bool XAP_EncodingManager::noncjk_letters(const UT_UCSChar* str,int len) const
 */
 bool XAP_EncodingManager::can_break_at(const UT_UCSChar c) const
 {
-    if (c == UCS_SPACE)
+    if (c == UCS_SPACE 
+	 || c == UCS_MINUS 
+	 || c == UCS_HYPHEN
+	 || c == UCS_EN_DASH
+	 || c == UCS_EM_DASH)
 	return 1;
     return is_cjk_letter(c);
 };
@@ -1044,6 +1067,7 @@ void 	XAP_EncodingManager::describe()
 {
 	UT_DEBUGMSG(("EncodingManager reports the following:\n"
 		"	NativeEncodingName is %s, LanguageISOName is %s,\n"
+		"   Native8BitEncodingName is %s,\n"
 		"	LanguageISOTerritory is %s,  fallbackchar is '%c'\n"		
 		"	TexPrologue follows:\n"
 		"---8<--------------\n" 
@@ -1052,7 +1076,7 @@ void 	XAP_EncodingManager::describe()
 		
 		"	WinLanguageCode is 0x%04x, WinCharsetCode is %d\n"
 		"	cjk_locale %d, can_break_words %d, swap_utos %d, swap_stou %d\n"
-		,getNativeEncodingName(),getLanguageISOName(),
+		,getNativeEncodingName(),getLanguageISOName(),getNative8BitEncodingName(),
 		getLanguageISOTerritory() ? getLanguageISOTerritory() : "NULL",
 		fallbackChar(1072), getTexPrologue(),getWinLanguageCode(),
 		 getWinCharsetCode(),
