@@ -27,7 +27,7 @@
 #include "ut_xml.h"
 #include "ut_iconv.h"
 
-struct XAP_LangInfo
+struct ABI_EXPORT XAP_LangInfo
 {
 	/*no memeber can have NULL value. If string is empty, then value is
 	 not defined. All fields are strings to simplify searches.
@@ -138,7 +138,7 @@ public:
     
     
     virtual char fallbackChar(UT_UCSChar c) const;
-	static XAP_EncodingManager *get_instance();
+	ABI_EXPORT static XAP_EncodingManager *get_instance();
 	void Delete_instance();
 
     /*  This tries to approximate the character with the string, e.g.
@@ -243,12 +243,12 @@ public:
 	 Same as above, but it will use buffer provided by caller.
 	*/
 	const char* strToNative(const char* in,const char* charset,char* buf,int bufsz,bool bUseSysEncoding=false) const;
-#ifndef	HAVE_LIBXML2
+
 	/*this is used by code that reads xml using expat*/
 	static int XAP_XML_UnknownEncodingHandler(void *encodingHandlerData,
                                           const XML_Char *name,
                                           XML_Encoding *info);
-#endif
+
 	/*it's terminated with the record with NULL in language name. */
 	static const XAP_LangInfo		langinfo[];
 	/*
