@@ -39,9 +39,9 @@
 #include "ut_units.h"
 
 fp_Page::fp_Page(FL_DocLayout* pLayout,
-				 FV_View* pView,
-				 const fp_PageSize& pageSize,
-				 fl_DocSectionLayout* pOwner)
+		 FV_View* pView,
+		 const fp_PageSize& pageSize,
+		 fl_DocSectionLayout* pOwner)
 	:	m_pLayout(pLayout),
 		m_pView(pView),
 		m_pNext(0),
@@ -59,8 +59,6 @@ fp_Page::fp_Page(FL_DocLayout* pLayout,
 	UT_ASSERT(pG);
 
 	m_iResolution = pG->getResolution();
-	UT_ASSERT(m_pOwner);
-	m_pOwner->addOwnedPage(this);
 }
 
 fp_Page::~fp_Page()
@@ -79,22 +77,22 @@ bool fp_Page::isEmpty(void) const
 
 UT_sint32 fp_Page::getWidth(void) const
 {
-	return (UT_sint32)(m_iResolution * m_pageSize.Width(fp_PageSize::inch));
+	return (UT_sint32)(m_iResolution * m_pageSize.Width(DIM_IN));
 }
 
 UT_sint32 fp_Page::getWidthInLayoutUnits(void) const
 {
-	return (UT_sint32)m_pageSize.Width(fp_PageSize::LayoutUnit);
+	return (UT_sint32)UT_convertSizeToLayoutUnits(m_pageSize.Width(DIM_IN), DIM_IN);
 }
 
 UT_sint32 fp_Page::getHeight(void) const
 {
-	return (UT_sint32)(m_iResolution * m_pageSize.Height(fp_PageSize::inch));
+	return (UT_sint32)(m_iResolution * m_pageSize.Height(DIM_IN));
 }
 
 UT_sint32 fp_Page::getHeightInLayoutUnits(void) const
 {
-	return (UT_sint32)m_pageSize.Height(fp_PageSize::LayoutUnit);
+	return (UT_sint32)UT_convertSizeToLayoutUnits(m_pageSize.Height(DIM_IN), DIM_IN);
 }
 
 /*!
@@ -180,8 +178,10 @@ UT_sint32 fp_Page::getBottom(void) const
 
 void fp_Page::getScreenOffsets(fp_Container* pContainer, UT_sint32& xoff, UT_sint32& yoff)
 {
-	UT_ASSERT(m_pView);
-	
+	if(!m_pView)
+	{
+	    return;
+	}
 	m_pView->getPageScreenOffsets(this, xoff, yoff);
 
 	xoff += pContainer->getX();
@@ -308,23 +308,24 @@ void fp_Page::draw(dg_DrawArgs* pDA, bool bAlwaysUseWhiteBackground)
 			pTmpCol = pNextCol;
 		}
 	}
-
-	if (m_pHeader)
+    if(m_pView->getViewMode() == VIEW_PRINT) 
 	{
-		dg_DrawArgs da = *pDA;
-  		da.xoff += m_pHeader->getX();
-  		da.yoff += m_pHeader->getY();
-		m_pHeader->draw(&da);
-	}
+		if (m_pHeader)
+		{
+			dg_DrawArgs da = *pDA;
+			da.xoff += m_pHeader->getX();
+			da.yoff += m_pHeader->getY();
+			m_pHeader->draw(&da);
+		}
 	
-	if (m_pFooter)
-	{
-		dg_DrawArgs da = *pDA;
-  		da.xoff += m_pFooter->getX();
-  		da.yoff += m_pFooter->getY();
-		m_pFooter->draw(&da);
+		if (m_pFooter)
+		{
+			dg_DrawArgs da = *pDA;
+			da.xoff += m_pFooter->getX();
+			da.yoff += m_pFooter->getY();
+			m_pFooter->draw(&da);
+		}
 	}
-	
 	m_bNeedsRedraw = false;
 }
 
@@ -552,14 +553,16 @@ void fp_Page::_reformat(void)
 	fl_DocSectionLayout* pFirstSectionLayout = (pFirstColumnLeader->getDocSectionLayout());
 	UT_ASSERT(m_pOwner == pFirstSectionLayout);
 	
-	UT_sint32 iLeftMargin = pFirstSectionLayout->getLeftMargin();
-	UT_sint32 iRightMargin = pFirstSectionLayout->getRightMargin();
+
+	UT_sint32 iLeftMargin = 0;
+	UT_sint32 iRightMargin = 0;
 	UT_sint32 iTopMargin = pFirstSectionLayout->getTopMargin();
 	UT_sint32 iBottomMargin = pFirstSectionLayout->getBottomMargin();
 
 
-	UT_sint32 iLeftMarginLayoutUnits = pFirstSectionLayout->getLeftMarginInLayoutUnits();
-	UT_sint32 iRightMarginLayoutUnits = pFirstSectionLayout->getRightMarginInLayoutUnits();
+	UT_sint32 iLeftMarginLayoutUnits = 0;
+	UT_sint32 iRightMarginLayoutUnits = 0;
+
 	UT_sint32 iTopMarginLayoutUnits = pFirstSectionLayout->getTopMarginInLayoutUnits();
 	UT_sint32 iBottomMarginLayoutUnits = pFirstSectionLayout->getBottomMarginInLayoutUnits();
 	
@@ -579,6 +582,9 @@ void fp_Page::_reformat(void)
 
 		fp_Column* pLeader = getNthColumnLeader(i);
 		fl_DocSectionLayout* pSL = (pLeader->getDocSectionLayout());
+	
+		iLeftMargin = pSL->getLeftMargin();
+		iRightMargin = pSL->getRightMargin();
 
 		UT_uint32 iSpace = getWidth() - iLeftMargin - iRightMargin;
 		pSL->checkAndAdjustColumnGap(iSpace);
@@ -587,6 +593,9 @@ void fp_Page::_reformat(void)
 		UT_uint32 iColumnGap = pSL->getColumnGap();
 		UT_uint32 iColumnGapLayoutUnits = pSL->getColumnGapInLayoutUnits();
 
+		iLeftMarginLayoutUnits = pSL->getLeftMarginInLayoutUnits();
+		iRightMarginLayoutUnits = pSL->getRightMarginInLayoutUnits();
+
 		UT_uint32 iSpaceLayoutUnits = getWidthInLayoutUnits() - iLeftMarginLayoutUnits - iRightMarginLayoutUnits;
 		UT_uint32 iColWidth = (iSpace - ((iNumColumns - 1) * iColumnGap)) / iNumColumns;
 		UT_uint32 iColWidthLayoutUnits = (iSpaceLayoutUnits - ((iNumColumns - 1) * iColumnGapLayoutUnits)) / iNumColumns;
@@ -780,7 +789,7 @@ bool fp_Page::insertColumnLeader(fp_Column* pLeader, fp_Column* pAfter)
 // Change ownership of the page. First remove this page from the set owned by
 // the old docSectionLayout.
 //
-			UT_DEBUGMSG(("SEVIOR: Deleting owned Page from Page \n"));
+			xxx_UT_DEBUGMSG(("SEVIOR: Deleting owned Page from Page \n"));
 			if(m_pOwner)
 				m_pOwner->deleteOwnedPage(this);
 			fl_DocSectionLayout * pDSLNew = pLeader->getDocSectionLayout();
@@ -895,40 +904,43 @@ void fp_Page::mapXYToPositionClick(UT_sint32 x, UT_sint32 y, PT_DocPosition& pos
 // Look in header for insertion point
 //
 	pShadow = NULL;
-	if(m_pHeader != NULL)
+	if(m_pView && m_pView->getViewMode() == VIEW_PRINT)
 	{
-		if (m_pHeader->getFirstLine())
+		if(m_pHeader != NULL)
 		{
-			if (
-				(x >= m_pHeader->getX())
-				&& (x < (m_pHeader->getX() + m_pHeader->getWidth()))
-				&& (y >= m_pHeader->getY())
-				&& (y < (m_pHeader->getY() + m_pHeader->getHeight()))
-				)
+			if (m_pHeader->getFirstLine())
 			{
+				if (
+					(x >= m_pHeader->getX())
+					&& (x < (m_pHeader->getX() + m_pHeader->getWidth()))
+					&& (y >= m_pHeader->getY())
+					&& (y < (m_pHeader->getY() + m_pHeader->getHeight()))
+					)
+				{
 					m_pHeader->mapXYToPosition(x - m_pHeader->getX(), y - m_pHeader->getY(), pos, bBOL, bEOL);
 					pShadow = m_pHeader->getShadow();
 					return;
+				}
 			}
 		}
-	}
 //
 // Look in footer for insertion point
 //
-	if(m_pFooter != NULL)
-	{
-		if (m_pFooter->getFirstLine())
+		if(m_pFooter != NULL)
 		{
-			if (
-				(x >= m_pFooter->getX())
-				&& (x < (m_pFooter->getX() + m_pFooter->getWidth()))
-				&& (y >= m_pFooter->getY())
-				&& (y < (m_pFooter->getY() + m_pFooter->getHeight()))
-				)
+			if (m_pFooter->getFirstLine())
 			{
+				if (
+					(x >= m_pFooter->getX())
+					&& (x < (m_pFooter->getX() + m_pFooter->getWidth()))
+					&& (y >= m_pFooter->getY())
+					&& (y < (m_pFooter->getY() + m_pFooter->getHeight()))
+					)
+				{
 					m_pFooter->mapXYToPosition(x - m_pFooter->getX(), y - m_pFooter->getY(), pos, bBOL, bEOL);
 					pShadow = m_pFooter->getShadow();
 					return;
+				}
 			}
 		}
 	}
@@ -1073,23 +1085,36 @@ const fp_PageSize&	fp_Page::getPageSize(void) const
 	return m_pageSize;
 }
 
-void fp_Page::removeHeader(void)
+void fp_Page::removeHdrFtr(HdrFtrType hfType)
 {
-	UT_DEBUGMSG(("SEVIOR: Deleting header from page m_pHeader = %x \n",m_pHeader));
-	if(m_pHeader == NULL)
-		return;
-	delete m_pHeader;
-	m_pHeader = NULL;
+	UT_ASSERT(hfType >= FL_HDRFTR_HEADER && hfType <= FL_HDRFTR_FOOTER_LAST);
+	if(hfType < FL_HDRFTR_FOOTER)
+	{
+		xxx_UT_DEBUGMSG(("SEVIOR: Deleting header from page %x m_pHeader = %x \n",this, m_pHeader));
+		if(m_pHeader == NULL)
+			return;
+		delete m_pHeader;
+		m_pHeader = NULL;
+	}
+	else
+	{
+		xxx_UT_DEBUGMSG(("SEVIOR: Deleting footer from page %x m_pFooter = %x \n",this,m_pFooter));
+		if(m_pFooter == NULL)
+			return;
+		delete m_pFooter;
+		m_pFooter = NULL;
+	}
 }
-
-
-void fp_Page::removeFooter(void)
+fp_ShadowContainer* fp_Page::getHdrFtrP(HdrFtrType hfType)
 {
-	UT_DEBUGMSG(("SEVIOR: Deleting header from page m_pFooter = %x \n",m_pFooter));
-	if(m_pFooter == NULL)
-		return;
-	delete m_pFooter;
-	m_pFooter = NULL;
+	if(hfType < FL_HDRFTR_FOOTER)
+	{
+		return m_pHeader;
+	}
+	else
+	{
+		return m_pFooter;
+	}
 }
 
 fp_ShadowContainer* fp_Page::getHeaderContainer(fl_HdrFtrSectionLayout* pHFSL)
@@ -1117,6 +1142,33 @@ fp_ShadowContainer* fp_Page::getHeaderContainer(fl_HdrFtrSectionLayout* pHFSL)
 	return m_pHeader;
 }
 
+
+fp_ShadowContainer* fp_Page::buildHeaderContainer(fl_HdrFtrSectionLayout* pHFSL)
+{
+	if (m_pHeader)
+	{
+		UT_ASSERT(UT_SHOULD_NOT_HAPPEN);
+		m_pHeader->getHdrFtrSectionLayout()->deletePage(this);
+	}
+	xxx_UT_DEBUGMSG(("SEVIOR: Building header container. page = %x hdrftr = %x \n",this,pHFSL)); 
+	// TODO fix these coordinates - Done!
+	//
+	// headerMargin is the height from the top of the page.
+	//
+	m_pHeader = new fp_ShadowContainer(m_pOwner->getLeftMargin(),
+									   m_pOwner->getHeaderMargin(),
+									   getWidth() - (m_pOwner->getLeftMargin() + m_pOwner->getRightMargin()),
+									   m_pOwner->getTopMargin() - m_pOwner->getHeaderMargin(),
+									   getWidthInLayoutUnits() - (m_pOwner->getLeftMarginInLayoutUnits() + m_pOwner->getRightMarginInLayoutUnits()),
+									   m_pOwner->getTopMarginInLayoutUnits() - m_pOwner->getHeaderMarginInLayoutUnits(),
+									   pHFSL);
+	// TODO outofmem
+
+	m_pHeader->setPage(this);
+
+	return m_pHeader;
+}
+
 fp_ShadowContainer* fp_Page::getFooterContainer(fl_HdrFtrSectionLayout* pHFSL)
 {
 	if (m_pFooter)
@@ -1142,3 +1194,30 @@ fp_ShadowContainer* fp_Page::getFooterContainer(fl_HdrFtrSectionLayout* pHFSL)
 
 	return m_pFooter;
 }
+
+fp_ShadowContainer* fp_Page::buildFooterContainer(fl_HdrFtrSectionLayout* pHFSL)
+{
+	if (m_pFooter)
+	{
+		UT_ASSERT(UT_SHOULD_NOT_HAPPEN);
+		m_pHeader->getHdrFtrSectionLayout()->deletePage(this);
+	}
+
+	// TODO fix these coordinates -Done !
+	//
+	// footerMargin is the distance from the bottom of the text to the
+	// top of the footer
+	//
+	m_pFooter = new fp_ShadowContainer(m_pOwner->getLeftMargin(),
+									   getHeight() - m_pOwner->getBottomMargin() + m_pOwner->getFooterMargin(),
+									   getWidth() - (m_pOwner->getLeftMargin() + m_pOwner->getRightMargin()),
+									   m_pOwner->getBottomMargin(),
+									   getWidthInLayoutUnits() - (m_pOwner->getLeftMarginInLayoutUnits() + m_pOwner->getRightMarginInLayoutUnits()),
+									   m_pOwner->getBottomMarginInLayoutUnits() - m_pOwner->getFooterMarginInLayoutUnits(),
+									   pHFSL);
+	// TODO outofmem
+
+	m_pFooter->setPage(this);
+
+	return m_pFooter;
+}
