@@ -1,5 +1,6 @@
 /* AbiWord
  * Copyright (C) 1998-2000 AbiSource, Inc.
+ * Copyright (c) 2001,2002 Tomas Frydrych
  * 
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -43,6 +44,7 @@ class fl_BlockLayout;
 class fl_PartOfBlock;
 class fp_Page;
 class fp_Run;
+class fp_HyperlinkRun;
 class FG_Graphic;
 class PD_Document;
 class GR_Graphics;
@@ -64,7 +66,7 @@ typedef enum _FVDocPos
 	FV_DOCPOS_BOP, FV_DOCPOS_EOP,	// page
 	FV_DOCPOS_BOL, FV_DOCPOS_EOL,	// line
 	FV_DOCPOS_BOS, FV_DOCPOS_EOS,	// sentence
-	FV_DOCPOS_BOW, FV_DOCPOS_EOW_MOVE, FV_DOCPOS_EOW_SELECT	// word
+	FV_DOCPOS_BOW, FV_DOCPOS_EOW_MOVE, FV_DOCPOS_EOW_SELECT // word
 } FV_DocPos;
 
 typedef enum _ToggleCase
@@ -73,7 +75,9 @@ typedef enum _ToggleCase
   CASE_LOWER,
   CASE_UPPER,
   CASE_TITLE,
-  CASE_TOGGLE
+  CASE_TOGGLE,
+  CASE_FIRST_CAPITAL,
+  CASE_ROTATE
 } ToggleCase;
 
 typedef enum
@@ -105,8 +109,9 @@ typedef enum
 
 typedef enum _AP_JumpTarget
 {
-	AP_JUMPTARGET_PAGE,				// beginning of page
+	AP_JUMPTARGET_PAGE, 			// beginning of page
 	AP_JUMPTARGET_LINE,
+	AP_JUMPTARGET_BOOKMARK,
 	AP_JUMPTARGET_PICTURE // TODO
 } AP_JumpTarget;
 		
@@ -132,18 +137,19 @@ struct FV_DocCount
 	UT_uint32 page;
 };
 
-class FV_View : public AV_View
+class ABI_EXPORT FV_View : public AV_View
 {
 	friend class fl_DocListener;
 	friend class fl_BlockLayout;
+	friend class fl_Squiggles;
 	friend class fl_DocSectionLayout;
 	
 public:
 	FV_View(XAP_App*, void*, FL_DocLayout*);
 	virtual ~FV_View();
 
-	inline GR_Graphics*		getGraphics(void) const { return m_pG; }
-	inline UT_uint32		getPoint(void) const { return m_iInsPoint; }
+	virtual inline GR_Graphics*    getGraphics(void) const { return m_pG; }
+	virtual inline UT_uint32	  getPoint(void) const { return m_iInsPoint; }
 	inline UT_uint32		getSelectionAnchor(void) const { return m_bSelection? m_iSelectionAnchor : m_iInsPoint; }
 
 	virtual void focusChange(AV_Focus focus);
@@ -151,26 +157,35 @@ public:
 	virtual void	setXScrollOffset(UT_sint32);
 	virtual void	setYScrollOffset(UT_sint32);
 	virtual void	cmdScroll(AV_ScrollCmd cmd, UT_uint32 iPos = 0);
+	
+	virtual void	cmdHyperlinkJump(UT_sint32 xPos, UT_sint32 yPos);
 
 	virtual void	draw(const UT_Rect* pRect=(UT_Rect*) NULL);
 
 	virtual bool	notifyListeners(const AV_ChangeMask hint);
 
 	virtual bool	canDo(bool bUndo) const;
+	virtual UT_uint32 undoCount (bool bUndo) const;
 	virtual void	cmdUndo(UT_uint32 count);
 	virtual void	cmdRedo(UT_uint32 count);
 	virtual UT_Error	cmdSave(void);
 	virtual UT_Error	cmdSaveAs(const char * szFilename, int ieft);
-	virtual UT_Error        cmdSaveAs(const char * szFilename, int ieft, bool cpy);
+	virtual UT_Error		cmdSaveAs(const char * szFilename, int ieft, bool cpy);
 
 	UT_Error		cmdInsertField(const char* szName);
-	UT_Error        cmdInsertField(const char* szName, const XML_Char ** extra_attrs);
+	UT_Error		cmdInsertField(const char* szName, const XML_Char ** extra_attrs);
+	UT_Error		cmdInsertBookmark(const char* szName);
+	UT_Error		cmdDeleteBookmark(const char* szName);
+	UT_Error		cmdInsertHyperlink(const char* szName);
+	UT_Error		cmdDeleteHyperlink();
+	UT_Error		cmdHyperlinkStatusBar(UT_sint32 xPos, UT_sint32 yPos);
 
 	UT_Error		cmdInsertGraphic(FG_Graphic*, const char*);
 
-	virtual void    toggleCase(ToggleCase c);
-	virtual void    setPaperColor(const XML_Char * clr);
+	virtual void	toggleCase(ToggleCase c);
+	virtual void	setPaperColor(const XML_Char * clr);
 
+	virtual bool    isDocumentPresent(void);
 	virtual void	cmdCopy(void);
 	virtual void	cmdCut(void);
 	virtual void	cmdPaste(bool bHonorFormatting = true);
@@ -178,19 +193,24 @@ public:
 
 	virtual void	getTopRulerInfo(AP_TopRulerInfo * pInfo);
 	virtual void	getLeftRulerInfo(AP_LeftRulerInfo * pInfo);
-
+        virtual void    setCursorWait(void);
+	virtual void    clearCursorWait(void);
+	virtual void    setCursorToContext(void);
 	virtual EV_EditMouseContext getMouseContext(UT_sint32 xPos, UT_sint32 yPos);
 	virtual EV_EditMouseContext getInsertionPointContext(UT_sint32 * pxPos, UT_sint32 * pyPos);
-
+	virtual void        updateLayout(void);
 	virtual bool		isSelectionEmpty(void) const;
 	virtual void		cmdUnselectSelection(void);
 	void				getDocumentRangeOfCurrentSelection(PD_DocumentRange * pdr);
 	
+	PT_DocPosition saveSelectedImage (const char * toFile );
+	PT_DocPosition saveSelectedImage (const UT_ByteBuf ** outByteBuf);
+
 // ----------------------
-	FL_DocLayout* 	getLayout() const;
+	FL_DocLayout*	getLayout() const;
 	UT_uint32		getCurrentPageNumForStatusBar(void) const;
 	fp_Page*		getCurrentPage(void) const;
-	fl_BlockLayout*	getCurrentBlock(void);
+	fl_BlockLayout* getCurrentBlock(void);
 
 	void draw(int page, dg_DrawArgs* da);
 
@@ -198,9 +218,9 @@ public:
 	
 	void	getPageScreenOffsets(fp_Page* pPage, UT_sint32& xoff, UT_sint32& yoff);
 	void	getPageYOffset(fp_Page* pPage, UT_sint32& yoff);
-	virtual	UT_sint32 getPageViewLeftMargin(void) const;
-	virtual	UT_sint32 getPageViewTopMargin(void) const;
-	virtual	UT_sint32 getPageViewSep(void) const;
+	virtual UT_sint32 getPageViewLeftMargin(void) const;
+	virtual UT_sint32 getPageViewTopMargin(void) const;
+	virtual UT_sint32 getPageViewSep(void) const;
 	
 	bool	setSectionFormat(const XML_Char * properties[]);
 	bool	getSectionFormat(const XML_Char *** properties);
@@ -213,8 +233,9 @@ public:
 	bool	setBlockFormat(const XML_Char * properties[]);
 	bool	getBlockFormat(const XML_Char *** properties,bool bExpandStyles=true);
 
-	bool    processPageNumber(HdrFtrType hfType, const XML_Char ** atts);
+	bool	processPageNumber(HdrFtrType hfType, const XML_Char ** atts);
 
+	bool	isTextMisspelled()const ;
 	bool	isTabListBehindPoint(void);
 	bool	isTabListAheadPoint(void);
 	void	processSelectedBlocks(List_Type listType);
@@ -240,16 +261,18 @@ public:
 
 	bool	setCharFormat(const XML_Char * properties[]);
 	bool	getCharFormat(const XML_Char *** properties,bool bExpandStyles=true);
+	bool	getCharFormat(const XML_Char *** properties,bool bExpandStyles, PT_DocPosition posStart);
 
+	fl_BlockLayout * getBlockFromSDH(PL_StruxDocHandle sdh);
 	bool	setStyle(const XML_Char * style, bool bDontGeneralUpdate=false);
 	bool	setStyleAtPos(const XML_Char * style, PT_DocPosition posStart, PT_DocPosition posEnd, bool bDontGeneralUpdate=false);
-	
+	bool    isNumberedHeadingHere(fl_BlockLayout * pBlock);
 	bool	getStyle(const XML_Char ** style);
 	bool appendStyle(const XML_Char ** style);
 
 	UT_uint32		getCurrentPageNumber(void);
 
-	bool    getEditableBounds(bool bEnd, PT_DocPosition & docPos, bool bOverride=false);
+	bool	getEditableBounds(bool bEnd, PT_DocPosition & docPos, bool bOverride=false);
 
 	void	insertParagraphBreak(void);
 	void	insertParagraphBreaknoListUpdate(void);
@@ -260,26 +283,35 @@ public:
 	// ----------------------
 	bool			isLeftMargin(UT_sint32 xPos, UT_sint32 yPos);
 	void			cmdSelect(UT_sint32 xPos, UT_sint32 yPos, FV_DocPos dpBeg, FV_DocPos dpEnd);
-	void                    cmdSelect(PT_DocPosition dpBeg, PT_DocPosition dpEnd);
+	void					cmdSelect(PT_DocPosition dpBeg, PT_DocPosition dpEnd);
 	void			cmdCharMotion(bool bForward, UT_uint32 count);
 	bool			cmdCharInsert(UT_UCSChar * text, UT_uint32 count, bool bForce = false);
 	void			cmdCharDelete(bool bForward, UT_uint32 count);
 	void			delTo(FV_DocPos dp);
-	UT_UCSChar * 	getSelectionText(void);
-		
+	UT_UCSChar *	getSelectionText(void);
+#if 0	
+	UT_UCSChar *	getTextBetweenPos(PT_DocPosition pos1, PT_DocPosition pos2);
+#endif
+	inline PT_DocPosition  getInsPoint () const { return m_iInsPoint; }
 	void			warpInsPtToXY(UT_sint32 xPos, UT_sint32 yPos, bool bClick);
-	void			moveInsPtTo(FV_DocPos dp);
-	void 			moveInsPtTo(PT_DocPosition dp);
+	void			moveInsPtTo(FV_DocPos dp, bool bClearSelection = true);
+	void			moveInsPtTo(PT_DocPosition dp);
 	void			warpInsPtNextPrevPage(bool bNext);
 	void			warpInsPtNextPrevLine(bool bNext);
+	void            warpInsPtNextPrevScreen(bool bNext);
 	void			extSelHorizontal(bool bForward, UT_uint32 count);
 	void			extSelToXY(UT_sint32 xPos, UT_sint32 yPos, bool bDrag);
 	void			extSelToXYword(UT_sint32 xPos, UT_sint32 yPos, bool bDrag);
 	void			extSelTo(FV_DocPos dp);
 	void			extSelNextPrevLine(bool bNext);
+	void            extSelNextPrevPage(bool bNext);
+	void            extSelNextPrevScreen(bool bNext);
 	void			endDrag(UT_sint32 xPos, UT_sint32 yPos);
-
-	void			updateScreen(bool bDirtyRunsOnly=true);
+	PT_DocPosition  getDocPositionFromXY(UT_sint32 xpos, UT_sint32 ypos);
+	PT_DocPosition  getDocPositionFromLastXY(void);
+	
+	fl_BlockLayout* getBlockAtPosition(PT_DocPosition pos) const {return _findBlockAtPosition(pos);};
+	virtual void	updateScreen(bool bDirtyRunsOnly=true);
 
 
 // ----------------------
@@ -294,35 +326,43 @@ public:
 // ----------------------
 // Stuff for edittable Headers/Footers
 //
-	void                setHdrFtrEdit(fl_HdrFtrShadow * pShadow);
-	void                clearHdrFtrEdit(void);
-	bool                isHdrFtrEdit(void);
-	fl_HdrFtrShadow *   getEditShadow(void);
-    void                rememberCurrentPosition(void);
-	PT_DocPosition      getSavedPosition(void);
-	void                clearSavedPosition(void);
-	void                markSavedPositionAsNeeded(void);
-	bool                needSavedPosition(void);
-	void                insertHeaderFooter(HdrFtrType hfType);
-	bool				insertHeaderFooter(const XML_Char ** props, HdrFtrType hfType);
+	void				setHdrFtrEdit(fl_HdrFtrShadow * pShadow);
+	void				clearHdrFtrEdit(void);
+	bool				isHdrFtrEdit(void);
+	fl_HdrFtrShadow *	getEditShadow(void);
+	void				rememberCurrentPosition(void);
+	PT_DocPosition		getSavedPosition(void);
+	void				clearSavedPosition(void);
+	void				markSavedPositionAsNeeded(void);
+	bool				needSavedPosition(void);
+	void				insertHeaderFooter(HdrFtrType hfType);
+	bool				insertHeaderFooter(const XML_Char ** props, HdrFtrType hfType, fl_DocSectionLayout * pDSL=NULL);
 
-	void                cmdEditHeader(void);
-	void                cmdEditFooter(void);
+	void				cmdEditHeader(void);
+	void				cmdEditFooter(void);
 
 	void                cmdRemoveHdrFtr(bool isHeader);
 	bool                isFooterOnPage(void);
 	bool                isHeaderOnPage(void);
 
+	void                SetupSavePieceTableState(void);
+	void                RestoreSavedPieceTableState(void);
+    void                removeThisHdrFtr(HdrFtrType hfType, bool bSkipPTSaves = false);
+	void                createThisHdrFtr(HdrFtrType hfType, bool bSkipPTSaves = false);
+	void                populateThisHdrFtr(HdrFtrType hfType, bool bSkipPTSaves = false);
+	void                _populateThisHdrFtr(fl_HdrFtrSectionLayout * pHdrFtrSrc, fl_HdrFtrSectionLayout * pHdrFtrDest);
+
+//
 // ----------------------
 // Stuff for edittable endnotes
 //
 	bool	insertEndnote();
 	bool	insertEndnoteSection(const XML_Char * enpid);
-	bool    insertEndnoteSection(const XML_Char ** blkprops, const XML_Char ** blkattrs);
+	bool	insertEndnoteSection(const XML_Char ** blkprops, const XML_Char ** blkattrs);
 
 // ----------------------
 
-	bool 		gotoTarget(AP_JumpTarget type, UT_UCSChar * data);
+	bool		gotoTarget(AP_JumpTarget type, UT_UCSChar * data);
 
 	void			changeNumColumns(UT_uint32 iNumColumns);
 	
@@ -330,27 +370,33 @@ public:
 
 	// find and replace
 	
-	// aid the edit method for the simple non-dialog findAgain()
-	bool 		findSetNextString(UT_UCSChar * string, bool matchCase);
+	bool			findSetNextString(UT_UCSChar* string, bool bMatchCase);
 	bool			findAgain(void);
 
-	void 			findSetStartAtInsPoint(void);
+	void			findSetStartAtInsPoint(void);
 
-	// finds the next "find" and selects it, filling bool when done the entire document
-	bool			findNext(const UT_UCSChar * find, bool matchCase = true, bool * bDoneEntireDocument = NULL);
-	bool			_findNext(const UT_UCSChar * find, bool matchCase = true, bool * bDoneEntireDocument = NULL);
-	bool			_findNext(const UT_UCSChar * find, UT_uint32 *prefix, bool matchCase = true, bool * bDoneEntireDocument = NULL);
-	// replaces the selection of "find" with "replace" and selects the next, filling
-	// bool when done the entire document
-	bool			_findReplace(const UT_UCSChar * find, const UT_UCSChar * replace,
-								 bool matchCase = false, bool * bDoneEntireDocument = NULL);
-	bool			_findReplace(const UT_UCSChar * find, const UT_UCSChar * replace, UT_uint32 *prefix,
-								 bool matchCase = false, bool * bDoneEntireDocument = NULL);
-	bool			findReplace(const UT_UCSChar * find, const UT_UCSChar * replace,
-								bool matchCase = false, bool * bDoneEntireDocument = NULL);
-	// replaces every occurance of "find" with "replace" without stopping for anything
-	UT_uint32		findReplaceAll(const UT_UCSChar * find, const UT_UCSChar * replace,
-								   bool matchCase = false);
+	bool			findNext(const UT_UCSChar* pFind, bool bMatchCase,
+							 bool& bDoneEntireDocument);
+	UT_uint32*		_computeFindPrefix(const UT_UCSChar* pFind,
+									   bool bMatchCase);
+	bool			_findNext(const UT_UCSChar* pFind, 
+							  UT_uint32* pPrefix,
+							  bool bMatchCase,
+							  bool& bDoneEntireDocument);
+
+	bool			_findReplace(const UT_UCSChar* pFind,
+								 const UT_UCSChar* pReplace, 
+								 UT_uint32* pPrefix,
+								 bool bMatchCase, 
+								 bool& bDoneEntireDocument);
+	bool			findReplace(const UT_UCSChar* pFind,
+								const UT_UCSChar* pReplace,
+								bool bMatchCase, 
+								bool& bDoneEntireDocument);
+
+	UT_uint32		findReplaceAll(const UT_UCSChar* pFind,
+								   const UT_UCSChar* pReplace,
+								   bool bMatchCase);
 		
 // ----------------------
 
@@ -360,7 +406,7 @@ public:
 
 // ----------------------
 
-	FV_DocCount			countWords(void);
+	FV_DocCount 		countWords(void);
 
 // -----------------------
 
@@ -370,37 +416,38 @@ public:
 // -----------------------
 
 	void				setShowPara(bool);
-	inline bool		getShowPara(void) const { return m_bShowPara; };
+	inline bool 	getShowPara(void) const { return m_bShowPara; };
 	
 	const fp_PageSize&	getPageSize(void) const;
 	UT_uint32			calculateZoomPercentForPageWidth();
 	UT_uint32			calculateZoomPercentForPageHeight();
 	UT_uint32			calculateZoomPercentForWholePage();
-	inline void             setViewMode (ViewMode vm) {m_viewMode = vm;}
-	inline ViewMode         getViewMode (void) const  {return m_viewMode;}
-	bool                isPreview(void) const {return VIEW_PREVIEW == m_viewMode;}
-	void                setPreviewMode(PreViewMode pre) {m_previewMode = pre;}
-	PreViewMode         getPreviewMode(void) { return m_previewMode;}
+	inline void 			setViewMode (ViewMode vm) {m_viewMode = vm;}
+	inline ViewMode 		getViewMode (void) const  {return m_viewMode;}
+	bool				isPreview(void) const {return VIEW_PREVIEW == m_viewMode;}
+	void				setPreviewMode(PreViewMode pre) {m_previewMode = pre;}
+	PreViewMode 		getPreviewMode(void) { return m_previewMode;}
 
-	void                setScreenUpdateOnGeneralUpdate( bool bDoit) 
+	void				setScreenUpdateOnGeneralUpdate( bool bDoit) 
 		{m_bDontUpdateScreenOnGeneralUpdate = !bDoit;}
-	bool                shouldScreenUpdateOnGeneralUpdate(void) const 
+	bool				shouldScreenUpdateOnGeneralUpdate(void) const 
 		{ return !m_bDontUpdateScreenOnGeneralUpdate;}
 
 	inline PD_Document * getDocument (void) const {return m_pDoc;}
 
 protected:
-	void                _saveAndNotifyPieceTableChange(void);
-	void                _restorePieceTableState(void);
+	void				_saveAndNotifyPieceTableChange(void);
+	void				_restorePieceTableState(void);
 	void				_generalUpdate(void);
 	
-	void 				_draw(UT_sint32, UT_sint32, UT_sint32, UT_sint32, bool bDirtyRunsOnly, bool bClip=false);
+	void				_draw(UT_sint32, UT_sint32, UT_sint32, UT_sint32, bool bDirtyRunsOnly, bool bClip=false);
 	
 	void				_drawBetweenPositions(PT_DocPosition left, PT_DocPosition right);
 	bool				_clearBetweenPositions(PT_DocPosition left, PT_DocPosition right, bool bFullLineHeightRect);
 	
 	bool				_ensureThatInsertionPointIsOnScreen(bool bDrawIP = true);
 	void				_moveInsPtNextPrevPage(bool bNext);
+	void				_moveInsPtNextPrevScreen(bool bNext);
 	void				_moveInsPtNextPrevLine(bool bNext);
 	fp_Page *			_getCurrentPage(void);
 	void				_moveInsPtNthPage(UT_uint32 n);
@@ -409,12 +456,12 @@ protected:
 
 	PT_DocPosition		_getDocPosFromPoint(PT_DocPosition iPoint, FV_DocPos dp, bool bKeepLooking=true);
 	PT_DocPosition		_getDocPos(FV_DocPos dp, bool bKeepLooking=true);
-	void 				_findPositionCoords(PT_DocPosition pos,
+	void				_findPositionCoords(PT_DocPosition pos,
 											bool b,
 											UT_sint32& x,
 											UT_sint32& y,
 											UT_sint32& x2, //these are needed for BiDi split carret
-	  										UT_sint32& y2,
+											UT_sint32& y2,
 
 											UT_uint32& height,
 											bool& bDirection,
@@ -435,11 +482,11 @@ protected:
 	void				_setSelectionAnchor(void);
 	void				_deleteSelection(PP_AttrProp *p_AttrProp_Before = NULL);
 	bool				_insertFormatPair(const XML_Char * szName, const XML_Char * properties[]);
-	void 				_eraseInsertionPoint();
+	void				_eraseInsertionPoint();
 	void				_drawInsertionPoint();
-	void 				_updateInsertionPoint();
+	void				_updateInsertionPoint();
 	void				_fixInsertionPointCoords();
-	void 				_xorInsertionPoint();
+	void				_xorInsertionPoint();
 	bool				_hasPointMoved(void);
 	void				_saveCurrentPoint(void); 
 	void				_clearOldPoint(void); 
@@ -451,8 +498,8 @@ protected:
 
 	UT_UCSChar *		_lookupSuggestion(fl_BlockLayout* pBL, fl_PartOfBlock* pPOB, UT_uint32 ndx);
 
-	static void			_autoScroll(UT_Worker * pTimer);
-	static void			_autoDrawPoint(UT_Worker * pTimer);
+	static void 		_autoScroll(UT_Worker * pTimer);
+	static void 		_autoDrawPoint(UT_Worker * pTimer);
 
 	// localize handling of insertion point logic
 	void				_setPoint(PT_DocPosition pt, bool bEOL = false);
@@ -463,8 +510,13 @@ protected:
 
 	void				_checkPendingWordForSpell(void);
 
-    bool                _isSpaceBefore(PT_DocPosition pos);
+	bool				_isSpaceBefore(PT_DocPosition pos);
+	void				_removeThisHdrFtr(fl_HdrFtrSectionLayout * pHdrFtr);
 
+	UT_Error			_deleteBookmark(const char* szName, bool bSignal, PT_DocPosition &i, PT_DocPosition &j);
+	UT_Error			_deleteHyperlink(PT_DocPosition &i, bool bSignal);	
+	fp_HyperlinkRun *   _getHyperlinkInRange(PT_DocPosition &posStart,
+											 PT_DocPosition &posEnd);
 	PT_DocPosition		m_iInsPoint;
 	UT_sint32			m_xPoint;
 	UT_sint32			m_yPoint;
@@ -474,16 +526,17 @@ protected:
 	UT_sint32			m_yPoint2;
 	UT_sint32			m_oldxPoint2;
 	UT_sint32			m_oldyPoint2;
-	bool             m_bPointDirection;
+	bool			 m_bPointDirection;
 
 #ifdef BIDI_ENABLED
 	bool				m_bDefaultDirectionRtl;
+	bool				m_bUseHebrewContextGlyphs;
 #endif
 	UT_uint32			m_iPointHeight;
 	UT_sint32			m_oldxPoint;
 	UT_sint32			m_oldyPoint;
 	UT_uint32			m_oldiPointHeight;
-	UT_sint32			m_xPointSticky;		// used only for _moveInsPtNextPrevLine()
+	UT_sint32			m_xPointSticky; 	// used only for _moveInsPtNextPrevLine()
 
 	bool				m_bPointVisible;
 	bool				m_bPointEOL;
@@ -516,36 +569,36 @@ protected:
 
 	bool				m_doneFind;
 
-	bool                m_bEditHdrFtr;
-	fl_HdrFtrShadow *   m_pEditShadow;
-	PT_DocPosition      m_iSavedPosition;
-	bool                m_bNeedSavedPosition;
-	PT_DocPosition 		_BlockOffsetToPos(fl_BlockLayout * block, PT_DocPosition offset);
+	bool				m_bEditHdrFtr;
+	fl_HdrFtrShadow *	m_pEditShadow;
+	PT_DocPosition		m_iSavedPosition;
+	bool				m_bNeedSavedPosition;
+	PT_DocPosition		_BlockOffsetToPos(fl_BlockLayout * block, PT_DocPosition offset);
 	
-	fl_BlockLayout * 	_findGetCurrentBlock(void);
-	PT_DocPosition	 	_findGetCurrentOffset(void);	
-	UT_UCSChar * 		_findGetNextBlockBuffer(fl_BlockLayout ** block, PT_DocPosition *offset);
+	fl_BlockLayout *	_findGetCurrentBlock(void);
+	PT_DocPosition		_findGetCurrentOffset(void);	
+	UT_UCSChar *		_findGetNextBlockBuffer(fl_BlockLayout ** block, PT_DocPosition *offset);
 
 	bool				_m_matchCase;
-	UT_UCSChar * 		_m_findNextString;
+	UT_UCSChar *		_m_findNextString;
 
-	// search routines (these return values will fall short of an
-	// extremely large document - fix them)
-	UT_sint32 			_findBlockSearchDumbCase(const UT_UCSChar * haystack, const UT_UCSChar * needle);
-	UT_sint32 			_findBlockSearchDumbNoCase(const UT_UCSChar * haystack, const UT_UCSChar * needle);	
 	UT_sint32			_findBlockSearchRegexp(const UT_UCSChar * haystack, const UT_UCSChar * needle);
 
 	// prefs listener - to change cursor blink on/off (and possibly others)
 	static void _prefsListener( XAP_App *, XAP_Prefs *, UT_StringPtrMap *, void *);
 
-	bool		         m_bShowPara;
-	ViewMode             m_viewMode;
-	PreViewMode          m_previewMode;
-	bool                 m_bDontUpdateScreenOnGeneralUpdate; 
+	bool				 m_bShowPara;
+	ViewMode			 m_viewMode;
+	PreViewMode 		 m_previewMode;
+	bool				 m_bDontUpdateScreenOnGeneralUpdate; 
 	//#TF had to change the whole logic of storing PT state, since
 	//the earlier implementation did not work with nested calls to
 	//_saveAndNotifyPieceTableChange();
 	UT_uint32			m_iPieceTableState;
+
+	UT_sint32           m_iMouseX;
+	UT_sint32           m_iMouseY;
 };
 
 #endif /* FV_VIEW_H */
+
