@@ -343,7 +343,7 @@ bool pt_PieceTable::changeSpanFmt(PTChangeFmt ptc,
 // Finally make the const XML_Char * array of properties
 //
 		UT_uint32 countp = vProps.getItemCount() + 1;
-		sProps = (const XML_Char **) calloc(countp, sizeof(XML_Char *));
+		sProps = (const XML_Char **) UT_calloc(countp, sizeof(XML_Char *));
 		countp--;
 		UT_uint32 i;
 		for(i=0; i<countp; i++)
@@ -355,11 +355,13 @@ bool pt_PieceTable::changeSpanFmt(PTChangeFmt ptc,
 	}
 	if (dpos1 == dpos2) 		// if length of change is zero, then we have a toggle format.
 	{
+		UT_uint32 startUndoPos = m_history.getUndoPos();
 		bool bRes = _insertFmtMarkFragWithNotify(ptc,dpos1,attributes,lProps);
+		UT_uint32 endUndoPos = m_history.getUndoPos();
 		// Won't be a persistant change if it's just a toggle
 		PX_ChangeRecord *pcr=0;
 		m_history.getUndo(&pcr);
-		if (pcr)
+		if (pcr && (startUndoPos != endUndoPos) )
 		{
 			UT_DEBUGMSG(("Setting persistance of change to false\n"));
 			pcr->setPersistance(false);
