@@ -1,5 +1,6 @@
 /* AbiWord
  * Copyright (C) 1998 AbiSource, Inc.
+ * Copyright (C) 2001,2002 Tomas Frydrych
  * 
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -301,6 +302,7 @@ bool pt_PieceTable::_tweakDeleteSpanOnce(PT_DocPosition & dpos1,
 		return false;
 		
 	case PTX_Section:
+#if 0
 		// if the previous container is a section, then pf_First
 		// must be the first block in the section.
 		UT_ASSERT((pf_First->getPrev() == pfsContainer));
@@ -312,7 +314,8 @@ bool pt_PieceTable::_tweakDeleteSpanOnce(PT_DocPosition & dpos1,
 		// section.
 		dpos1 -= pfsContainer->getLength();
 		return true;
-
+#endif
+	case PTX_SectionEndnote:
 	case PTX_SectionHdrFtr:
 		// if the previous container is a Header/Footersection, then pf_First
 		// must be the first block in the section.
@@ -396,7 +399,6 @@ bool pt_PieceTable::_tweakDeleteSpanOnce(PT_DocPosition & dpos1,
                 //  if we delete a whole lot of blank lines. These get popped off then deleted
                 //  only to find the same pointer waiting to come off the stack.
 				pf_Frag * pScan = pf_Before->getNext();
-				UT_uint32 count = 0;
 				while(pScan && pScan != pf_Last && (pScan->getType() != pf_Frag::PFT_Strux))
 				{
 					pScan = pScan->getNext();
@@ -574,13 +576,570 @@ bool pt_PieceTable::_deleteComplexSpan(PT_DocPosition dpos1,
 			UT_ASSERT(bResult);
 		}
 		break;
-
+		// the bookmark and hyperlink objects require special treatment; since
+		// they come in pairs, we have to ensure that both are deleted together
+		// so we have to find the other part of the pair, delete it, adjust the
+		// positional variables and the delete the one we were originally asked
+		// to delete
 		case pf_Frag::PFT_Object:
 		{
-			bool bResult
-				= _deleteObjectWithNotify(dpos1,static_cast<pf_Frag_Object *>(pf_First),
-										  fragOffset_First,lengthThisStep,
+			bool bResult, bResult2;
+			pf_Frag_Object *pO = static_cast<pf_Frag_Object *>(pf_First);
+			switch(pO->getObjectType())
+			{
+				case PTO_Bookmark:
+				{
+					bool bFoundStrux2;
+					PT_DocPosition posComrade;
+					pf_Frag_Strux * pfsContainer2 = NULL;
+					
+					po_Bookmark * pB = pO->getBookmark();
+					UT_ASSERT(pB);
+					pf_Frag * pF;
+					if(pB->getBookmarkType() == po_Bookmark::POBOOKMARK_END)
+					{
+				    	pF = pO->getPrev();
+				    	while(pF)
+				    	{
+							if(pF->getType() == pf_Frag::PFT_Object)
+							{
+								pf_Frag_Object *pOb = static_cast<pf_Frag_Object*>(pF);
+								po_Bookmark * pB1 = pOb->getBookmark();
+								if(!strcmp(pB->getName(),pB1->getName()))
+								{
+									
+									m_pDocument->removeBookmark(pB1->getName());
+									
+									posComrade = getFragPosition(pOb);
+									bFoundStrux2 = _getStruxFromPosition(posComrade,&pfsContainer2);
+									UT_ASSERT(bFoundStrux2);
+									
+									bResult2 =
+											_deleteObjectWithNotify(posComrade,pOb,0,1,
+										  							pfsContainer2,0,0);
+									
+									// now adjusting the positional variables
+									if(posComrade <= dpos1)
+										// delete before the start of the segement we are working on
+										dpos1--;
+									else
+									{
+										// we are inside that section
+										length--;
+										
+									}
+									break;
+								}
+							}
+							pF = pF->getPrev();
+				    	}
+					}
+					else
+					{
+				    	pF = pO->getNext();
+				    	while(pF)
+				    	{
+							if(pF->getType() == pf_Frag::PFT_Object)
+							{
+								pf_Frag_Object *pOb = static_cast<pf_Frag_Object*>(pF);
+								po_Bookmark * pB1 = pOb->getBookmark();
+								if(!strcmp(pB->getName(),pB1->getName()))
+								{
+									m_pDocument->removeBookmark(pB1->getName());
+								
+									posComrade = getFragPosition(pOb);
+									bool bFoundStrux2 = _getStruxFromPosition(posComrade,&pfsContainer2);
+									UT_ASSERT(bFoundStrux2);
+									
+									bResult2 =
+											_deleteObjectWithNotify(posComrade,pOb,0,1,
+										  							pfsContainer2,0,0);
+									if(posComrade < dpos1 + length)
+										length--;
+									break;
+								}
+							}
+							pF = pF->getNext();
+				    	}
+					}
+				bResult
+					= _deleteObjectWithNotify(dpos1,pO,fragOffset_First,lengthThisStep,
+									  pfsContainer,&pfNewEnd,&fragOffsetNewEnd);
+					
+				}	
+				break;
+				// the one singnificant difference compared to the bookmarks is
+				// that we have to always delete the start marker first; this is
+				// so that in the case of undo the endmarker would be restored
+				// first, because the mechanism that marks runs between them
+				// as a hyperlink depents on the end-marker being in place before
+				// the start marker
+				case PTO_Hyperlink:
+				{
+					bool bFoundStrux2;
+					PT_DocPosition posComrade;
+					pf_Frag_Strux * pfsContainer2 = NULL;
+					
+					pf_Frag * pF;
+		
+		    		const PP_AttrProp * pAP = NULL;
+				    pO->getPieceTable()->getAttrProp(pO->getIndexAP(),&pAP);
+				    UT_ASSERT(pAP);
+				    const XML_Char* pszHref = NULL;
+				    const XML_Char* pszHname  = NULL;
+		            UT_uint32 k = 0;
+        		    bool bStart = false;
+				    while((pAP)->getNthAttribute(k++,pszHname, pszHref))
+				    {
+		    			if(!UT_strcmp(pszHname, "xlink:href"))
+				    	{
+				    		bStart = true;
+		    				break;
+				    	}
+				    }
+					
+					if(!bStart)
+					{
+						// in this case we are looking for the start marker
+						// and so we delete it and then move on
+				    	pF = pO->getPrev();
+				    	while(pF)
+				    	{
+							if(pF->getType() == pf_Frag::PFT_Object)
+							{
+								pf_Frag_Object *pOb = static_cast<pf_Frag_Object*>(pF);
+								if(pOb->getObjectType() == PTO_Hyperlink)
+								{
+									posComrade = getFragPosition(pOb);
+									bFoundStrux2 = _getStruxFromPosition(posComrade,&pfsContainer2);
+									UT_ASSERT(bFoundStrux2);
+									
+									bResult2 =
+											_deleteObjectWithNotify(posComrade,pOb,0,1,
+										  							pfsContainer2,0,0);
+										
+									// now adjusting the positional variables
+									if(posComrade <= dpos1)
+										// delete before the start of the segement we are working on
+										dpos1--;
+									else
+									{
+										// we are inside that section
+										length--;
+									}
+									break;
+								}
+							}
+							pF = pF->getPrev();
+				    	}
+						bResult
+							= _deleteObjectWithNotify(dpos1,pO,fragOffset_First,lengthThisStep,
 										  pfsContainer,&pfNewEnd,&fragOffsetNewEnd);
+				    	
+					}
+					else
+					{
+						// in this case we are looking for the end marker,
+						// so we have to be carefult the get rid of the start
+						// marker first
+				    	pF = pO->getNext();
+				    	while(pF)
+				    	{
+							if(pF->getType() == pf_Frag::PFT_Object)
+							{
+								pf_Frag_Object *pOb = static_cast<pf_Frag_Object*>(pF);
+								if(pOb->getObjectType() == PTO_Hyperlink)
+								{
+									posComrade = getFragPosition(pOb);
+									bFoundStrux2 = _getStruxFromPosition(posComrade,&pfsContainer2);
+									UT_ASSERT(bFoundStrux2);
+					                // delete the original start marker
+									bResult
+										= _deleteObjectWithNotify(dpos1,pO,fragOffset_First,lengthThisStep,
+													  pfsContainer,&pfNewEnd,&fragOffsetNewEnd);
+									
+									// now adjusting the positional variables
+									posComrade--;
+
+									bResult2 =
+											_deleteObjectWithNotify(posComrade,pOb,0,1,
+									  							pfsContainer2,0,0);
+
+									if(posComrade >= dpos1 && posComrade <= dpos1 + length - 2)
+									{
+										// the endmarker was inside of the segment we are working
+										// so we have to adjust the length
+										length--;
+									}
+									
+									break;
+								}
+							}
+							pF = pF->getNext();
+				    	}
+					}
+				}	
+				break;
+				
+				case PTO_Field:
+				{
+					// when deleting endnote reference, have to delete the
+					// the associated stuff in the endnote section as well
+					if(pO->getField()->getFieldType() == fd_Field::FD_Endnote_Ref)
+					{
+			    		const PP_AttrProp * pAP = NULL;
+					    pO->getPieceTable()->getAttrProp(pO->getIndexAP(),&pAP);
+					    UT_ASSERT(pAP);
+					    const XML_Char* pszEid = NULL;
+					    const XML_Char* pszEidName  = NULL;
+			            UT_uint32 k = 0;
+    	    		    bool bFound = false;
+
+    	    		    pf_Frag_Strux *pMySection, *pESection;
+    	    		    					
+					    while((pAP)->getNthAttribute(k++,pszEidName, pszEid))
+					    {
+		    				if(!UT_strcmp(pszEidName, "endnote-id"))
+				    		{
+				    			bFound = true;
+		    					break;
+					    	}
+					    }
+					
+					    UT_ASSERT(bFound);
+											
+						pf_Frag_Strux * pfsContainer = NULL;
+						pf_Frag * pF;
+
+						// in theory, we should find our section, from
+						// there get the id of the endnote section, find the endnote
+						// section and then delete our block
+						// however, to get to the section we have to scroll through
+						// the fragments one by one, so we might just as well not
+						// bother and test for the endnote-id directly
+						// we know that the endnote section is definitely after this
+						// fragment, so we will start with the next one
+						
+						// on further thought we will bother
+					
+					    const XML_Char* pszEnote = NULL;
+						
+						pF = pO->getPrev();
+						while(pF)
+						{
+							if(pF->getType() == pf_Frag::PFT_Strux)
+							{
+								pfsContainer = static_cast<pf_Frag_Strux *>(pF);
+								if(pfsContainer->getStruxType() == PTX_Section)
+								{
+								    pfsContainer->getPieceTable()->getAttrProp(pfsContainer->getIndexAP(),&pAP);
+								    UT_ASSERT(pAP);
+								    const XML_Char* pszEnoteName  = NULL;
+						            k = 0;
+    	    		    			bFound = false;
+					
+								    while((pAP)->getNthAttribute(k++,pszEnoteName, pszEnote))
+								    {
+		    							if(!UT_strcmp(pszEnoteName, "endnote"))
+							    		{
+						    				bFound = true;
+		    								break;
+		    							}
+					    			}
+					    			
+					    			UT_ASSERT(bFound);
+					    			break;
+					   			}
+					   		}
+					   		pF = pF->getPrev();
+					    }
+					
+					    pMySection = static_cast<pf_Frag_Strux*>(pF);
+					
+					    // now that we have got our endnote section id
+					    // we will find it
+					
+				    	pF = pO->getNext();
+				    	while(pF)
+				    	{
+							if(pF->getType() == pf_Frag::PFT_Strux)
+							{
+								pfsContainer = static_cast<pf_Frag_Strux *>(pF);
+								if(pfsContainer->getStruxType() == PTX_SectionEndnote)
+								{
+								    pfsContainer->getPieceTable()->getAttrProp(pfsContainer->getIndexAP(),&pAP);
+								    UT_ASSERT(pAP);
+								    const XML_Char* pszId = NULL;
+								    const XML_Char* pszIdName  = NULL;
+						            k = 0;
+    	    		    			bFound = false;
+					
+								    while((pAP)->getNthAttribute(k++,pszIdName, pszId))
+								    {
+		    							if(!UT_strcmp(pszIdName, "id"))
+							    		{
+							    			if(!UT_strcmp(pszId, pszEnote))
+							    			{
+						    					bFound = true;
+		    									break;
+		    								}
+		    							}
+					    			}
+									if(bFound)					    			
+					    				break;
+					   			}
+					   		}
+					   		pF = pF->getNext();
+				    	
+				    	}
+				    	
+				    	// we should have our endnote section now
+				    	UT_ASSERT(pF);
+				    	
+				    	pESection = static_cast<pf_Frag_Strux*>(pF);
+				    	
+				    	// lets get deleting ...
+				    	pF = pF->getNext();
+				    	UT_ASSERT(pF);
+				    	
+				    	pf_Frag_Strux * pFirstBlock = NULL;
+				    	
+				    	while(pF)
+				    	{
+				    		// first check we are still in our section
+							if(pF->getType() == pf_Frag::PFT_Strux)
+							{
+								pfsContainer = static_cast<pf_Frag_Strux *>(pF);
+								if(pfsContainer->getStruxType() != PTX_Block)
+								{
+									UT_DEBUGMSG(("non-block strux\n"));
+									break;
+								}
+							}
+							
+							PT_AttrPropIndex iAPI = 0;
+							bool bHasIndex = true;
+							
+							switch(pF->getType())
+							{
+								case pf_Frag::PFT_Strux:
+									{
+										pf_Frag_Strux * pFS = static_cast<pf_Frag_Strux *>(pF);
+										iAPI = pFS->getIndexAP();
+									}
+									break;
+								case pf_Frag::PFT_Object:
+									{
+										pf_Frag_Object * pFO = static_cast<pf_Frag_Object *>(pF);
+										iAPI = pFO->getIndexAP();
+									}
+									break;
+								case pf_Frag::PFT_Text:
+									{
+										pf_Frag_Text * pFT = static_cast<pf_Frag_Text *>(pF);
+										iAPI = pFT->getIndexAP();
+									}
+									break;
+								case pf_Frag::PFT_FmtMark:
+									// formating marks are real pain and we
+									// we delete any we find in this section
+									// without mercy
+									{
+										UT_DEBUGMSG(("Deleting FmtMark\n"));
+										pf_Frag_FmtMark * pFM = static_cast<pf_Frag_FmtMark *>(pF);
+										pF = pF->getPrev();
+			
+										pf_Frag_Strux * pfsContainerTemp = NULL;
+										bool bFoundStrux = _getStruxFromPosition(pFM->getPos(),&pfsContainerTemp);
+										UT_ASSERT(bFoundStrux);
+										bool bRes = _deleteFmtMarkWithNotify(pFM->getPos(),pFM,
+													pfsContainerTemp,NULL,NULL);
+										UT_ASSERT(bRes);
+
+										pF = pF->getNext();
+										continue;
+										
+									}
+									break;
+								case pf_Frag::PFT_EndOfDoc:
+								    	bHasIndex = false;
+									break;
+								default:
+									UT_ASSERT(UT_SHOULD_NOT_HAPPEN);
+									bHasIndex = false;
+							}
+							
+							if(bHasIndex)
+							{
+							    pF->getPieceTable()->getAttrProp(iAPI,&pAP);
+							    UT_ASSERT(pAP);
+							    const XML_Char* pszEid2 = NULL;
+				            	k = 0;
+    	    			    	bFound = false;
+    	    			    	bool bResult = false;
+					
+					    		while((pAP)->getNthAttribute(k++,pszEidName, pszEid2))
+						    	{
+				    				if(!UT_strcmp(pszEidName, "endnote-id"))
+						    		{
+						    			if(!UT_strcmp(pszEid,pszEid2))
+						    			{
+					    					bFound = true;
+		    								break;
+		    							}
+						    		}
+							    }
+							
+							    // OK, if we have got something that carries our id
+							    // we have to delete it, we do not know what it is
+							    // the simplest thing is to call deleteSpan recursively
+							    // this works in all cases but the first block, where
+							    // calling deleteSpan deletes both the block and the
+							    // section break; further, when deleting the first block
+							    // we have to delete the content first
+							    if(bFound)
+							    {
+#ifdef DEBUG
+							    	UT_DEBUGMSG(("FOUND AND DELETING ... type %d\n", pF->getType()));
+									if(pF->getType() == pf_Frag::PFT_Strux)
+									{
+										pf_Frag_Strux * pFS = static_cast<pf_Frag_Strux*>(pF);
+										UT_DEBUGMSG(("strux type %d\n", pFS->getStruxType()));
+									}
+#endif
+							    	PT_DocPosition posStart = pF->getPos();
+								    PT_DocPosition posEnd   = pF->getLength() + posStart;
+									
+								    // if this is a strux, we
+								    // are going to need it, otherwise
+								    // we just need to get the frag ahead of us
+									pf_Frag_Strux * pFS;
+									if(pF->getType() == pf_Frag::PFT_Strux)
+										pFS = static_cast<pf_Frag_Strux *> (pF);
+									else
+										pFS = NULL;
+								
+								    // get the run before us
+								    pF = pF->getPrev();
+#ifdef DEBUG
+								    UT_DEBUGMSG(("type prev %d\n", pF ? pF->getType() : -1));
+									if(pF->getType() == pf_Frag::PFT_Strux)
+									{
+										pf_Frag_Strux * pFS = static_cast<pf_Frag_Strux*>(pF);
+										UT_DEBUGMSG(("strux type %d\n", pFS->getStruxType()));
+									}
+#endif
+									// if this is a strux and the previous
+									// is also a strux, we need to handle this
+									// with more care, so we do not delelete both
+									// together
+								    if(pFS && pF->getType() == pf_Frag::PFT_Strux)
+								    {
+								    	UT_DEBUGMSG(("Two strux in sequence\n"));
+								    	if(pF == pESection)
+								    	{
+								    		// we trying to delete the first block
+								    		// it will not work, we have to delete
+								    		// the content first, so we just skip it
+								    		// for the moment
+								    		UT_DEBUGMSG(("First block in Endnote section\n"));
+								    		pFirstBlock = pFS;
+								    		pF = pFS;
+								    		bResult = true;
+								    	}
+								    	else
+											bResult = _deleteStruxWithNotify(dpos1,pFS,NULL,NULL);
+									}
+									else
+										bResult = deleteSpan(posStart,posEnd,NULL,false);
+									UT_ASSERT(bResult);
+								}
+							}
+							pF = pF->getNext();
+							UT_DEBUGMSG(("pF 0x%x\n", pF));
+							UT_DEBUGMSG(("type %d\n", pF ? pF->getType() : -1));
+#ifdef DEBUG
+							if(pF && pF->getType() == pf_Frag::PFT_Strux)
+							{
+								pf_Frag_Strux * pFS = static_cast<pf_Frag_Strux*>(pF);
+								UT_DEBUGMSG(("strux type %d\n", pFS->getStruxType()));
+							}
+#endif
+					  	}
+					  	
+					  	// now if we ignored the first block, we should
+					  	// be able to delete it. Also, if we delete the
+					  	// first block, we need to check whether there
+					  	// are any endnotes left, and if not then we have to
+					  	// delete the endnote section and reset the endnote
+					  	// attribute of the original section.
+					  	if(pFirstBlock)
+					  	{
+					  		bool bSectionEmpty = true;
+					  		
+					  		pF = pFirstBlock->getNext();
+					  		UT_ASSERT(pF);
+				  			switch(pF->getType())
+				  			{
+								case pf_Frag::PFT_Strux:
+									{
+										pf_Frag_Strux * pFS = static_cast<pf_Frag_Strux *>(pF);
+										switch(pFS->getStruxType())
+										{
+											case PTX_Block:
+												bSectionEmpty = false;
+											default:
+												;
+										}
+									}
+									break;
+								case pf_Frag::PFT_FmtMark:
+								case pf_Frag::PFT_Text:
+								case pf_Frag::PFT_Object:
+									bSectionEmpty = false;
+									break;
+								case pf_Frag::PFT_EndOfDoc:
+									break;
+								default:
+									UT_ASSERT(UT_SHOULD_NOT_HAPPEN);
+					  			
+					  		}
+							
+							if(bSectionEmpty)
+							{
+								UT_DEBUGMSG(("Endnote section empty, deleting\n"));
+								bResult = _deleteStruxWithNotify(pESection->getPos(),pESection,NULL,NULL);
+								
+								// now we need to remove the endnote attribute from the orig section
+								// we assume here that each endnote section is used by only one
+								// other section in the document; if this should change in the
+								// future, we will have to scan the whole doc ...
+								const XML_Char*	sec_attributes2[] = {
+									"endnote", "",
+									NULL, NULL
+									};
+
+								PT_DocPosition posSec = pMySection->getPos();
+								changeStruxFmt(PTC_RemoveFmt, posSec, posSec, sec_attributes2, NULL, PTX_Section);
+								
+							}
+					  			
+					  		UT_DEBUGMSG(("Deleting first block\n"));
+							bResult = _deleteStruxWithNotify(pFirstBlock->getPos(),pFirstBlock,NULL,NULL);
+							UT_ASSERT(bResult);
+						}
+				
+				    }
+				    // no break, let everything fall through
+				}
+				default:
+					bResult
+						= _deleteObjectWithNotify(dpos1,pO,fragOffset_First,lengthThisStep,
+									  pfsContainer,&pfNewEnd,&fragOffsetNewEnd);
+				
+			}
+				
+			
 			UT_ASSERT(bResult);
 		}
 		break;
@@ -752,6 +1311,15 @@ bool pt_PieceTable::deleteSpan(PT_DocPosition dpos1,
 		}
 	}
 
+	// The code used to only glob for the complex case. But when
+	// there's a simple delete, we may still end up adding the
+	// formatmark below (i.e., when deleting all the text in a
+	// document), and thus creating a two-step undo for a perceived
+	// one-step operation. See Bug FIXME
+	if(!bDontGlob)
+	{
+		beginMultiStepGlob();
+	}
 
 	bool bIsSimple = _isSimpleDeleteSpan(dpos1, dpos2) && stDelayStruxDelete.getDepth() == 0;
 	if (bIsSimple)
@@ -765,12 +1333,7 @@ bool pt_PieceTable::deleteSpan(PT_DocPosition dpos1,
 	{
 		//  If the delete spans multiple fragments, we need to 
 		//  be a bit more careful about deleting the formatting 
-		//  first, and then the actual spans.  Also, glob all
-		//  changes together.
-		if(!bDontGlob)
-		{
-			beginMultiStepGlob();
-		}
+		//  first, and then the actual spans.
 		_changePointWithNotify(old_dpos2);
 
 		bSuccess = _deleteFormatting(dpos1, dpos2);
@@ -810,16 +1373,15 @@ bool pt_PieceTable::deleteSpan(PT_DocPosition dpos1,
 		xxx_UT_DEBUGMSG(("pt_PieceTable::deleteSpan Paragraph empty\n"));
 
 		// All text in paragraph is deleted so insert a text format.
-		// Except if we're realy don't want it. We know we dont if bDontGlob is true.
+		// Except if we're realy don't want it. We know we dont if
+		// bDontGlob is true.
 		if(!bDontGlob)
 			_insertFmtMarkFragWithNotify(PTC_AddFmt, dpos1, &AttrProp_Before);
 
 	}
 
-	// By ending the glob after having inserted the FmtMark, an undo
-	// behaves properly, instead of being a two-step operation. See
-	// bug 1140.
-	if (!bIsSimple && !bDontGlob)
+	// End the glob after (maybe) having inserted the FmtMark
+	if (!bDontGlob)
 	{
 		endMultiStepGlob();
 	}
