@@ -46,10 +46,9 @@
 /*****************************************************************/
 
 pt_PieceTable::pt_PieceTable(PD_Document * pDocument) 
-	: m_hashStyles(11)
+  : m_hashStyles(11), m_pts(PTS_Create), m_pDocument(pDocument),
+    m_atomicGlobCount(0)
 {
-	m_pts = PTS_Create;
-	m_pDocument = pDocument;
 
 	setPieceTableState(PTS_Create);
 	loading.m_indexCurrentInlineAP = 0;
@@ -420,6 +419,11 @@ PT_DocPosition pt_PieceTable::getStruxPosition(PL_StruxDocHandle sdh) const
 	return getFragPosition(pfToFind);
 }
 
+void pt_PieceTable::deleteHdrFtrStrux(pf_Frag_Strux * pfs)
+{
+	_deleteHdrFtrStruxWithNotify(pfs);
+}
+
 PT_DocPosition pt_PieceTable::getFragPosition(const pf_Frag * pfToFind) const
 {
 	if(m_fragments.areFragsDirty())
