@@ -158,6 +158,25 @@ int AP_QNXDialog_Options::s_allowTransparentColor(PtWidget_t *widget, void *data
 	return Pt_CONTINUE;
 }
 
+bool AP_QNXDialog_Options::_gatherAllowCustomToolbars(void)
+{
+	return false;
+}
+
+void AP_QNXDialog_Options::_setAllowCustomToolbars(const bool b)
+{
+}
+
+// TODO:  Add these item to dialog and impliment
+bool AP_QNXDialog_Options::_gatherAutoLoadPlugins(void)
+{
+	return false;
+}
+
+void AP_QNXDialog_Options::_setAutoLoadPlugins(const bool b)
+{
+}
+
 
 /*****************************************************************/
 
@@ -368,7 +387,7 @@ PtWidget_t* AP_QNXDialog_Options::_constructWindow ()
 	int		n;
 
 #define WIN_WIDTH  450	
-#define WIN_HEIGHT 320	
+#define WIN_HEIGHT 350	
 	n = 0;
 	PtSetArg(&args[n++], Pt_ARG_WINDOW_TITLE, pSS->getValue(AP_STRING_ID_DLG_Options_OptionsTitle), 0);
 	PtSetArg(&args[n++], Pt_ARG_WINDOW_RENDER_FLAGS, 0, ABI_MODAL_WINDOW_RENDER_FLAGS);
@@ -433,7 +452,7 @@ PtWidget_t* AP_QNXDialog_Options::_constructWindow ()
 	n = 0;
 	PtSetArg(&args[n++], Pt_ARG_TEXT_STRING, TR(pSS->getValue( AP_STRING_ID_DLG_Options_Label_SpellInternet)), 0);
 	checkbuttonSpellInternet = PtCreateWidget(PtToggleButton, vspellgroup, n, args);
-
+ 
 	n = 0;
 	PtSetArg(&args[n++], Pt_ARG_TEXT_STRING, TR(pSS->getValue( AP_STRING_ID_DLG_Options_Label_SpellCheckAsType)), 0);
 	checkbuttonSpellCheckAsType = PtCreateWidget(PtToggleButton, vspellgroup, n, args);
@@ -969,7 +988,7 @@ void AP_QNXDialog_Options::_controlEnable( tControl id, bool value )
 	PtArg_t arg;
 	PtSetArg(&arg, Pt_ARG_FLAGS, 
 				(value) ? (Pt_BLOCKED | Pt_GHOST) : 0,
-				Pt_BLOCKED | Pt_GHOST);
+				Pt_BLOCKED | Pt_GHOST|Pt_SET);
 	PtSetResources(w, 1, &arg);
 }
 
