@@ -44,9 +44,12 @@
 #include "xap_UnixFontManager.h"
 #include "ap_UnixStatusBar.h"
 #include "ap_UnixViewListener.h"
-#include "ut_dialogHelper.h"
+#include "xap_UnixDialogHelper.h"
 #if 1
 #include "ev_UnixMenuBar.h"
+#include "ev_Menu_Layouts.h"
+#include "ev_Menu_Labels.h"
+#include "ev_Menu_Actions.h"
 #endif
 
 #ifdef ABISOURCE_LICENSED_TRADEMARKS
@@ -84,7 +87,6 @@ UT_Error AP_UnixFrame::_showDocument(UT_uint32 iZoom)
 		UT_ASSERT(UT_SHOULD_NOT_HAPPEN);
 		return UT_IE_IMPORTERROR;
 	}
-
 	GR_UnixGraphics * pG = NULL;
 	FL_DocLayout * pDocLayout = NULL;
 	AV_View * pView = NULL;
@@ -100,21 +102,15 @@ UT_Error AP_UnixFrame::_showDocument(UT_uint32 iZoom)
 
 	gboolean bFocus;
 	XAP_UnixFontManager * fontManager = ((XAP_UnixApp *) getApp())->getFontManager();
-	
+	gtk_widget_show(m_dArea);
 	pG = new GR_UnixGraphics(m_dArea->window, fontManager, getApp());
 	ENSUREP(pG);
 	pG->setZoomPercentage(iZoom);
-	
+
 	pDocLayout = new FL_DocLayout(static_cast<PD_Document *>(m_pDoc), pG);
-	ENSUREP(pDocLayout);
-  
-//	pDocLayout->formatAll();
+	ENSUREP(pDocLayout);  
 
 	pView = new FV_View(getApp(), this, pDocLayout);
-	if (m_pView != NULL)
-	{
-		point = ((FV_View *) m_pView)->getPoint();
-	}
 	ENSUREP(pView);
 
 	bFocus=GPOINTER_TO_INT(gtk_object_get_data(GTK_OBJECT(m_wTopLevelWindow),"toplevelWindowFocus"));
@@ -159,7 +155,6 @@ UT_Error AP_UnixFrame::_showDocument(UT_uint32 iZoom)
 	if (!pView->addListener(static_cast<AV_Listener *>(pScrollbarViewListener),
 							&lidScrollbarViewListener))
 		goto Cleanup;
-
 	nrToolbars = m_vecToolbarLayoutNames.getItemCount();
 	for (k = 0; k < nrToolbars; k++)
 	{
@@ -221,37 +216,43 @@ UT_Error AP_UnixFrame::_showDocument(UT_uint32 iZoom)
 		((AP_FrameData*)m_pData)->m_pLeftRuler->setView(pView, iZoom);
 	}
 
-	if ( ((AP_FrameData*)m_pData)->m_pStatusBar )
+	if ( ((AP_FrameData*)m_pData)->m_pStatusBar && (m_iFrameMode != XAP_NoMenusWindowLess))
 	  ((AP_FrameData*)m_pData)->m_pStatusBar->setView(pView);
+    ((FV_View *) m_pView)->setShowPara(((AP_FrameData*)m_pData)->m_bShowPara);
 
 	pView->setInsertMode(((AP_FrameData*)m_pData)->m_bInsertMode);
-    ((FV_View *) m_pView)->setShowPara(((AP_FrameData*)m_pData)->m_bShowPara);
 	
 	m_pView->setWindowSize(GTK_WIDGET(m_dArea)->allocation.width,
 						   GTK_WIDGET(m_dArea)->allocation.height);
+
 	setXScrollRange();
 	setYScrollRange();
 	updateTitle();
 
+	pDocLayout->fillLayouts();   
+
+	if (m_pView != NULL)
+	{
+	  // WL: adding this method into the UnixFrame from the win32 code to fix 2615
+	  // we cannot just set the insertion position to that of the previous
+	  // view, since the new document could be shorter or completely
+	  // different from the previous one (see bug 2615)
+	  // Instead we have to test that the original position is within
+	  // the editable bounds, and if not, we will set the point
+	  // to the end of the document (i.e., if reloading an earlier
+	  // version of the same document we try to get the point as near
+	  // the users editing position as possible
+	  point = ((FV_View *) m_pView)->getPoint();
+	  PT_DocPosition posEOD;
+	  static_cast<FV_View *>(pView)->getEditableBounds(true, posEOD, false);
+	  if(point > posEOD)
+	    point = posEOD;
+	}
+   
 	if (point != 0)
-		((FV_View *) m_pView)->moveInsPtTo(point);
-#if 1
-	/*
-	  UPDATE:  this code is back, but I'm leaving these comments as
-	  an audit trail.  See bug 99.  This only happens when loading
-	  a document into an empty window -- the case where a frame gets
-	  reused.  TODO consider putting an expose into ap_EditMethods.cpp
-	  instead of a draw() here.
-	*/
-	
-	/*
-	  I've removed this once again.  (Eric)  I replaced it with a call
-	  to draw() which is now in the configure event handler in the GTK
-	  section of the code.  See me if this causes problems.
-	*/
-	m_pView->draw();
-#endif	
-	
+        ((FV_View *) m_pView)->moveInsPtTo(point);
+      m_pView->draw();
+
 	if ( ((AP_FrameData*)m_pData)->m_bShowRuler  ) 
 	{
 		if ( ((AP_FrameData*)m_pData)->m_pTopRuler )
@@ -260,9 +261,10 @@ UT_Error AP_UnixFrame::_showDocument(UT_uint32 iZoom)
 		if ( ((AP_FrameData*)m_pData)->m_pLeftRuler )
 			((AP_FrameData*)m_pData)->m_pLeftRuler->draw(NULL);
 	}
-
-	((AP_FrameData*)m_pData)->m_pStatusBar->draw();
-	
+	if(isStatusBarShown())
+	{
+		((AP_FrameData*)m_pData)->m_pStatusBar->draw();
+	}
 	return UT_OK;
 
 Cleanup:
@@ -355,11 +357,13 @@ AP_UnixFrame::~AP_UnixFrame()
 	killFrameData();
 }
 
-bool AP_UnixFrame::initialize()
+bool AP_UnixFrame::initialize(XAP_FrameMode frameMode)
 {
-	UT_DEBUGMSG(("AP_UnixFrame::initialize"));
+	UT_DEBUGMSG(("AP_UnixFrame::initialize!!!! \n"));
+	m_iFrameMode = frameMode;
 	if (!initFrameData())
 		return false;
+	UT_DEBUGMSG(("AP_UnixFrame:: Initializing base class!!!! \n"));
 
 	if (!XAP_UNIXBASEFRAME::initialize(AP_PREF_KEY_KeyBindings,AP_PREF_DEFAULT_KeyBindings,
 								   AP_PREF_KEY_MenuLayout, AP_PREF_DEFAULT_MenuLayout,
@@ -368,20 +372,20 @@ bool AP_UnixFrame::initialize()
 								   AP_PREF_KEY_ToolbarLabelSet, AP_PREF_DEFAULT_ToolbarLabelSet))
 		return false;
 
-#ifndef ABI_OPT_WIDGET
+	UT_DEBUGMSG(("AP_UnixFrame:: Creating Toplevel Window!!!! \n"));
+
 	_createTopLevelWindow();
-	// needs to be shown so that the following functions work
-	// TODO: get rid of cursed flicker caused by initially
-	// TODO: showing these and then hiding them (esp.
-	// TODO: noticable in the gnome build with a toolbar disabled)
 	gtk_widget_show(m_wTopLevelWindow);
-	_showOrHideToolbars();
-	_showOrHideStatusbar();
-#endif
+	if(m_iFrameMode == XAP_NormalFrame)
+	{
+		// needs to be shown so that the following functions work
+		// TODO: get rid of cursed flicker caused by initially
+		// TODO: showing these and then hiding them (esp.
+		// TODO: noticable in the gnome build with a toolbar disabled)
+		_showOrHideToolbars();
+		_showOrHideStatusbar();
+	}
 
-#if 0
-	m_pUnixMenu->addPath("/&Edit/Adieu");
-#endif
 	return true;
 }
 
@@ -404,6 +408,22 @@ void AP_UnixFrame::_showOrHideToolbars()
 	}
 }
 
+/*!
+ * Refills the framedata class with pointers to the current toolbars. We 
+ * need to do this after a toolbar icon and been dragged and dropped.
+ */
+void AP_UnixFrame::	refillToolbarsInFrameData(void)
+{
+	UT_uint32 cnt = m_vecToolbarLayoutNames.getItemCount();
+
+	for (UT_uint32 i = 0; i < cnt; i++)
+	{
+		EV_UnixToolbar * pUnixToolbar = static_cast<EV_UnixToolbar *> (m_vecToolbars.getNthItem(i));
+		static_cast<AP_FrameData*> (m_pData)->m_pToolbar[i] = pUnixToolbar;
+	}
+}
+
+
 // Does the initial show/hide of statusbar (based on the user prefs).
 // Idem.
 void AP_UnixFrame::_showOrHideStatusbar()
@@ -542,9 +562,25 @@ ReplaceDocument:
 XAP_Frame * AP_UnixFrame::cloneFrame()
 {
 	AP_UnixFrame * pClone = new AP_UnixFrame(this);
-	UT_Error error = UT_OK;
 	ENSUREP(pClone);
+	return static_cast<XAP_Frame *>(pClone);
 
+ Cleanup:
+	// clean up anything we created here
+	if (pClone)
+	{
+		m_pUnixApp->forgetFrame(pClone);
+		delete pClone;
+	}
+	return NULL;
+}
+
+
+XAP_Frame * AP_UnixFrame::buildFrame(XAP_Frame * pF)
+{
+	UT_Error error = UT_OK;
+	AP_UnixFrame * pClone =	static_cast<AP_UnixFrame *>(pF);
+	ENSUREP(pClone);
 	if (!pClone->initialize())
 		goto Cleanup;
 
@@ -553,17 +589,15 @@ XAP_Frame * AP_UnixFrame::cloneFrame()
 		goto Cleanup;
 
 	pClone->show();
+	return static_cast<XAP_Frame *>(pClone);
 
-	return pClone;
-
-Cleanup:
+ Cleanup:
 	// clean up anything we created here
 	if (pClone)
 	{
 		m_pUnixApp->forgetFrame(pClone);
 		delete pClone;
 	}
-
 	return NULL;
 }
 
@@ -690,6 +724,7 @@ void AP_UnixFrame::_scrollFuncX(void * pData, UT_sint32 xoff, UT_sint32 /*xrange
 	pView->setXScrollOffset((UT_sint32)xoffNew);
 }
 
+
 GtkWidget * AP_UnixFrame::_createDocumentWindow()
 {
 	bool bShowRulers = static_cast<AP_FrameData*> (m_pData)->m_bShowRuler;
@@ -775,7 +810,7 @@ GtkWidget * AP_UnixFrame::_createDocumentWindow()
 
 	// create a table for scroll bars, rulers, and drawing area
 
-	m_table = gtk_table_new(1, 1, FALSE);
+	m_table = gtk_table_new(1, 1, FALSE); //was 1,1
 	gtk_object_set_user_data(GTK_OBJECT(m_table),this);
 
 	// NOTE:  in order to display w/ and w/o rulers, gtk needs two tables to
@@ -791,11 +826,11 @@ GtkWidget * AP_UnixFrame::_createDocumentWindow()
 	// scroll bars
 	gtk_table_attach(GTK_TABLE(m_table), m_hScroll, 0, 1, 1, 2,
 					 (GtkAttachOptions) (GTK_EXPAND | GTK_FILL),
-					 (GtkAttachOptions) (GTK_FILL),
+					 (GtkAttachOptions) (GTK_FILL), // was just GTK_FILL
 					 0, 0);
 
 	gtk_table_attach(GTK_TABLE(m_table), m_vScroll, 1, 2, 0, 1,
-					 (GtkAttachOptions) (GTK_FILL),
+					 (GtkAttachOptions) (GTK_FILL), // was just GTK_FILL
 					 (GtkAttachOptions) (GTK_EXPAND | GTK_FILL),
 					 0, 0);
 
@@ -814,10 +849,11 @@ GtkWidget * AP_UnixFrame::_createDocumentWindow()
 						 (GtkAttachOptions)(GTK_FILL),
 						 0, 0);
 
-		gtk_table_attach(GTK_TABLE(m_innertable), m_leftRuler, 0, 1, 1, 2,
-						 (GtkAttachOptions)(GTK_FILL),
-						 (GtkAttachOptions)(GTK_EXPAND | GTK_FILL),
-						 0, 0);
+		if (m_leftRuler)
+			gtk_table_attach(GTK_TABLE(m_innertable), m_leftRuler, 0, 1, 1, 2,
+							 (GtkAttachOptions)(GTK_FILL),
+							 (GtkAttachOptions)(GTK_EXPAND | GTK_FILL),
+							 0, 0);
 
 		gtk_table_attach(GTK_TABLE(m_innertable), m_dArea,   1, 2, 1, 2,
 						 (GtkAttachOptions) (GTK_EXPAND | GTK_FILL),
@@ -962,7 +998,14 @@ void AP_UnixFrame::toggleLeftRuler(bool bRulerOn)
 		     bRulerOn, pFrameData->m_pLeftRuler));
 
 	if (bRulerOn)
-	  {
+	{
+//
+// if there is an old ruler just return.
+//
+		if(m_leftRuler)
+		{
+			return;
+		}
 		pUnixLeftRuler = new AP_UnixLeftRuler(this);
 		UT_ASSERT(pUnixLeftRuler);
 		m_leftRuler = pUnixLeftRuler->createWidget();
@@ -972,15 +1015,16 @@ void AP_UnixFrame::toggleLeftRuler(bool bRulerOn)
 				 (GtkAttachOptions)(GTK_EXPAND | GTK_FILL),
 				 0,0);
 		pUnixLeftRuler->setView(m_pView);
-	  }
+		setYScrollRange();
+	}
 	else
-	  {
+	{
 	    if (m_leftRuler && GTK_IS_OBJECT(m_leftRuler))
 		gtk_object_destroy( GTK_OBJECT(m_leftRuler) );
 	    
 	    DELETEP(((AP_FrameData*)m_pData)->m_pLeftRuler);
 	    m_leftRuler = NULL;
-	  }
+	}
 
 	((AP_FrameData*)m_pData)->m_pLeftRuler = pUnixLeftRuler;
 }
