@@ -21,8 +21,6 @@
 ** Only one of these is created by the application.
 *****************************************************************/
 
-#define AW_WIN32_USE_NEW_SPELL_CHECKER
-
 #define WIN32_LEAN_AND_MEAN
 #include <stdlib.h>
 #include <windows.h>
@@ -33,6 +31,7 @@
 
 #include <stdio.h>
 #include <string.h>
+#include <io.h>
 
 #include "ut_debugmsg.h"
 #include "ut_bytebuf.h"
@@ -41,17 +40,16 @@
 #include "ap_Convert.h"
 #include "ap_Win32Frame.h"
 #include "ap_Win32App.h"
-#ifdef AW_WIN32_USE_NEW_SPELL_CHECKER
 #include "spell_manager.h"
-#else
-#include "sp_spell.h"
-#endif
+
 #include "ap_Strings.h"
 #include "ap_LoadBindings.h"
 #include "xap_EditMethods.h"
+#include "xap_Menu_Layouts.h"
 #include "xap_Menu_ActionSet.h"
 #include "xap_Toolbar_ActionSet.h"
 #include "xap_EncodingManager.h"
+#include "xap_ModuleManager.h"
 
 #include "ap_Win32Resources.rc2"
 #include "ap_Clipboard.h"
@@ -82,15 +80,10 @@ AP_Win32App::AP_Win32App(HINSTANCE hInstance, XAP_Args * pArgs, const char * szA
 
 AP_Win32App::~AP_Win32App(void)
 {
-#ifndef AW_WIN32_USE_NEW_SPELL_CHECKER
-	SpellCheckCleanup();
-#endif
-
 	DELETEP(m_pStringSet);
 	DELETEP(m_pClipboard);
 
-	IE_Exp::unregisterAllExporters ();
-	IE_Imp::unregisterAllImporters ();
+	IE_ImpExp_UnRegisterXP ();
 }
 
 static bool s_createDirectoryIfNecessary(const char * szDir)
@@ -162,30 +155,7 @@ bool AP_Win32App::initialize(void)
 	//////////////////////////////////////////////////////////////////
 	
 	{
-#ifdef AW_WIN32_USE_NEW_SPELL_CHECKER
 		SpellManager::instance().requestDictionary(xap_encoding_manager_get_language_iso_name());
-#else
-		const char * szISpellDirectory = NULL;
-		getPrefsValueDirectory(false,AP_PREF_KEY_SpellDirectory,&szISpellDirectory);
-		UT_ASSERT((szISpellDirectory) && (*szISpellDirectory));
-
-		const char * szSpellCheckWordList = NULL;
-		getPrefsValue(AP_PREF_KEY_SpellCheckWordList,&szSpellCheckWordList);
-		UT_ASSERT((szSpellCheckWordList) && (*szSpellCheckWordList));
-		
-		char * szPathname = (char *)calloc(sizeof(char),strlen(szISpellDirectory)+strlen(szSpellCheckWordList)+2);
-		UT_ASSERT(szPathname);
-		
-		sprintf(szPathname,"%s%s%s",
-				szISpellDirectory,
-				((szISpellDirectory[strlen(szISpellDirectory)-1]=='\\') ? "" : "\\"),
-				szSpellCheckWordList);
-
-		UT_DEBUGMSG(("Loading SpellCheckWordList [%s]\n",szPathname));
-		SpellCheckInit(szPathname);
-		free(szPathname);
-#endif	
-		// we silently go on if we cannot load it....
 	}
 	
 	//////////////////////////////////////////////////////////////////
@@ -240,8 +210,6 @@ bool AP_Win32App::initialize(void)
 		}
 	}
 	// Now we have the strings loaded we can populate the field names correctly
-	// CHECK THIS: the following was added by a Linux developer who can't test
-	// on Win32, or even compile, so someone with a Windows box needs to check it
 	int i;
 	
 	for (i = 0; fp_FieldTypes[i].m_Type != FPFIELDTYPE_END; i++)
@@ -256,7 +224,60 @@ bool AP_Win32App::initialize(void)
 	    UT_DEBUGMSG(("Setting field desc for field %s, desc=%s\n", fp_FieldFmts[i].m_Tag, fp_FieldFmts[i].m_Desc));
 	}
 
+    ///////////////////////////////////////////////////////////////////////
+    /// Build a labelset so the plugins can add themselves to something ///
+    ///////////////////////////////////////////////////////////////////////
+
+	const char * szMenuLabelSetName = NULL;
+	if (getPrefsValue( AP_PREF_KEY_MenuLabelSet, (const XML_Char**)&szMenuLabelSetName)
+		&& (szMenuLabelSetName) && (*szMenuLabelSetName))
+	{
+		;
+	}
+	else
+		szMenuLabelSetName = AP_PREF_DEFAULT_MenuLabelSet ;
+
+	getMenuFactory()->buildMenuLabelSet(szMenuLabelSetName);	
+	
+	
 	//////////////////////////////////////////////////////////////////
+	// load the all Plugins from the correct directory
+	//////////////////////////////////////////////////////////////////
+
+	char szPath[_MAX_PATH];
+	char szPlugin[_MAX_PATH];
+	_getExeDir( szPath, _MAX_PATH);
+	strcat(szPath, "..\\Plugins\\*.dll");
+
+    struct _finddata_t cfile;
+	long findtag = _findfirst( szPath, &cfile );
+	if( findtag != -1 )
+	{
+		do
+		{	
+			_getExeDir( szPlugin, _MAX_PATH );
+			strcat( szPlugin, "..\\Plugins\\" );
+			strcat( szPlugin, cfile.name );
+			XAP_ModuleManager::instance().loadModule( szPlugin );
+		} while( _findnext( findtag, &cfile ) == 0 );
+	}
+	_findclose( findtag );
+
+	UT_String pluginName( getUserPrivateDirectory() ); 
+	UT_String pluginDir( getUserPrivateDirectory() );
+	pluginDir += "\\AbiWord\\plugins\\*.dll";
+	findtag = _findfirst( pluginDir.c_str(), &cfile );
+	if( findtag != -1 )
+	{
+		do
+		{	
+			pluginName = getUserPrivateDirectory();
+			pluginName += "\\AbiWord\\plugins\\";
+			pluginName += cfile.name;
+			XAP_ModuleManager::instance().loadModule( pluginName.c_str() );
+		} while( _findnext( findtag, &cfile ) == 0 );
+	}
+	_findclose( findtag );
 
 	return true;
 }
@@ -723,8 +744,8 @@ int AP_Win32App::WinMain(const char * szAppName, HINSTANCE hInstance,
 		InitCommonControls();
 		MessageBox(NULL,
 			"AbiWord is designed for a newer version of the system file COMCTL32.DLL\n"
-			"then the one currently on your system.\n"
-			"A solution to this problem is explained in the FAQ on AbiSource web site\n"
+			"than the one currently on your system.\n"
+			"A solution to this problem is explained in the FAQ on the AbiSource web site\n"
 			"\n\thttp://www.abisource.com\n\n"
 			"We hope this problem can be solved, until then you can use the program,\n"
 			"but the toolbar will be missing.", NULL, MB_OK);
@@ -738,12 +759,16 @@ int AP_Win32App::WinMain(const char * szAppName, HINSTANCE hInstance,
 	if (!hinstRich)
 		hinstRich = LoadLibrary("riched20.dll");
 	UT_ASSERT(hinstRich);
+	
+	AP_Win32App * pMyWin32App;
 
+// We put this in a block to force the destruction of Args in the stack
+{	
 	// Load the command line into an XAP_Args class
 	XAP_Args Args = XAP_Args(szCmdLine);
 
 	// initialize our application.
-	AP_Win32App * pMyWin32App = new AP_Win32App(hInstance, &Args, szAppName);
+	pMyWin32App = new AP_Win32App(hInstance, &Args, szAppName);
 	pMyWin32App->initialize();
   
 	// Quick & Dirty command-line 
@@ -753,14 +778,20 @@ int AP_Win32App::WinMain(const char * szAppName, HINSTANCE hInstance,
 	// so [0] is the first argument
 	int nFirstArg = 0;
 	int k;
- 
+ 			
 	for (k=nFirstArg; (k<Args.m_argc); k++) {
 		if (*Args.m_argv[k] == '-') {
-			if (UT_stricmp(Args.m_argv[k],"-to") == 0 || UT_stricmp(Args.m_argv[k],"-help") == 0) {
+			if ( (UT_stricmp(Args.m_argv[k],"-to") == 0) ||
+                 (UT_stricmp(Args.m_argv[k],"--to") == 0) || 
+                 (UT_stricmp(Args.m_argv[k],"-help") == 0) ||
+                 (UT_stricmp(Args.m_argv[k],"--help") == 0) )
+           {
  				bShowApp = false;
 				bShowSplash = false;
 			}
-			if( UT_stricmp(Args.m_argv[k],"-nosplash") == 0 ) {
+			if( (UT_stricmp(Args.m_argv[k],"-nosplash") == 0) ||
+                (UT_stricmp(Args.m_argv[k],"--nosplash") == 0) )
+			{
 				bShowSplash = false;
 			}
 		}
@@ -768,7 +799,9 @@ int AP_Win32App::WinMain(const char * szAppName, HINSTANCE hInstance,
 
     for (k=nFirstArg; (k<Args.m_argc); k++) {
 		if (*Args.m_argv[k] == '-') {
-			if (UT_stricmp(Args.m_argv[k],"-show") == 0) {
+			if ((UT_stricmp(Args.m_argv[k],"-show") == 0) ||
+				(UT_stricmp(Args.m_argv[k],"--show") == 0) )
+            {
  				bShowApp = true;
 				bShowSplash = true;
 			}
@@ -791,7 +824,13 @@ int AP_Win32App::WinMain(const char * szAppName, HINSTANCE hInstance,
 #endif
 
 	pMyWin32App->ParseCommandLine(iCmdShow);
-
+}
+//
+// This block is controlled by the Structured Exception Handle
+// if any crash happens here we will recover it and save the file (cross fingers)
+//	
+__try
+{				
 	if (bShowApp)
 	{
 		while( GetMessage(&msg, NULL, 0, 0) )
@@ -801,7 +840,7 @@ int AP_Win32App::WinMain(const char * szAppName, HINSTANCE hInstance,
 	      	if (pMyWin32App->handleModelessDialogMessage(&msg)) 
 				continue;
 	    	DispatchMessage(&msg);	
-
+	    	
 			// Check for idle condition
 			while( !UT_Win32Idle::_isEmpty() &&
                    !PeekMessage(&msg, NULL, 0, 0, PM_NOREMOVE) ) 
@@ -815,12 +854,65 @@ int AP_Win32App::WinMain(const char * szAppName, HINSTANCE hInstance,
 	// destroy the App.  It should take care of deleting all frames.
 	pMyWin32App->shutdown();
 	delete pMyWin32App;
+	
+}// end of thes block is controlled by the SEH 
+
+//
+// If an exception happens, with "catch" the block
+// and then the save it into disk
+//
+__except (1)
+{
+	AP_Win32App *pApp = (AP_Win32App *) XAP_App::getApp();
+	
+	UT_ASSERT(pApp);
+	
+	UT_uint32 i = 0;
+	
+	for(;i<pApp->m_vecFrames.getItemCount();i++)
+	{
+		AP_Win32Frame * curFrame = (AP_Win32Frame*)pApp->m_vecFrames[i];
+		UT_ASSERT(curFrame);
+		
+		curFrame->backup(".CRASHED");
+	}	
+}// end of except
 
 	SET_CRT_DEBUG_FIELD( _CRTDBG_LEAK_CHECK_DF );
-
 	return msg.wParam;
 }
 
+/* This function takes a description and compares it all the registerd 
+   importers descriptions and returns either the appropriate importer's
+   IEFileType or returns IEFT_Unknown if no match was made.
+*/
+IEFileType AP_Win32App::_getFileTypeFromDesc(const char *desc)
+{
+	const char *iftDesc;
+	const char *iftSuffixList;
+	IEFileType ift;
+
+	// no description given or description == 'UNKNOWN' then unknown
+	if (!desc || !*desc || (UT_stricmp(desc, "Unknown")==0)) 
+		return IEFT_Unknown;  
+
+	UT_uint32 i = 0;
+	while (IE_Imp::enumerateDlgLabels(i, &iftDesc, &iftSuffixList, &ift))
+	{
+		// TODO: change to actually test all but suffixes, 
+		// ie if iftDesc == 'Some FileType (*.sft, *.someft)' then only
+            // test against 'Some FileType'
+		if (UT_strnicmp(iftDesc, desc, strlen(desc)) == 0)
+			return ift;
+		
+		// try next importer
+		i++;
+	}
+
+	// if we made it here then description didn't match anything, so unknown.
+	return IEFT_Unknown;
+}
+
 void AP_Win32App::ParseCommandLine(int iCmdShow)
 {
 	// parse the command line
@@ -839,23 +931,27 @@ void AP_Win32App::ParseCommandLine(int iCmdShow)
 	char *to = NULL;
 	int verbose = 1;
 	bool show = false, bHelp = false;
+	char *iftDesc = NULL;
 	
 	for (k=nFirstArg; (k<m_pArgs->m_argc); k++)
 	{
 		if (*m_pArgs->m_argv[k] == '-')
 		{
-			if (UT_stricmp(m_pArgs->m_argv[k],"-script") == 0)
+			if ((UT_stricmp(m_pArgs->m_argv[k],"-script") == 0) ||
+				(UT_stricmp(m_pArgs->m_argv[k],"--script") == 0))
 			{
 				// [-script scriptname]
 				k++;
-			}
-			else if (UT_stricmp(m_pArgs->m_argv[k],"-lib") == 0)
+				}
+			else if ((UT_stricmp(m_pArgs->m_argv[k],"-lib") == 0) ||
+                                                                (UT_stricmp(m_pArgs->m_argv[k],"--lib") == 0) )
 			{
 				// [-lib <AbiSuiteLibDirectory>]
 				// we've already processed this when we initialized the App class
 				k++;
 			}
-			else if (UT_stricmp(m_pArgs->m_argv[k],"-dumpstrings") == 0)
+			else if ((UT_stricmp(m_pArgs->m_argv[k],"-dumpstrings") == 0) ||
+					  (UT_stricmp(m_pArgs->m_argv[k],"--dumpstrings") == 0))
 			{
 				// [-dumpstrings]
 #ifdef DEBUG
@@ -866,20 +962,24 @@ void AP_Win32App::ParseCommandLine(int iCmdShow)
 				delete pBuiltinStringSet;
 #endif
 			}
-			else if (UT_stricmp(m_pArgs->m_argv[k],"-nosplash") == 0)
+			else if (UT_stricmp(m_pArgs->m_argv[k],"-nosplash") == 0 ||
+					 UT_stricmp(m_pArgs->m_argv[k], "--nosplash") == 0)
 			{
 				// we've alrady processed this before we initialized the App class
 			}
-			else if (UT_stricmp (m_pArgs->m_argv[k],"-to") == 0)
+			else if (UT_stricmp(m_pArgs->m_argv[k],"-to") == 0 ||
+					 UT_stricmp(m_pArgs->m_argv[k], "--to") == 0)
 			{
 				k++;
 				to = m_pArgs->m_argv[k];
 			}
-			else if (UT_stricmp (m_pArgs->m_argv[k], "-show") == 0)
+			else if (UT_stricmp (m_pArgs->m_argv[k], "-show") == 0 ||
+					 UT_stricmp(m_pArgs->m_argv[k], "--show") == 0)
 			{
 				show = true;
 			}
-			else if (UT_stricmp (m_pArgs->m_argv[k], "-verbose") == 0)
+			else if (UT_stricmp (m_pArgs->m_argv[k], "-verbose") == 0 ||
+					 UT_stricmp(m_pArgs->m_argv[k], "--verbose") == 0)
 			{
 				k++;
 				if(k<m_pArgs->m_argc)
@@ -889,20 +989,31 @@ void AP_Win32App::ParseCommandLine(int iCmdShow)
 					verbose = atoi (m_pArgs->m_argv[k]);
 				}
 			}
+			else if ( (UT_stricmp (m_pArgs->m_argv[k], "-filetype") == 0) ||
+			          (UT_stricmp (m_pArgs->m_argv[k], "-ft") == 0) ||
+					  (UT_stricmp(m_pArgs->m_argv[k], "--filetype") == 0) ||
+					  (UT_stricmp(m_pArgs->m_argv[k], "--ft") == 0) ) 
+			{
+				if ((k+1) < m_pArgs->m_argc) // if no more arguments follow then ignore
+				{
+					k++;	// point to description
+					iftDesc = m_pArgs->m_argv[k];  // store description
+				}
+			}
 			else if ( (UT_stricmp (m_pArgs->m_argv[k], "-help") == 0) && (!bHelp) )
 			{
 				char *pszMessage = (char*)malloc( 500 );
 
 				strcpy( pszMessage, "Usage: AbiWord.exe [option]... [file]...\n\n" );
-				strcat( pszMessage, "-to\nThe target format of the file\n\n" );
-				strcat( pszMessage, "-verbose\nThe verbosity level (0, 1, 2)\n\n" );
-				strcat( pszMessage, "-show\nIf you really want to start the GUI (even if you use the -to or -help options)\n\n" );
+				strcat( pszMessage, "--to\nThe target format of the file\n\n" );
+				strcat( pszMessage, "--verbose\nThe verbosity level (0, 1, 2)\n\n" );
+				strcat( pszMessage, "--show\nIf you really want to start the GUI (even if you use the -to or -help options)\n\n" );
 #ifdef DEBUG
-				strcat( pszMessage, "-dumpstrings\nDump strings strings to file\n\n" );
+				strcat( pszMessage, "--dumpstrings\nDump strings strings to file\n\n" );
 #endif
-				strcat( pszMessage, "-geometry <geom>\nSet initial frame geometry [UNIMPLEMENTED]\n\n" );
-				strcat( pszMessage, "-lib <dir>\nUse dir for application components\n\n" );
-				strcat( pszMessage, "-nosplash\nDo not show splash screen\n\n" );
+				strcat( pszMessage, "--geometry <geom>\nSet initial frame geometry [UNIMPLEMENTED]\n\n" );
+				strcat( pszMessage, "--lib <dir>\nUse dir for application components\n\n" );
+				strcat( pszMessage, "--nosplash\nDo not show splash screen\n\n" );
 				
 				MessageBox(NULL,
 					pszMessage, "Command Line Options", MB_OK);
@@ -931,7 +1042,7 @@ void AP_Win32App::ParseCommandLine(int iCmdShow)
 			{
 				AP_Win32Frame* pFirstWin32Frame = (AP_Win32Frame*)newFrame();
 
-				UT_Error error = pFirstWin32Frame->loadDocument(m_pArgs->m_argv[k], IEFT_Unknown);
+				UT_Error error = pFirstWin32Frame->loadDocument(m_pArgs->m_argv[k], _getFileTypeFromDesc(iftDesc));
 				if (!error)
 				{
 					kWindowsOpened++;
@@ -954,7 +1065,7 @@ void AP_Win32App::ParseCommandLine(int iCmdShow)
 					// TODO for now.  this would cause us to get 2 untitled documents
 					// TODO if the user gave us 2 bogus pathnames....
 					kWindowsOpened++;
-					pFirstWin32Frame->loadDocument(NULL, IEFT_Unknown);
+					pFirstWin32Frame->loadDocument(NULL, _getFileTypeFromDesc(iftDesc));
 					HWND hwnd = pFirstWin32Frame->getTopLevelWindow();
 					if (kWindowsOpened > 1) // only set 1st opened window to stored state
 					ShowWindow(hwnd, iCmdShow);
@@ -979,7 +1090,7 @@ void AP_Win32App::ParseCommandLine(int iCmdShow)
 
 		AP_Win32Frame* pFirstWin32Frame = (AP_Win32Frame*)newFrame();
 
-		pFirstWin32Frame->loadDocument(NULL, IEFT_Unknown);
+		pFirstWin32Frame->loadDocument(NULL, _getFileTypeFromDesc(iftDesc));
 
 		HWND hwnd = pFirstWin32Frame->getTopLevelWindow();
 		ShowWindow(hwnd, setupWindowFromPrefs(iCmdShow, hwnd));
@@ -1018,3 +1129,4 @@ bool AP_Win32App::handleModelessDialogMessage( MSG * msg )
 
 	return false;
 }
+
