@@ -41,6 +41,8 @@
 #	include "xap_Win32Dlg_Language.h"
 #	include "xap_Win32Dlg_Encoding.h"
 #	include "xap_Win32Dlg_PluginManager.h"
+#	include "xap_Win32Dlg_Password.h"
+#	include "xap_Win32Dlg_Image.h"
 
 #	include "ap_Win32Dialog_Replace.h"
 #	include "ap_Win32Dialog_Break.h"
@@ -59,7 +61,10 @@
 #	include "ap_Win32Dialog_PageSetup.h"
 #	include "ap_Win32Dialog_ToggleCase.h"
 #	include "ap_Win32Dialog_Background.h"
-
+#	include "ap_Win32Dialog_HdrFtr.h"
+#	include "ap_Win32Dialog_InsertBookmark.h"
+#	include "ap_Win32Dialog_InsertHyperlink.h"
+#	include "ap_Win32Dialog_New.h"
 	// ... add new dialogs here ...
 
 #else
@@ -67,6 +72,9 @@
 	DeclareDialog(XAP_DIALOG_ID_MESSAGE_BOX,	XAP_Win32Dialog_MessageBox)
 	DeclareDialog(XAP_DIALOG_ID_FILE_OPEN,		XAP_Win32Dialog_FileOpenSaveAs)
 	DeclareDialog(XAP_DIALOG_ID_FILE_SAVEAS,	XAP_Win32Dialog_FileOpenSaveAs)
+	DeclareDialog(XAP_DIALOG_ID_FILE_IMPORT,	XAP_Win32Dialog_FileOpenSaveAs)
+	DeclareDialog(XAP_DIALOG_ID_FILE_EXPORT,	XAP_Win32Dialog_FileOpenSaveAs)
+	DeclareDialog(XAP_DIALOG_ID_INSERT_FILE,	XAP_Win32Dialog_FileOpenSaveAs)
 	DeclareDialog(XAP_DIALOG_ID_PRINT,			XAP_Win32Dialog_Print)
 	DeclareDialog(XAP_DIALOG_ID_PRINTTOFILE,	XAP_Win32Dialog_FileOpenSaveAs)
 	DeclareDialog(XAP_DIALOG_ID_FONT,			XAP_Win32Dialog_FontChooser)
@@ -78,6 +86,8 @@
 	DeclareDialog(XAP_DIALOG_ID_INSERT_PICTURE, XAP_Win32Dialog_FileOpenSaveAs)
 	DeclareDialog(XAP_DIALOG_ID_ENCODING,		XAP_Win32Dialog_Encoding)
 	DeclareDialog(XAP_DIALOG_ID_PLUGIN_MANAGER, XAP_Win32Dialog_PluginManager)
+	DeclareDialog(XAP_DIALOG_ID_PASSWORD,		XAP_Win32Dialog_Password)
+	DeclareDialog(XAP_DIALOG_ID_IMAGE,			XAP_Win32Dialog_Image)
 
 	DeclareDialog(AP_DIALOG_ID_REPLACE,			AP_Win32Dialog_Replace)
 	DeclareDialog(AP_DIALOG_ID_FIND,			AP_Win32Dialog_Replace)
@@ -97,8 +107,10 @@
 	DeclareDialog(AP_DIALOG_ID_FILE_PAGESETUP,	AP_Win32Dialog_PageSetup)
 	DeclareDialog(AP_DIALOG_ID_TOGGLECASE,		AP_Win32Dialog_ToggleCase)
 	DeclareDialog(AP_DIALOG_ID_BACKGROUND,		AP_Win32Dialog_Background)
-        DeclareDialog(XAP_DIALOG_ID_FILE_IMPORT, XAP_Win32Dialog_FileOpenSaveAs)
-        DeclareDialog(XAP_DIALOG_ID_FILE_EXPORT, XAP_Win32Dialog_FileOpenSaveAs)
-	// ... also add new dialogs here ...
+	DeclareDialog(AP_DIALOG_ID_HDRFTR,  		AP_Win32Dialog_HdrFtr)
+ 	DeclareDialog(AP_DIALOG_ID_INSERTBOOKMARK,	AP_Win32Dialog_InsertBookmark)	
+ 	DeclareDialog(AP_DIALOG_ID_INSERTHYPERLINK,	AP_Win32Dialog_InsertHyperlink)
+	DeclareDialog(AP_DIALOG_ID_FILE_NEW,		AP_Win32Dialog_New)
+ 	// ... also add new dialogs here ...
 
 #endif
