@@ -46,7 +46,7 @@
 #define BUFSIZE		128
 #define SIGDIGIT	4
 
-float mScale[] = { 1.0f, 10.0f, 25.4f };
+float mScale[] = { 25.4f, 10.0f, 1.0f };
 
 /*****************************************************************/
 
@@ -59,7 +59,8 @@ XAP_Dialog* AP_Win32Dialog_PageSetup::static_constructor(XAP_DialogFactory* pDlg
 AP_Win32Dialog_PageSetup::AP_Win32Dialog_PageSetup(	XAP_DialogFactory* pDlgFactory,
 													XAP_Dialog_Id id)
 :	AP_Dialog_PageSetup (pDlgFactory, id),
-    m_PageSize(fp_PageSize::Letter)
+    m_PageSize(fp_PageSize::psLetter),
+	m_pWin32Frame(NULL)
 {
 }
 
@@ -74,7 +75,7 @@ void AP_Win32Dialog_PageSetup::runModal(XAP_Frame *pFrame)
 
 	// raise the dialog
 	XAP_Win32App * pWin32App = static_cast<XAP_Win32App *>(m_pApp);
-	XAP_Win32Frame * pWin32Frame = static_cast<XAP_Win32Frame *>(pFrame);
+	m_pWin32Frame = static_cast<XAP_Win32Frame *>(pFrame);
 
 	LPCTSTR lpTemplate = NULL;
 
@@ -83,7 +84,7 @@ void AP_Win32Dialog_PageSetup::runModal(XAP_Frame *pFrame)
 	lpTemplate = MAKEINTRESOURCE(AP_RID_DIALOG_PAGE_SETUP);
 
 	int result = DialogBoxParam(pWin32App->getInstance(),lpTemplate,
-								pWin32Frame->getTopLevelWindow(),
+								m_pWin32Frame->getTopLevelWindow(),
 								(DLGPROC)s_dlgProc,(LPARAM)this);
 	UT_ASSERT((result != -1));
 
@@ -143,9 +144,21 @@ typedef struct _tabParam
 	WORD which;
 } TabParam;
 
-
-#define _DS(c,s)	SetDlgItemText(hWnd,AP_RID_DIALOG_##c,pSS->getValue(AP_STRING_ID_##s))
-#define _DSX(c,s)	SetDlgItemText(hWnd,XAP_RID_DIALOG_##c,pSS->getValue(XAP_STRING_ID_##s))
+// As Tabbed Dialogs have problems with HotKeys, these macros have been replaced to remove &
+//#define _DS(c,s)	SetDlgItemText(hWnd,AP_RID_DIALOG_##c,pSS->getValue(AP_STRING_ID_##s))
+//#define _DSX(c,s)	SetDlgItemText(hWnd,XAP_RID_DIALOG_##c,pSS->getValue(XAP_STRING_ID_##s))
+#define _DS(c,s)  { \
+                    XML_Char* p = NULL; \
+                    UT_XML_cloneNoAmpersands( p, pSS->getValue(AP_STRING_ID_##s));\
+                    SetDlgItemText(hWnd,AP_RID_DIALOG_##c,p); \
+					FREEP(p); \
+                  }
+#define _DSX(c,s) { \
+                    XML_Char* p = NULL; \
+                    UT_XML_cloneNoAmpersands( p, pSS->getValue(XAP_STRING_ID_##s));\
+                    SetDlgItemText(hWnd,XAP_RID_DIALOG_##c,p); \
+					FREEP(p); \
+                  }
 #define _GV(s)		(pSS->getValue(AP_STRING_ID_##s))
 #define _GVX(s)		(pSS->getValue(XAP_STRING_ID_##s))
 
@@ -243,9 +256,19 @@ BOOL AP_Win32Dialog_PageSetup::_onCommand(HWND hWnd, WPARAM wParam, LPARAM lPara
 	switch (wId)
 	{
 	case AP_RID_DIALOG_PAGE_SETUP_BTN_OK:
-		setAnswer( a_OK );
-		setPageSize( m_PageSize );
-		EndDialog(hWnd,0);
+		if ( validatePageSettings() ) 
+		{
+			setAnswer (a_OK);
+			setPageSize( m_PageSize );
+			EndDialog(hWnd,0);
+		}
+		else 
+		{
+			// "The margins selected are too large to fit on the page."
+			m_pWin32Frame->showMessageBox(AP_STRING_ID_DLG_PageSetup_ErrBigMargins, 
+									 	  XAP_Dialog_MessageBox::b_O,
+									 	  XAP_Dialog_MessageBox::a_OK);
+		}
 		return 0;
 
 	case AP_RID_DIALOG_PAGE_SETUP_BTN_CANCEL:
@@ -354,14 +377,14 @@ BOOL AP_Win32Dialog_PageSetup::_onInitTab(HWND hWnd, WPARAM wParam, LPARAM lPara
 			_DS(PAGE_SETUP_LBL_WITDH,			DLG_PageSetup_Width);			
 			_DS(PAGE_SETUP_LBL_HEIGHT,			DLG_PageSetup_Height);			
 			_DS(PAGE_SETUP_LBL_UNITS,			DLG_PageSetup_Units);			
-			_DS(PAGE_SETUP_LBL_PORTRAIT,		DLG_PageSetup_Portrait);			
-			_DS(PAGE_SETUP_LBL_LANDSCAPE,		DLG_PageSetup_Landscape);			
+			_DS(PAGE_SETUP_RDO_PORTRAIT,		DLG_PageSetup_Portrait);			
+			_DS(PAGE_SETUP_RDO_LANDSCAPE,		DLG_PageSetup_Landscape);			
 			_DS(PAGE_SETUP_LBL_ADJUSTTO,		DLG_PageSetup_Adjust);			
 			_DS(PAGE_SETUP_LBL_PERCENTOFSIZE,	DLG_PageSetup_Percent);			
 
 			// Populate Combo and List Boxes
 			HWND hwndPaperSize = GetDlgItem(hWnd, AP_RID_DIALOG_PAGE_SETUP_LBX_PAPERSIZE);
-			for (UT_uint32 i = 0; i < (UT_uint32)fp_PageSize::_last_predefined_pagesize_dont_use_; i++)
+			for (UT_uint32 i = (UT_uint32)fp_PageSize::_first_predefined_pagesize_; i < (UT_uint32)fp_PageSize::_last_predefined_pagesize_dont_use_; i++)
 			{
 				SendMessage( hwndPaperSize,
                              CB_INSERTSTRING ,
@@ -369,11 +392,21 @@ BOOL AP_Win32Dialog_PageSetup::_onInitTab(HWND hWnd, WPARAM wParam, LPARAM lPara
 						     (LPARAM) fp_PageSize::PredefinedToName( (fp_PageSize::Predefined)i ) );
 			}
 			HWND hwndUnits = GetDlgItem(hWnd, AP_RID_DIALOG_PAGE_SETUP_LBX_UNITS);
-			SendMessage( hwndUnits, CB_INSERTSTRING , (WPARAM) fp_PageSize::mm,   (LPARAM) _GVX(DLG_Unit_mm) );                                
-			SendMessage( hwndUnits, CB_INSERTSTRING , (WPARAM) fp_PageSize::cm,   (LPARAM) _GVX(DLG_Unit_cm) );                                
-			SendMessage( hwndUnits, CB_INSERTSTRING , (WPARAM) fp_PageSize::inch, (LPARAM) _GVX(DLG_Unit_inch) );                                
+			// NB: cannot insert string at index 1 before inserting one at index 0
+			SendMessage( hwndUnits, CB_INSERTSTRING , (WPARAM) DIM_IN, (LPARAM) _GVX(DLG_Unit_inch) );                                
+			SendMessage( hwndUnits, CB_INSERTSTRING , (WPARAM) DIM_CM,   (LPARAM) _GVX(DLG_Unit_cm) );                                
+			SendMessage( hwndUnits, CB_INSERTSTRING , (WPARAM) DIM_MM,   (LPARAM) _GVX(DLG_Unit_mm) );                                
 			
 			// Initialize Data
+			if( getPageOrientation() == PORTRAIT )
+			{
+				m_PageSize.setPortrait();
+			}
+			else
+			{
+				m_PageSize.setLandscape();
+			}
+
 			char buf[BUFSIZE];
 			updateWidth();
 			updateHeight();
@@ -444,6 +477,7 @@ BOOL AP_Win32Dialog_PageSetup::_onInitTab(HWND hWnd, WPARAM wParam, LPARAM lPara
 	case AP_RID_DIALOG_PAGE_SETUP_MARGINS:
 		{
 			// Localize Controls
+			_DS(PAGE_SETUP_LBL_UNITS,			DLG_PageSetup_Units);			
 			_DS(PAGE_SETUP_LBL_TOP,				DLG_PageSetup_Top);
 			_DS(PAGE_SETUP_LBL_BOTTOM,			DLG_PageSetup_Bottom);
 			_DS(PAGE_SETUP_LBL_LEFT,	     	DLG_PageSetup_Left);
@@ -451,9 +485,9 @@ BOOL AP_Win32Dialog_PageSetup::_onInitTab(HWND hWnd, WPARAM wParam, LPARAM lPara
 			_DS(PAGE_SETUP_LBL_HEADER,			DLG_PageSetup_Header);			
 			_DS(PAGE_SETUP_LBL_FOOTER,			DLG_PageSetup_Footer);			
 			HWND hwndMarginUnits = GetDlgItem(hWnd, AP_RID_DIALOG_PAGE_SETUP_LBX_MARGINUNITS);
-			SendMessage( hwndMarginUnits, CB_INSERTSTRING , (WPARAM) fp_PageSize::mm,   (LPARAM) _GVX(DLG_Unit_mm) );                                
-			SendMessage( hwndMarginUnits, CB_INSERTSTRING , (WPARAM) fp_PageSize::cm,   (LPARAM) _GVX(DLG_Unit_cm) );                                
-			SendMessage( hwndMarginUnits, CB_INSERTSTRING , (WPARAM) fp_PageSize::inch, (LPARAM) _GVX(DLG_Unit_inch) );                                
+			SendMessage( hwndMarginUnits, CB_INSERTSTRING , (WPARAM) DIM_IN, (LPARAM) _GVX(DLG_Unit_inch) );                                
+			SendMessage( hwndMarginUnits, CB_INSERTSTRING , (WPARAM) DIM_CM,   (LPARAM) _GVX(DLG_Unit_cm) );                                
+			SendMessage( hwndMarginUnits, CB_INSERTSTRING , (WPARAM) DIM_MM,   (LPARAM) _GVX(DLG_Unit_mm) );                                
 
 			// Initialize Data
 			SendMessage( hwndMarginUnits, CB_SETCURSEL, (WPARAM) getMarginUnits(), (LPARAM) 0 );
@@ -487,14 +521,14 @@ BOOL AP_Win32Dialog_PageSetup::_onCommandTab(HWND hWnd, WPARAM wParam, LPARAM lP
 			if( selected != previous )
 			{
 				m_PageSize.Set( (fp_PageSize::Predefined)selected );
-				if( getPageUnits() != m_PageSize.getUnit() )
+				if( getPageUnits() != m_PageSize.getDims() )
 				{
 					SendDlgItemMessage( hWnd,
 								 		AP_RID_DIALOG_PAGE_SETUP_LBX_UNITS,
 								 		CB_SETCURSEL, 
-								 		(WPARAM) m_PageSize.getUnit(),
+								 		(WPARAM) m_PageSize.getDims(),
 							 			(LPARAM) 0);
-					setPageUnits( m_PageSize.getUnit() );
+					setPageUnits( m_PageSize.getDims() );
 				}
 				updateWidth();
 				updateHeight();
@@ -506,7 +540,7 @@ BOOL AP_Win32Dialog_PageSetup::_onCommandTab(HWND hWnd, WPARAM wParam, LPARAM lP
 	case AP_RID_DIALOG_PAGE_SETUP_LBX_UNITS:
 		if( wNotifyCode == CBN_SELCHANGE )
 		{
-			fp_PageSize::Unit unit = (fp_PageSize::Unit)SendMessage( hWndCtrl, CB_GETCURSEL, (WPARAM) 0, (LPARAM) 0 );
+			UT_Dimension unit = (UT_Dimension)SendMessage( hWndCtrl, CB_GETCURSEL, (WPARAM) 0, (LPARAM) 0 );
 			if( unit != getPageUnits() )
 			{
 				setPageUnits( unit );
@@ -605,7 +639,7 @@ BOOL AP_Win32Dialog_PageSetup::_onCommandTab(HWND hWnd, WPARAM wParam, LPARAM lP
 		if( wNotifyCode == CBN_SELCHANGE )
 		{
 			UT_sint32 selected = SendMessage( hWndCtrl, CB_GETCURSEL, (WPARAM) 0, (LPARAM) 0 );
-			if( getMarginUnits() != (fp_PageSize::Unit) selected )
+			if( getMarginUnits() != (UT_Dimension) selected )
 			{
 				setMarginTop(    getMarginTop()   * mScale[getMarginUnits()] / mScale[selected] );
 				setMarginBottom( getMarginBottom()* mScale[getMarginUnits()] / mScale[selected] );
@@ -614,7 +648,7 @@ BOOL AP_Win32Dialog_PageSetup::_onCommandTab(HWND hWnd, WPARAM wParam, LPARAM lP
 				setMarginHeader( getMarginHeader()* mScale[getMarginUnits()] / mScale[selected] );
 				setMarginFooter( getMarginFooter()* mScale[getMarginUnits()] / mScale[selected] );
 				updateMargins();
-				setMarginUnits( (fp_PageSize::Unit) selected );
+				setMarginUnits( (UT_Dimension) selected );
 			}
 		}
 		return 0;
@@ -730,13 +764,13 @@ void AP_Win32Dialog_PageSetup::doSpinControl(UT_uint32 id, UT_sint32 delta)
 {
 	char buf[BUFSIZE];
 	int updatedData =  0;
-	int pageScale   = ( getPageUnits()   == fp_PageSize::mm ) ? 1 : 10;
-	int marginScale = ( getMarginUnits() == fp_PageSize::mm ) ? 1 : 10;
+	int pageScale   = ( getPageUnits()   == DIM_MM ) ? 1 : 10;
+	int marginScale = ( getMarginUnits() == DIM_MM ) ? 1 : 10;
 
 	switch( id )
 	{
 	case AP_RID_DIALOG_PAGE_SETUP_SPN_WIDTH:
-		updatedData = (int)( m_PageSize.Width(getPageUnits()) * pageScale + delta );
+		updatedData = (int)( m_PageSize.Width(getPageUnits()) * pageScale + delta + 0.05f );
 		if( updatedData >= 0 )
 		{
 			m_PageSize.Set( (double) updatedData/pageScale,
@@ -749,7 +783,7 @@ void AP_Win32Dialog_PageSetup::doSpinControl(UT_uint32 id, UT_sint32 delta)
 		break;
 
 	case AP_RID_DIALOG_PAGE_SETUP_SPN_HEIGHT:
-		updatedData = (int)( m_PageSize.Height(getPageUnits()) * pageScale + delta );
+		updatedData = (int)( m_PageSize.Height(getPageUnits()) * pageScale + delta + 0.05f );
 		if( updatedData >= 0 )
 		{
 			m_PageSize.Set( m_PageSize.Width(getPageUnits()),
@@ -949,8 +983,8 @@ void AP_Win32Dialog_PageSetup::updatePreview()
 	UT_uint16 borderWidth  = rectBorder.right  - rectBorder.left - offset;
 	UT_uint16 borderHeight = rectBorder.bottom - rectBorder.top  - offset;
 
-	UT_uint16 pageWidth  = (UT_uint16) m_PageSize.Width( (fp_PageSize::Unit)0 );  // in mm
-	UT_uint16 pageHeight = (UT_uint16) m_PageSize.Height( (fp_PageSize::Unit)0 ); // in mm
+	UT_uint16 pageWidth  = (UT_uint16) m_PageSize.Width( DIM_MM );  // in mm
+	UT_uint16 pageHeight = (UT_uint16) m_PageSize.Height( DIM_MM ); // in mm
 	UT_uint16 marginTop  = (UT_uint16) (getMarginTop() * mScale[getMarginUnits()] );
 	UT_uint16 marginLeft = (UT_uint16) (getMarginLeft() * mScale[getMarginUnits()] );
 	UT_uint16 marginRight= (UT_uint16) (getMarginRight() * mScale[getMarginUnits()] );
