@@ -57,7 +57,7 @@ AP_Dialog_Paragraph::AP_Dialog_Paragraph(XAP_DialogFactory* pDlgFactory, XAP_Dia
 	
 	XAP_Prefs* pPrefs = m_pApp->getPrefs();
 	UT_ASSERT(pPrefs);
-	
+
 	const bool bHasRulerUnits =
 		pPrefs->getPrefsValue((XML_Char*)AP_PREF_KEY_RulerUnits, &szRulerUnits);
 
@@ -363,7 +363,7 @@ bool AP_Dialog_Paragraph::setDialogData(const XML_Char ** pProps)
 
 #define ALLOC_PROP_PAIR(p)									\
         do {                								\
-            p = (propPair *) calloc(1, sizeof(propPair));	\
+            p = (propPair *) UT_calloc(1, sizeof(propPair));	\
             if (!p) return false;						\
         } while (0)											\
 
@@ -525,7 +525,7 @@ bool AP_Dialog_Paragraph::getDialogData(const XML_Char **& pProps)
 			break;
 		case spacing_ATLEAST:
 			nSize = UT_XML_strlen(pString);
-			pTmp = (XML_Char *) calloc(nSize + 2, sizeof(XML_Char));
+			pTmp = (XML_Char *) UT_calloc(nSize + 2, sizeof(XML_Char));
 			UT_ASSERT(pTmp);
 
 			UT_XML_strncpy(pTmp, nSize, pString);
@@ -621,7 +621,7 @@ bool AP_Dialog_Paragraph::getDialogData(const XML_Char **& pProps)
 	// export everything in the array
 	UT_uint32 count = v.getItemCount()*2 + 1;
 
-	const XML_Char ** newprops = (const XML_Char **) calloc(count, sizeof(XML_Char *));
+	const XML_Char ** newprops = (const XML_Char **) UT_calloc(count, sizeof(XML_Char *));
 	if (!newprops)
 		return false;
 
@@ -824,11 +824,20 @@ void AP_Dialog_Paragraph::_setSpinItemValue(tControl item, const XML_Char * valu
 		
 	case id_SPIN_BEFORE_SPACING:
 	case id_SPIN_AFTER_SPACING:
-	case id_SPIN_SPECIAL_SPACING:
 		// NOTE : line spacing can't be negative, so make absolut
 
-		UT_XML_strncpy((XML_Char *) pItem->pData, SPIN_BUF_TEXT_SIZE, UT_reformatDimensionString(DIM_PT, _makeAbsolute(value)));			
-
+		UT_XML_strncpy((XML_Char *) pItem->pData, SPIN_BUF_TEXT_SIZE, UT_reformatDimensionString(DIM_PT, _makeAbsolute(value)));
+		break;
+		
+	case id_SPIN_SPECIAL_SPACING:
+		if(_getMenuItemValue(id_MENU_SPECIAL_SPACING) == spacing_MULTIPLE)
+		{
+			UT_XML_strncpy((XML_Char *) pItem->pData, SPIN_BUF_TEXT_SIZE, UT_reformatDimensionString(DIM_none, _makeAbsolute(value),".2"));
+		}
+		else
+		{
+			UT_XML_strncpy((XML_Char *) pItem->pData, SPIN_BUF_TEXT_SIZE, UT_reformatDimensionString(DIM_PT, _makeAbsolute(value)));
+		}
 		break;
 
 	default:
@@ -920,7 +929,7 @@ void AP_Dialog_Paragraph::_doSpin(tControl edit, UT_sint32 amt)
 			// fall through
 		case spacing_MULTIPLE:
 			dimSpin = DIM_none;
-			dSpinUnit = 0.5;
+			dSpinUnit = 0.1;
 			dMin = 0.5;
 			bMin = true;
 			break;
