@@ -70,8 +70,9 @@ EV_Menu_ActionSet * AP_CreateMenuActionSet(void)
 	_s(AP_MENU_ID_FILE,				1,0,0,	NULL,				NULL,					NULL);
 	_s(AP_MENU_ID_FILE_NEW,			0,1,0,	"fileNew",			NULL,					NULL);
 	_s(AP_MENU_ID_FILE_OPEN,		0,1,0,	"fileOpen",			NULL,					NULL);
-	_s(AP_MENU_ID_FILE_SAVE,		0,0,0,	"fileSave",			NULL,					NULL);
+	_s(AP_MENU_ID_FILE_SAVE,		0,0,0,	"fileSave",			ap_GetState_Changes,					NULL);
 	_s(AP_MENU_ID_FILE_SAVEAS,		0,1,0,	"fileSaveAs",		NULL,					NULL);
+	_s(AP_MENU_ID_FILE_SAVEIMAGE,		0,1,0,	"fileSaveImage",		NULL,					NULL);
 	_s(AP_MENU_ID_FILE_IMPORT, 0,1,0, "fileImport", NULL, NULL);
 	_s(AP_MENU_ID_FILE_EXPORT, 0,1,0, "fileExport", NULL, NULL);
 	_s(AP_MENU_ID_FILE_CLOSE,		0,0,0,	"closeWindow",		NULL,					NULL);
@@ -90,6 +91,7 @@ EV_Menu_ActionSet * AP_CreateMenuActionSet(void)
 	_s(AP_MENU_ID_FILE_RECENT_7,	0,0,0,	"openRecent_7",		NULL,					ap_GetLabel_Recent);
 	_s(AP_MENU_ID_FILE_RECENT_8,	0,0,0,	"openRecent_8",		NULL,					ap_GetLabel_Recent);
 	_s(AP_MENU_ID_FILE_RECENT_9,	0,0,0,	"openRecent_9",		NULL,					ap_GetLabel_Recent);
+	_s(AP_MENU_ID_FILE_REVERT, 0,0,0, "fileRevert", ap_GetState_Changes, NULL);
 	_s(AP_MENU_ID_FILE_EXIT,		0,0,0,	"querySaveAndExit",	NULL,					NULL);
 	_s(AP_MENU_ID_OPEN_TEMPLATE, 0,1,0, "openTemplate", NULL, NULL);
 
@@ -105,8 +107,8 @@ EV_Menu_ActionSet * AP_CreateMenuActionSet(void)
 	_s(AP_MENU_ID_EDIT_FIND,		0,1,0,	"find",				NULL,					NULL);
 	_s(AP_MENU_ID_EDIT_REPLACE,		0,1,0,	"replace",			NULL,					NULL);
 	_s(AP_MENU_ID_EDIT_GOTO,		0,1,0,	"go",				NULL,					NULL);
-	_s(AP_MENU_ID_EDIT_EDITHEADER,		0,0,0,	"editHeader",	NULL,					NULL);
-	_s(AP_MENU_ID_EDIT_EDITFOOTER,		0,0,0,	"editFooter",	NULL,					NULL);
+	_s(AP_MENU_ID_EDIT_EDITHEADER,		0,0,0,	"editHeader",	ap_GetState_Changes,					NULL);
+	_s(AP_MENU_ID_EDIT_EDITFOOTER,		0,0,0,	"editFooter",	ap_GetState_Changes,					NULL);
 	_s(AP_MENU_ID_EDIT_REMOVEHEADER,		0,0,0,"removeHeader",ap_GetState_Changes	,					NULL);
 	_s(AP_MENU_ID_EDIT_REMOVEFOOTER,		0,0,0,	"removeFooter",ap_GetState_Changes,					NULL);
 
@@ -134,26 +136,37 @@ EV_Menu_ActionSet * AP_CreateMenuActionSet(void)
 
 	_s(AP_MENU_ID_INSERT,			1,0,0,	NULL,				NULL,					NULL);
 	_s(AP_MENU_ID_INSERT_BREAK,		0,1,0,	"insBreak",			NULL,					NULL);
+	_s(AP_MENU_ID_INSERT_BOOKMARK,  0,1,0,	"insertBookmark",	NULL,					NULL);
+	_s(AP_MENU_ID_INSERT_HYPERLINK, 0,1,0,	"insertHyperlink",	ap_GetState_SomethingSelected,					NULL);
+	_s(AP_MENU_ID_INSERT_DELETE_HYPERLINK, 0,0,0,	"deleteHyperlink",	NULL,			NULL);
 	_s(AP_MENU_ID_INSERT_PAGENO,	0,1,0,	"insPageNo",		NULL,					NULL);
 	_s(AP_MENU_ID_INSERT_DATETIME,	0,1,0,	"insDateTime",		NULL,					NULL);
 	_s(AP_MENU_ID_INSERT_FIELD,		0,1,0,	"insField",			NULL,					NULL);
+	_s(AP_MENU_ID_INSERT_FILE, 0,1,0, "insFile", NULL, NULL);
 	_s(AP_MENU_ID_INSERT_SYMBOL,	0,1,0,	"insSymbol",		NULL,					NULL);
 #ifdef DEBUG
 	// QUACK Un-DEBUG here to activate endnotes!
 	_s(AP_MENU_ID_INSERT_ENDNOTE,	0,0,0,	"insEndnote",		NULL,					NULL);
 #endif
+#ifdef HAVE_GNOME
 	_s(AP_MENU_ID_INSERT_PICTURE,   1,0,0,  NULL, NULL, NULL);
+#else
+	_s(AP_MENU_ID_INSERT_PICTURE,   0,1,0,  "fileInsertGraphic",NULL,                   NULL);
+#endif
 	_s(AP_MENU_ID_INSERT_CLIPART,   0,1,0,  "insertClipart",    NULL,                   NULL);
 	_s(AP_MENU_ID_INSERT_GRAPHIC,	0,1,0,	"fileInsertGraphic",NULL,					NULL);
 	_s(AP_MENU_ID_FORMAT,			1,0,0,	NULL,				NULL,					NULL);
+	_s(AP_MENU_ID_FMT,			1,0,0,	NULL,				NULL,					NULL);
 	_s(AP_MENU_ID_FMT_LANGUAGE,			0,1,0,	"dlgLanguage",	NULL,					NULL);
-	_s(AP_MENU_ID_FMT_FONT,			0,1,0,	"dlgFont",			NULL,					NULL);
-	_s(AP_MENU_ID_FMT_PARAGRAPH,	0,1,0,	"dlgParagraph",		NULL,					NULL);
-	_s(AP_MENU_ID_FMT_BULLETS,		0,1,0,	"dlgBullets",		NULL,					NULL);
+	_s(AP_MENU_ID_FMT_FONT,			0,1,0,	"dlgFont",	ap_GetState_StylesLocked,	NULL);
+	_s(AP_MENU_ID_FMT_PARAGRAPH,	0,1,0,	"dlgParagraph",		ap_GetState_StylesLocked,	NULL);
+	_s(AP_MENU_ID_FMT_BULLETS,		0,1,0,	"dlgBullets",	ap_GetState_StylesLocked,	NULL);
 	_s(AP_MENU_ID_FMT_DOCUMENT, 0,1,0, "pageSetup", NULL, NULL);
 	_s(AP_MENU_ID_FMT_BORDERS,		0,1,0,	"dlgBorders",		NULL,					NULL);
-	_s(AP_MENU_ID_FMT_COLUMNS,		0,1,0,	"dlgColumns",		NULL,					NULL);
+	_s(AP_MENU_ID_FMT_COLUMNS,		0,1,0,	"dlgColumns",		ap_GetState_ColumnsActive,					NULL);
 	_s(AP_MENU_ID_FMT_BACKGROUND, 0,1,0, "dlgBackground", NULL, NULL);
+	_s(AP_MENU_ID_FMT_HDRFTR,     0,1,0, "dlgHdrFtr", NULL, NULL);
+	_s(AP_MENU_ID_FMT_IMAGE, 0,1,0, "dlgFmtImage", NULL, NULL);
 	_s(AP_MENU_ID_FMT_STYLE,		0,1,0,	"dlgStyle",			NULL,					NULL);
 	_s(AP_MENU_ID_FMT_TABS,			0,1,0,	"dlgTabs",			NULL,					NULL);
 	_s(AP_MENU_ID_FMT_BOLD,			0,0,1,	"toggleBold",		ap_GetState_CharFmt,	NULL);
@@ -165,26 +178,24 @@ EV_Menu_ActionSet * AP_CreateMenuActionSet(void)
 	_s(AP_MENU_ID_FMT_BOTTOMLINE,		0,0,1,	"toggleBottomline",		ap_GetState_CharFmt,	NULL);
 	_s(AP_MENU_ID_FMT_SUPERSCRIPT,		0,0,1,	"toggleSuper",	ap_GetState_CharFmt,	NULL);
 	_s(AP_MENU_ID_FMT_SUBSCRIPT,		0,0,1,	"toggleSub",	ap_GetState_CharFmt,	NULL);
-	_s(AP_MENU_ID_FMT_TOGGLECASE,           0,1,0,  "dlgToggleCase", NULL, NULL);
+	_s(AP_MENU_ID_FMT_TOGGLECASE,           0,1,0,  "dlgToggleCase", ap_GetState_SomethingSelected, NULL);
 
-	_s(AP_MENU_ID_ALIGN,			1,0,0,	NULL,				NULL,					NULL);
+	_s(AP_MENU_ID_ALIGN,			1,0,0,	NULL,				ap_GetState_StylesLocked,	NULL);
 	_s(AP_MENU_ID_ALIGN_LEFT,		0,0,1,	"alignLeft",		ap_GetState_BlockFmt,	NULL);
 	_s(AP_MENU_ID_ALIGN_CENTER,		0,0,1,	"alignCenter",		ap_GetState_BlockFmt,	NULL);
 	_s(AP_MENU_ID_ALIGN_RIGHT,		0,0,1,	"alignRight",		ap_GetState_BlockFmt,	NULL);
 	_s(AP_MENU_ID_ALIGN_JUSTIFY,	0,0,1,	"alignJustify",		ap_GetState_BlockFmt,	NULL);
 
 	_s(AP_MENU_ID_TOOLS,			1,0,0,	NULL,				NULL,					NULL);
-	_s(AP_MENU_ID_TOOLS_SPELLING,	        1,0,0,	NULL,				NULL,					NULL);
-	_s(AP_MENU_ID_TOOLS_SPELL,	        0,1,0,	"dlgSpell",		NULL,					NULL);
+	_s(AP_MENU_ID_TOOLS_SPELLING,	        1,0,0,	NULL,				NULL,				       NULL);
+	_s(AP_MENU_ID_TOOLS_SPELL,	        0,1,0,	"dlgSpell",		ap_GetState_Spelling,					NULL);
 	_s(AP_MENU_ID_TOOLS_SPELLPREFS, 0,1,0, "dlgSpellPrefs", NULL, NULL);
 	_s(AP_MENU_ID_TOOLS_AUTOSPELL,          0,0,1,  "toggleAutoSpell",      ap_GetState_Prefs, NULL);
 	_s(AP_MENU_ID_TOOLS_LANGUAGE, 1,0,0, NULL, NULL, NULL);
 	_s(AP_MENU_ID_TOOLS_WORDCOUNT,		0,1,0,	"dlgWordCount",			NULL,					NULL);
 	_s(AP_MENU_ID_TOOLS_PLUGINS, 0,1,0, "dlgPlugins", NULL, NULL);
 	_s(AP_MENU_ID_TOOLS_OPTIONS,		0,1,0,	"dlgOptions",		NULL,					NULL);
-#ifdef ABI_OPT_PERL
-	_s(AP_MENU_ID_TOOLS_SCRIPTS,	1,0,0,	"scriptPlay",		NULL, NULL);
-#endif
+	_s(AP_MENU_ID_TOOLS_SCRIPTS,	1,0,0,	"scriptPlay", ap_GetState_ScriptsActive, NULL);
 
 	_s(AP_MENU_ID_WINDOW,			1,0,0,	NULL,				NULL,					NULL);
 	_s(AP_MENU_ID_WINDOW_NEW,		0,0,0,	"newWindow",		NULL,					NULL);
@@ -213,6 +224,7 @@ EV_Menu_ActionSet * AP_CreateMenuActionSet(void)
 	_s(AP_MENU_ID_HELP_ABOUT_GNU, 0,0,0, "helpAboutGnu", NULL, NULL);
 	_s(AP_MENU_ID_HELP_ABOUT_GNOMEOFFICE, 0,0,0, "helpAboutGnomeOffice", NULL, NULL);
 	_s(AP_MENU_ID_HELP_CREDITS, 0,0,0, "helpCredits", NULL, NULL);
+	_s(AP_MENU_ID_HELP_REPORT_BUG, 0,0,0, "helpReportBug", NULL, NULL);
 
 	_s(AP_MENU_ID_SPELL_SUGGEST_1,	0,0,0,	"spellSuggest_1",	ap_GetState_Suggest,	ap_GetLabel_Suggest);
 	_s(AP_MENU_ID_SPELL_SUGGEST_2,	0,0,0,	"spellSuggest_2",	ap_GetState_Suggest,	ap_GetLabel_Suggest);
