@@ -28,7 +28,7 @@ class s_AbiWord_1_Listener;
 
 // The exporter/writer for AbiWord file format version 1.
 
-class IE_Exp_AbiWord_1_Sniffer : public IE_ExpSniffer
+class ABI_EXPORT IE_Exp_AbiWord_1_Sniffer : public IE_ExpSniffer
 {
 	friend class IE_Exp;
 
@@ -44,7 +44,7 @@ public:
 										IE_Exp ** ppie);
 };
 
-class IE_Exp_AbiWord_1 : public IE_Exp
+class ABI_EXPORT IE_Exp_AbiWord_1 : public IE_Exp
 {
 public:
 	IE_Exp_AbiWord_1(PD_Document * pDocument, bool isTemplate = false);
