@@ -36,6 +36,7 @@
 #include "xap_App.h"
 #include "xap_EncodingManager.h"
 
+#include "ut_path.h"
 #include "ut_string_class.h"
 /*****************************************************************/
 /*****************************************************************/
@@ -98,7 +99,7 @@ ABI_FAR_CALL
 int abi_plugin_supports_version (UT_uint32 major, UT_uint32 minor, 
 								 UT_uint32 release)
 {
-	return isCurrentAbiVersion (major, minor, release) ? 1 : 0;
+  return 1;
 }
 
 #endif
@@ -193,6 +194,7 @@ protected:
 	void				_convertFontSize(char* szDest, const char* pszFontSize);
 	void				_convertColor(char* szDest, const char* pszColor);
 	
+private:
 	PD_Document *		        m_pDocument;
 	IE_Exp_DocBook *		m_pie;
 	bool				m_bInSection;
@@ -203,8 +205,10 @@ protected:
 	// Need to look up proper type, and place to stick #defines...
 
 	UT_uint16		m_iBlockType;	// BT_*
-        bool                 m_bWasSpace;
+    bool                 m_bWasSpace;
 
+	UT_Vector		m_utvDataIDs;	// list of data ids for image enumeration
+	char *          _stripSuffix(const char* from, char delimiter);
 };
 
 void s_DocBook_Listener::_closeSection(void)
@@ -214,7 +218,7 @@ void s_DocBook_Listener::_closeSection(void)
 		return;
 	}
 	
-	m_pie->write("</section>\n");
+	m_pie->write("</section>\n</chapter>\n");
 	m_bInSection = false;
 	return;
 }
@@ -344,7 +348,7 @@ void s_DocBook_Listener::_openParagraph(PT_AttrPropIndex api)
 
 void s_DocBook_Listener::_openSection(PT_AttrPropIndex /* api*/)
 {
-	m_pie->write("<section>\n<title></title>\n");
+	m_pie->write("<chapter><section>\n<title></title>\n");
 }
 
 
@@ -529,10 +533,7 @@ void s_DocBook_Listener::_outputData(const UT_UCSChar * data, UT_uint32 length)
 					UT_UCSChar c = XAP_EncodingManager::get_instance()->try_UToNative(*pData);
 					if (c==0 || c>255)
 					{
-						char localBuf[20];
-						char * plocal = localBuf;
-						sprintf(localBuf,"&#x%x;",*pData++);
-						sBuf += plocal;
+						sBuf += UT_String_sprintf("&#x%x;",*pData++);
 					}
 					else
 					{
@@ -572,45 +573,6 @@ s_DocBook_Listener::s_DocBook_Listener(PD_Document * pDocument,
 	m_pie->write("<!-- AbiWord is a free, Open Source word processor.                                   -->\n");
 	m_pie->write("<!-- You may obtain more information about AbiWord at www.abisource.com               -->\n");
 	m_pie->write("<!-- ================================================================================ -->\n");
-	m_pie->write("\n");
-
-	if (XAP_App::s_szBuild_ID && XAP_App::s_szBuild_ID[0])
-	{
-		m_pie->write("<!--         Build_ID          = ");
-		m_pie->write(XAP_App::s_szBuild_ID);
-		m_pie->write(" -->\n");
-	}
-	if (XAP_App::s_szBuild_Version && XAP_App::s_szBuild_Version[0])
-	{
-		m_pie->write("<!--         Build_Version     = ");
-		m_pie->write(XAP_App::s_szBuild_Version);
-		m_pie->write(" -->\n");
-	}
-	if (XAP_App::s_szBuild_Options && XAP_App::s_szBuild_Options[0])
-	{
-		m_pie->write("<!--         Build_Options     = ");
-		m_pie->write(XAP_App::s_szBuild_Options);
-		m_pie->write(" -->\n");
-	}
-	if (XAP_App::s_szBuild_Target && XAP_App::s_szBuild_Target[0])
-	{
-		m_pie->write("<!--         Build_Target      = ");
-		m_pie->write(XAP_App::s_szBuild_Target);
-		m_pie->write(" -->\n");
-	}
-	if (XAP_App::s_szBuild_CompileTime && XAP_App::s_szBuild_CompileTime[0])
-	{
-		m_pie->write("<!--         Build_CompileTime = ");
-		m_pie->write(XAP_App::s_szBuild_CompileTime);
-		m_pie->write(" -->\n");
-	}
-	if (XAP_App::s_szBuild_CompileDate && XAP_App::s_szBuild_CompileDate[0])
-	{
-		m_pie->write("<!--         Build_CompileDate = ");
-		m_pie->write(XAP_App::s_szBuild_CompileDate);
-		m_pie->write(" -->\n");
-	}
-	
 	m_pie->write("\n\n");
 }
 
@@ -649,26 +611,58 @@ bool s_DocBook_Listener::populate(PL_StruxFmtHandle /*sfh*/,
 
 	case PX_ChangeRecord::PXT_InsertObject:
 		{
-#if 0			
 			const PX_ChangeRecord_Object * pcro = static_cast<const PX_ChangeRecord_Object *> (pcr);
+			const XML_Char* szValue;
 			PT_AttrPropIndex api = pcr->getIndexAP();
+			UT_String buf;
+
+			const PP_AttrProp * pAP = NULL;
+			bool bHaveProp = m_pDocument->getAttrProp(api,&pAP);
+
 			switch (pcro->getObjectType())
 			{
 			case PTO_Image:
-				// TODO we *could* insert the images and create separate GIF files.
+				// Lifted from HTML exporter.
+
+				if(bHaveProp && pAP && pAP->getAttribute("dataid", szValue))
+				{
+					char* dataid = strdup((char*) szValue);
+
+					m_utvDataIDs.push_back(dataid);
+
+					char * temp = _stripSuffix(UT_basename(szValue), '_');
+					char * fstripped = _stripSuffix(temp, '.');
+					FREEP(temp);
+					UT_String_sprintf(buf, "%s.png", fstripped);
+					FREEP(fstripped);
+					
+					m_pie->write("<figure>\n<title>");
+					m_pie->write(buf);
+					m_pie->write("</title>\n<mediaobject><imageobject>");
+                    m_pie->write("<imagedata fileref=\"");
+					m_pie->write(UT_basename(m_pie->getFileName()));
+					m_pie->write("_data/");
+					m_pie->write(buf);
+					m_pie->write("\" format=\"PNG\"></imagedata></imageobject>");
+                    m_pie->write("</mediaobject></figure>");
+					
+				}
 				return true;
 
 			case PTO_Field:
 				// we do nothing with computed fields.
 				return true;
 
+				// todo: support these
+			case PTO_Hyperlink:
+			case PTO_Bookmark:
+			  return true;
+
 			default:
 				UT_ASSERT(0);
 				return false;
 			}
-#else
-			return true;
-#endif
+			return false;
 		}
 
 	case PX_ChangeRecord::PXT_InsertFmtMark:
@@ -816,14 +810,90 @@ UT_Error IE_Exp_DocBook::_writeDocument(void)
 /*****************************************************************/
 /*****************************************************************/
 
-void s_DocBook_Listener::_handleDataItems(void)
+/*!
+   removes the suffix from a string by searching backwards for the specified 
+   character delimiter. If the delimiter is not found, a copy of the original 
+   string is returned
+   
+   eg. _stripSuffix("/home/user/file.png, '.') returns "/home/user/file" 
+       _stripSuffix("/home/user/foo_bar, '_') returns /home/user/foo 
+       _stripSuffix("/home/user/file.png, '_') returns /home/user/file.png"
+   TODO: put this in UT_String somehow, it came from ie_exp_HTML.
+*/
+char *s_DocBook_Listener::_stripSuffix(const char* from, char delimiter)
 {
+    char * fremove_s = (char *)malloc(strlen(from)+1);
+    strcpy(fremove_s, from);   
+
+    char * p = fremove_s + strlen(fremove_s);
+    while ((p >= fremove_s) && (*p != delimiter))
+        p--;
+	
+    if (p >= fremove_s)
+	*p = '\0';
+    
+    return fremove_s;
 }
 
-
-
-
-
-
-
-
+void s_DocBook_Listener::_handleDataItems(void)
+{
+	// Lifted from HTML listener
+ 	const char * szName;
+	const char * szMimeType;
+	const UT_ByteBuf * pByteBuf;
+	
+	for (UT_uint32 k=0; (m_pDocument->enumDataItems(k,NULL,&szName,&pByteBuf,(void**)&szMimeType)); k++)
+	{
+		UT_sint32 loc = -1;
+		for (UT_uint32 i = 0; i < m_utvDataIDs.getItemCount(); i++)
+		{
+			if(UT_strcmp((char*) m_utvDataIDs[i], szName) == 0)
+			{
+				loc = i;
+				break;
+			}
+		}
+		
+		if(loc > -1)
+		{
+			FILE *fp;
+			UT_String fname; // EVIL EVIL bad hardcoded buffer size
+			
+			UT_String_sprintf(fname, "%s_data", m_pie->getFileName());
+			int result = m_pDocument->getApp()->makeDirectory(fname.c_str(), 0750);
+			
+			if (!UT_strcmp(szMimeType, "image/svg-xml"))
+				UT_String_sprintf(fname, "%s/%s_%d.svg", fname.c_str(), szName, loc);
+			if (!UT_strcmp(szMimeType, "text/mathml"))
+				UT_String_sprintf(fname, "%s/%s_%d.mathml", fname.c_str(), szName, loc);
+			else // PNG Image
+			{  
+				char * temp = _stripSuffix(UT_basename(szName), '_');
+				char * fstripped = _stripSuffix(temp, '.');
+				FREEP(temp);
+				UT_String_sprintf(fname, "%s/%s.png", fname.c_str(), fstripped);
+				FREEP(fstripped);
+			}
+			
+			if (!UT_isRegularFile(fname.c_str()))
+			{
+			    fp = fopen (fname.c_str(), "wb+");
+			
+			    if(!fp)
+				    continue;
+			
+			    int cnt = 0, len = pByteBuf->getLength();
+			
+			    while (cnt < len)
+			    {
+				    cnt += fwrite (pByteBuf->getPointer(cnt), 
+							     sizeof(UT_Byte), len-cnt, fp);
+			    }
+			
+			    fclose(fp);
+			}
+		}
+	}
+	
+	return;
+}
