@@ -50,85 +50,7 @@
 #include "ut_string_class.h"
 
 // We terminate each line with a \r\n sequence to make IE think that
-// our XHTML is really HTML. This is a stupid IE bug
-
-/*****************************************************************/
-/*****************************************************************/
-
-#ifdef ENABLE_PLUGINS
-
-// completely generic code to allow this to be a plugin
-
-#include "xap_Module.h"
-
-ABI_PLUGIN_DECLARE("HTML")
-
-// we use a reference-counted sniffer
-static IE_Exp_HTML_Sniffer * m_sniffer = 0;
-static IE_Exp_HTML4_Sniffer * m_4sniffer = 0;
-
-ABI_FAR_CALL
-int abi_plugin_register (XAP_ModuleInfo * mi)
-{
-
-	if (!m_sniffer)
-	{
-		m_sniffer = new IE_Exp_HTML_Sniffer ();
-		m_4sniffer = new IE_Exp_HTML4_Sniffer ();
-	}
-	else
-	{
-		m_sniffer->ref();
-		m_4sniffer->ref();
-	}
-
-	mi->name = "HTML Exporter";
-	mi->desc = "Export HTML4.0 and XHTML Documents";
-	mi->version = ABI_VERSION_STRING;
-	mi->author = "Abi the Ant";
-	mi->usage = "No Usage";
-
-	IE_Exp::registerExporter (m_sniffer);
-	IE_Exp::registerExporter (m_4sniffer);
-	return 1;
-}
-
-ABI_FAR_CALL
-int abi_plugin_unregister (XAP_ModuleInfo * mi)
-{
-	mi->name = 0;
-	mi->desc = 0;
-	mi->version = 0;
-	mi->author = 0;
-	mi->usage = 0;
-
-	UT_ASSERT (m_sniffer);
-	UT_ASSERT (m_4sniffer);
-
-	IE_Exp::unregisterExporter (m_sniffer);
-	IE_Exp::unregisterExporter (m_4sniffer);
-
-	if (!m_sniffer->unref())
-	{
-		m_sniffer = 0;
-	}
-
-	if(!m_4sniffer->unref())
-	{
-	    m_4sniffer = 0;
-	}
-
-	return 1;
-}
-
-ABI_FAR_CALL
-int abi_plugin_supports_version (UT_uint32 major, UT_uint32 minor, 
-								 UT_uint32 release)
-{
-	return isCurrentAbiVersion (major, minor, release) ? 1 : 0;
-}
-
-#endif
+// our XHTML is really HTML. This is a stupid IE bug. Sorry
 
 /*****************************************************************/
 /*****************************************************************/
@@ -189,7 +111,7 @@ bool IE_Exp_HTML4_Sniffer::getDlgLabels(const char ** pszDesc,
 IE_Exp_HTML::IE_Exp_HTML(PD_Document * pDocument, bool is4)
   : IE_Exp(pDocument), m_pListener(0), m_bIs4(is4)
 {
-	m_error = 0;
+	m_error = UT_OK;
 }
 
 IE_Exp_HTML::~IE_Exp_HTML()
@@ -199,6 +121,8 @@ IE_Exp_HTML::~IE_Exp_HTML()
 /*****************************************************************/
 /*****************************************************************/
 
+#define IS_TRANSPARENT_COLOR(c) (!UT_strcmp(c, "transparent"))
+
 #define BT_NORMAL		1
 #define BT_HEADING1		2
 #define BT_HEADING2		3
@@ -247,9 +171,10 @@ protected:
 	void				_outputInheritanceLine(const char* ClassName);
 	void 				_outputBegin(PT_AttrPropIndex api);
 	void				_handleDataItems(void);
-	void				_convertFontSize(char* szDest, const char* pszFontSize);
-	void				_convertColor(char* szDest, const char* pszColor);
+	void				_convertFontSize(UT_String& szDest, const char* pszFontSize);
+	void				_convertColor(UT_String& szDest, const char* pszColor);
 	void				_storeStyles(void);
+	char *				_stripSuffix(const char* from, char delimiter);
 	
 	PD_Document *		m_pDocument;
 	IE_Exp_HTML *		m_pie;
@@ -280,7 +205,7 @@ protected:
 
 static char* removeWhiteSpace(const char * text)
 {
-	char* temp = static_cast<char *>(calloc(strlen(text)+1, sizeof(char)));
+	char* temp = static_cast<char *>(UT_calloc(strlen(text)+1, sizeof(char)));
 	char* ref = temp;
 	char* ptr = (char *)text; // only a pointer for quick traversal
 
@@ -415,26 +340,52 @@ void s_HTML_Listener::_openTag(PT_AttrPropIndex api)
 	{
 		const XML_Char * szValue;
 		const XML_Char * szDefault = "Normal";
+		/*	TODO: Perhaps the above should be/is a #define somewhere?	*/
 		const XML_Char * szLevel;
 		const XML_Char * szListID;
 		const XML_Char * szStyleType;
 		UT_uint16 *popped, *pushed;
 
+		/*	This is the point at which we differentiate between different
+		 *	types of tags in HTML.  We do a sequence of checks on the "style"
+		 *	and other useful attributes of the current block.  First we check
+		 *	if we are in a block which has a named "style" or which contains
+		 *	list information.
+		 *
+		 *	Weaknesses in this code include the mutability of our stored
+		 *	document state.  I've had it happen where the representation of
+		 *	lists in the abiword format changes, which tends to break this
+		 *	code.
+		 */
+
 		if (
 		   (pAP->getAttribute(PT_STYLE_ATTRIBUTE_NAME, szValue) ||
 		   	(pAP->getAttribute(PT_LISTID_ATTRIBUTE_NAME, szListID) && 
 			 0 != UT_strcmp(szListID, "0")))
 		   )
 		{
-			if(pAP->getAttribute(PT_LISTID_ATTRIBUTE_NAME, szListID))
-			{	// we're in a list
-				if(!pAP->getAttribute(PT_STYLE_ATTRIBUTE_NAME, szValue)) szValue = szDefault;
+			if(pAP->getAttribute(PT_LISTID_ATTRIBUTE_NAME, szListID)
+				&& 0 != UT_strcmp(szListID, "0"))
+			{
+				/*	A nonzero "listid" attribute value indicates that we
+				 *	are in a list item, so we need to process it, HTML-style.
+				 */
+
+				/*	Specify a default style name for this list item if it
+				 *	doesn't already have one.	*/
+				if(!pAP->getAttribute(PT_STYLE_ATTRIBUTE_NAME, szValue)) 
+					szValue = szDefault;
+
+				/*	Find out how deeply nested this list item is.	*/
 				pAP->getAttribute("level", szLevel);
 				m_iListDepth = atoi((const char*) szLevel);
 				if(!pAP->getProperty("list-style", szStyleType))
 				{
 					szStyleType = szValue;
 				}
+
+				/*	If our list is getting deeper, we need to start a
+				 *	nested list.	*/
 				if(m_iListDepth > m_iPrevListDepth)
 				{
 					if(UT_strcmp((const char*) szStyleType, "Bullet List") != 0)
@@ -452,6 +403,11 @@ void s_HTML_Listener::_openTag(PT_AttrPropIndex api)
 					pushed = new UT_uint16(m_iBlockType);
 					m_utsListType.push(pushed);
 				}
+				/*	If our list is not getting deeper, but the list type
+				 *	switched from a numbered list to a bullet list, then we
+				 *	pop list types off the stack until we are at the same
+				 *	depth, closing out lists as necessary.  This may only be
+				 *	once, if we are already at the same depth. */
 				else if(m_iPrevListDepth > 0 && (
 					(m_iBlockType == BT_BULLETLIST && (
 					 UT_strcmp((const char*) szStyleType, 
@@ -461,9 +417,10 @@ void s_HTML_Listener::_openTag(PT_AttrPropIndex api)
 					 UT_strcmp((const char*) szStyleType, 
 					 	"Bullet List") == 0 ||
 					 _inherits((const char*) szStyleType, "BulletList"))) ) )
-				{						/* list type switched */
+				{
 					m_pie->write("</li>\r\n");
 
+					/*	Toggle the current block type	*/
 					m_iBlockType = (m_iBlockType == BT_NUMBEREDLIST ?
 						BT_BULLETLIST : BT_NUMBEREDLIST);
 
@@ -527,7 +484,7 @@ void s_HTML_Listener::_openTag(PT_AttrPropIndex api)
 				}
 				m_pie->write("<li");
 				m_iPrevListDepth = m_iListDepth;
-				wasWritten = true;	
+				wasWritten = true;
 			}
 			else 
 			{
@@ -797,7 +754,7 @@ void s_HTML_Listener::_openSection(PT_AttrPropIndex api)
 	m_pie->write("<div>\r\n");
 }
 
-void s_HTML_Listener::_convertColor(char* szDest, const char* pszColor)
+void s_HTML_Listener::_convertColor(UT_String& szDest, const char* pszColor)
 {
 	/*
 	  TODO we might want to be a little more careful about this.
@@ -806,10 +763,10 @@ void s_HTML_Listener::_convertColor(char* szDest, const char* pszColor)
 	  forgiving than we are, so this is probably not a big
 	  problem.
 	*/
-	strcpy(szDest, pszColor);
+	szDest = pszColor;
 }
 
-void s_HTML_Listener::_convertFontSize(char* szDest, const char* pszFontSize)
+void s_HTML_Listener::_convertFontSize(UT_String& szDest, const char* pszFontSize)
 {
 	double fSizeInPoints = UT_convertToPoints(pszFontSize);
 
@@ -820,31 +777,31 @@ void s_HTML_Listener::_convertFontSize(char* szDest, const char* pszFontSize)
 	
 	if (fSizeInPoints <= 7)
 	{
-		strcpy(szDest, "1");
+		szDest = "1";
 	}
 	else if (fSizeInPoints <= 10)
 	{
-		strcpy(szDest, "2");
+		szDest = "2";
 	}
 	else if (fSizeInPoints <= 12)
 	{
-		strcpy(szDest, "3");
+		szDest = "3";
 	}
 	else if (fSizeInPoints <= 16)
 	{
-		strcpy(szDest, "4");
+		szDest = "4";
 	}
 	else if (fSizeInPoints <= 24)
 	{
-		strcpy(szDest, "5");
+		szDest = "5";
 	}
 	else if (fSizeInPoints <= 36)
 	{
-		strcpy(szDest, "6");
+		szDest = "6";
 	}
 	else
 	{
-		strcpy(szDest, "7");
+		szDest = "7";
 	}
 }
 
@@ -872,7 +829,6 @@ void s_HTML_Listener::_openSpan(PT_AttrPropIndex api)
 		return;
 	}
 	
-	m_bWroteText = true;
 	const PP_AttrProp * pAP = NULL;
 	bool bHaveProp = m_pDocument->getAttrProp(api,&pAP);
 	
@@ -1087,14 +1043,22 @@ void s_HTML_Listener::_openSpan(PT_AttrPropIndex api)
 			{
 				if (!span)
 				  {
-				    m_pie->write("<span style=\"color:#");
-				    m_pie->write(pszColor);
+				    if(IS_TRANSPARENT_COLOR(pszColor))
+				      m_pie->write("<span style=\"");
+				    else
+				      {
+					m_pie->write("<span style=\"color:#");
+					m_pie->write(pszColor);
+				      }
 				    span = true;
 				}
 				else 
 				{
-					m_pie->write("; color:#");	
-					m_pie->write(pszColor);
+				  if(!IS_TRANSPARENT_COLOR(pszColor))
+				    {
+				      m_pie->write("; color:#");	
+				      m_pie->write(pszColor);
+				    }
 				}
 			}
 			
@@ -1127,32 +1091,24 @@ void s_HTML_Listener::_openSpan(PT_AttrPropIndex api)
 	
 			}
 			
-			char szSize[4];
-
 			if (pszFontSize)
 			{
 				if (!span)
 				{
-					m_pie->write("<span style=\"font-size: ");	
 					setlocale (LC_NUMERIC, "C");
-					sprintf(szSize, "%f", UT_convertToPoints(pszFontSize));
+					m_pie->write(UT_String_sprintf("<span style=\"font-size: %fpt", UT_convertToPoints(pszFontSize)));
 					setlocale (LC_NUMERIC, "");
-					m_pie->write(szSize);
-					m_pie->write("pt");
 					span = true;
 				}
 				else 
 				{
-					m_pie->write("; font-size: ");	
 					setlocale (LC_NUMERIC, "C");
-					sprintf(szSize, "%f", UT_convertToPoints(pszFontSize));
+					m_pie->write(UT_String_sprintf("; font-size: %fpt", UT_convertToPoints(pszFontSize)));
 					setlocale (LC_NUMERIC, "");
-					m_pie->write(szSize);
-					m_pie->write("pt");
 				}
 			}
 			
-			if (pszBgColor)
+			if (pszBgColor && !IS_TRANSPARENT_COLOR(pszBgColor))
 			{
 				if (!span)
 				{
@@ -1166,6 +1122,11 @@ void s_HTML_Listener::_openSpan(PT_AttrPropIndex api)
 					m_pie->write(pszBgColor);
 				}
 			}
+			else if (pszBgColor && !span)
+			  {
+			    m_pie->write("<span style=\"");
+			    span = true;
+			  }
 		}
 
 		char* szStyle = NULL;
@@ -1190,7 +1151,7 @@ void s_HTML_Listener::_openSpan(PT_AttrPropIndex api)
 	the span should have rtl placement on a line, but it will ignore this 
 	value when printing the actual span.
 */
-		if(!span && (pAP->getProperty("dir-override", szValue) || pAP->getProperty("dir", szValue)))
+		if(!span && (pAP->getProperty("dir-override", szValue) /*|| pAP->getProperty("dir", szValue)###TF*/))
 		{
 			if(*szValue == 'r' || *szValue == 'l')
 			{
@@ -1206,7 +1167,7 @@ void s_HTML_Listener::_openSpan(PT_AttrPropIndex api)
 		{
 			m_pie->write("\"");
 #ifdef BIDI_ENABLED
-			if (!bDir && (pAP->getProperty("dir-override", szValue) || pAP->getProperty("dir", szValue)))
+			if (!bDir && (pAP->getProperty("dir-override", szValue) /*|| pAP->getProperty("dir", szValue)###TF*/))
 			{
 				if(*szValue == 'r' || *szValue == 'l')
 				{
@@ -1260,6 +1221,7 @@ void s_HTML_Listener::_closeSpan(void)
 			(pAP->getProperty("color", szValue))
 		    || (pAP->getProperty("font-size", szValue))
 		    || (pAP->getProperty("font-family", szValue))
+		    || (pAP->getProperty("bgcolor", szValue))
 			)
 		{
 			closeSpan = true;
@@ -1313,8 +1275,6 @@ void s_HTML_Listener::_closeSpan(void)
 
 void s_HTML_Listener::_outputData(const UT_UCSChar * data, UT_uint32 length)
 {
-	m_bWroteText = true;
-
 	UT_String sBuf;
 	const UT_UCSChar * pData;
 
@@ -1327,6 +1287,11 @@ void s_HTML_Listener::_outputData(const UT_UCSChar * data, UT_uint32 length)
 
 	for (pData=data; (pData<data+length); /**/)
 	{
+		if (*pData != ' ' && *pData != '\t')
+			// There has been some non-whitespace data output
+			m_bWroteText = true;
+		
+		// Check to see if the character which follows this one is whitespace 
 		pData++;
 		if (*pData == ' ' || *pData == '\t')
 		{
@@ -1411,23 +1376,33 @@ void s_HTML_Listener::_outputData(const UT_UCSChar * data, UT_uint32 length)
 			break;
 
 		case UCS_FF:					// page break, convert to line break
-			sBuf += "<br />";
+			if (!m_bIs4)
+			{
+				sBuf += "<br />";
+			}
+			else
+			{
+				sBuf += "<br>";
+			}
 			pData++;
 			break;
 		
 		default:
 			if (*pData > 0x007f)
 			{
+#ifdef IE_EXP_HTML_UTF8_OPTIONAL
+				// "try_nativeToU(0xa1) == 0xa1" looks dodgy to me (fjf)
 				if(XAP_EncodingManager::get_instance()->isUnicodeLocale() || 
 				   (XAP_EncodingManager::get_instance()->try_nativeToU(0xa1) == 0xa1))
-
 				{
+#endif /* IE_EXP_HTML_UTF8_OPTIONAL */
 					XML_Char * pszUTF8 = UT_encodeUTF8char(*pData++);
 					while (*pszUTF8)
 					{
 						sBuf += (char)*pszUTF8;
 						pszUTF8++;
 					}
+#ifdef IE_EXP_HTML_UTF8_OPTIONAL
 				}
 				else
 				{
@@ -1444,10 +1419,7 @@ void s_HTML_Listener::_outputData(const UT_UCSChar * data, UT_uint32 length)
 					UT_UCSChar c = XAP_EncodingManager::get_instance()->try_UToNative(*pData);
 					if (c==0 || c>255)
 					{
-						char localBuf[20];
-						char * plocal = localBuf;
-						sprintf(localBuf,"&#x%x;",*pData++);
-						sBuf += plocal;
+					  sBuf += UT_String_sprintf("&#x%x;",*pData++);
 					}
 					else
 					{
@@ -1455,6 +1427,7 @@ void s_HTML_Listener::_outputData(const UT_UCSChar * data, UT_uint32 length)
 						pData++;
 					}
 				}
+#endif /* IE_EXP_HTML_UTF8_OPTIONAL */
 			}
 			else
 			{
@@ -1582,18 +1555,22 @@ void s_HTML_Listener::_outputBegin(PT_AttrPropIndex api)
 	m_pie->write(m_pie->getFileName());
 	m_pie->write("</title>\r\n");
 	m_pie->write("<style type=\"text/css\">\r\n");
-	m_pie->write("body\r\n{\r\n\t");
 	if(bHaveProp && pAP)
 	{							// global page styles refer to the <body> tag
+		const XML_Char *szValue;
 		PD_Style *pStyle = NULL;
 		m_pDocument->getStyle("Normal", &pStyle);
-		if(pStyle)				// Add normal styles to body styles for
-		{						// global effect
-			const XML_Char *szName, *szValue;
+
+		m_pie->write("body\r\n{\r\n\t");
+		if(pStyle)				// Add normal styles to any descendent of the
+								// body for global effect
+		{						
+			const XML_Char *szName;
 			for(UT_uint16 i = 0; i < pStyle->getPropertyCount(); i++)
 			{
 				pStyle->getNthProperty(i, szName, szValue);
-				if(!is_CSS(static_cast<const char*>(szName)))
+				if(!is_CSS(static_cast<const char*>(szName)) ||
+					strstr(szName, "margin"))	// ... don't include margins
 					continue;
 
 				m_pie->write(szName);
@@ -1608,21 +1585,30 @@ void s_HTML_Listener::_outputBegin(PT_AttrPropIndex api)
 					m_pie->write("\"");
 					m_pie->write(szValue);
 					m_pie->write("\"");
-				}						// only quote non-keyword family names
+				} // only quote non-keyword family names
+				else if (!UT_strcmp(szName, "color"))
+				  {
+				    if (!IS_TRANSPARENT_COLOR(szValue))
+				      {
+					m_pie->write("#");
+					m_pie->write(szValue);
+				      }
+				  }
 				else
 					m_pie->write(szValue);
 				m_pie->write(";\r\n\t");
 			}
 		}
 
-		const XML_Char * szValue;
-
 		szValue = PP_evalProperty("background-color",
 								  NULL, NULL, pAP, m_pDocument, true);
-		m_pie->write("background-color: ");
-		if (*szValue != '#')
-			m_pie->write("#");
-		m_pie->write(szValue);
+		if(!IS_TRANSPARENT_COLOR(szValue))
+		  {
+		    m_pie->write("background-color: ");
+		    if (*szValue != '#')
+		      m_pie->write("#");
+		    m_pie->write(szValue);
+		  }
 		m_pie->write(";\r\n}\r\n\r\n@media print\r\n{\r\n\tbody\r\n\t{\r\n\t\t");
 		
 		szValue = PP_evalProperty("page-margin-top",
@@ -1681,13 +1667,15 @@ void s_HTML_Listener::_outputBegin(PT_AttrPropIndex api)
 			m_pie->write(myStyleName);
 			m_pie->write("\r\n{");
 
-			FREEP(myStyleName);
-
 			UT_uint32 i = 0, j = 0;
 			while(pAP_style->getNthAttribute(i++, szName, szValue))
 			{
 				if(!is_CSS(static_cast<const char*>(szName)))
 					continue;
+				if(strstr(myStyleName, "List") && (
+					strstr(szName, "margin") || UT_strcmp(szName, "text-indent") == 0))
+					continue;
+					// see line 770 of this file for reasoning behind skipping here
 				
 				m_pie->write("\r\n\t");
 				m_pie->write(szName);
@@ -1700,6 +1688,11 @@ void s_HTML_Listener::_outputBegin(PT_AttrPropIndex api)
 			{
 				if(!is_CSS(static_cast<const char*>(szName)))
 					continue;
+				if(strstr(myStyleName, "List") && (
+					strstr(szName, "margin") || UT_strcmp(szName, "text-indent") == 0))
+					continue;
+					// see line 770 of this file for reasoning behind skipping here
+
 				m_pie->write("\r\n\t");
 				m_pie->write(szName);
 				m_pie->write(": ");
@@ -1716,13 +1709,20 @@ void s_HTML_Listener::_outputBegin(PT_AttrPropIndex api)
 				}						// only quote non-keyword family names
 				else 
 				{
-					if(strstr(szName, "color"))
-						m_pie->write("#");
-					m_pie->write(szValue);
+				  if(strstr(szName, "color") && !IS_TRANSPARENT_COLOR(szValue))
+				    {
+				      m_pie->write("#");
+				      m_pie->write(szValue);
+				    }
+				  else
+				    {
+				      m_pie->write(szValue);
+				    }
 				}
 				
 				m_pie->write(";");
 			}
+			FREEP(myStyleName);
 
 			m_pie->write("\r\n}\r\n\r\n");
 		}
@@ -1766,7 +1766,7 @@ s_HTML_Listener::~s_HTML_Listener()
 		DELETEP(popped);
 	}
 
-	UT_VECTOR_PURGEALL(char*, m_utvDataIDs);
+	UT_VECTOR_FREEALL(char*, m_utvDataIDs);
 
 	m_pie->write("</body>\r\n");
 	m_pie->write("</html>\r\n");
@@ -1802,7 +1802,7 @@ bool s_HTML_Listener::populate(PL_StruxFmtHandle /*sfh*/,
 			const PX_ChangeRecord_Object * pcro = 
 				static_cast<const PX_ChangeRecord_Object *> (pcr);
 			const XML_Char* szValue;
-			char buf[16];
+			UT_String buf;
 
 			fd_Field* field;
 			PT_AttrPropIndex api = pcr->getIndexAP();
@@ -1817,24 +1817,52 @@ bool s_HTML_Listener::populate(PL_StruxFmtHandle /*sfh*/,
 
 				if(bHaveProp && pAP && pAP->getAttribute("dataid", szValue))
 				{
-					char* dataid = new char[strlen((char*) szValue)];
-					strcpy(dataid, (char*) szValue);
+					char* dataid = strdup((char*) szValue);
 
 					m_utvDataIDs.push_back(dataid);
 
-					sprintf(buf, "%d.png", m_iImgCnt++);
-					m_pie->write("<img alt=\"AbiWord Image");
+					char * temp = _stripSuffix(UT_basename(szValue), '_');
+					char * fstripped = _stripSuffix(temp, '.');
+					FREEP(temp);
+					UT_String_sprintf(buf, "%s.png", fstripped);
+					FREEP(fstripped);
+					
+					m_pie->write("<img alt=\"AbiWord Image ");
 					m_pie->write(buf);
 					m_pie->write("\" src=\"");
 					m_pie->write(UT_basename(m_pie->getFileName()));
-					m_pie->write("_d/");
+					m_pie->write("_data/");
 					m_pie->write(buf);
+					m_pie->write("\" ");
+					
+					const XML_Char * szWidth = 0;
+					const XML_Char * szHeight = 0;
+
+					if(pAP->getProperty("width", szWidth) &&
+					   pAP->getProperty("height", szHeight))
+					  {
+					    if(szWidth)
+					      {
+						UT_String_sprintf(buf, "%d", (int) UT_convertToDimension(szWidth, DIM_PX));
+						m_pie->write (" width=\"");
+						m_pie->write (buf);
+						m_pie->write ("\" ");
+					      }
+					    if(szHeight)
+					      {
+						UT_String_sprintf(buf, "%d", (int) UT_convertToDimension(szHeight, DIM_PX));
+						m_pie->write (" height=\"");
+						m_pie->write (buf);
+						m_pie->write ("\" ");
+					      }
+					  }
+
 					// close the img tag with "/" only in XHTML
 					if (! m_bIs4) {
-						m_pie->write("\" />\r\n");
+						m_pie->write(" />\r\n");
 					}
 					else {
-						m_pie->write("\">\r\n");
+						m_pie->write(" >\r\n");
 					}
 				}
 				return true;
@@ -1855,6 +1883,36 @@ bool s_HTML_Listener::populate(PL_StruxFmtHandle /*sfh*/,
 				}
 				return true;
 
+			case PTO_Hyperlink:
+				if(bHaveProp && pAP && pAP->getAttribute("xlink:href", szValue))
+				{
+					m_pie->write("<a href=\"");
+					m_pie->write(szValue);
+					m_pie->write("\">");
+				}
+				else
+				{
+					m_pie->write("</a>");
+				}
+				return true;
+
+			case PTO_Bookmark:
+				if(bHaveProp && pAP && pAP->getAttribute("type", szValue))
+				{
+					if( UT_XML_stricmp(szValue, "start") == 0 )
+					{
+						pAP->getAttribute("name", szValue);
+						m_pie->write("<a name=\"");
+						m_pie->write(szValue);
+						m_pie->write("\">");
+					}
+					else
+					{
+						m_pie->write("</a>");
+					}
+				}
+				return true;
+
 			default:
 				UT_ASSERT(0);
 				return false;
@@ -1880,6 +1938,7 @@ bool s_HTML_Listener::populateStrux(PL_StruxDocHandle /*sdh*/,
 
 	switch (pcrx->getStruxType())
 	{
+	case PTX_SectionHdrFtr:
 	case PTX_Section:
 	{
 		_closeSpan();
@@ -1902,40 +1961,14 @@ bool s_HTML_Listener::populateStrux(PL_StruxDocHandle /*sdh*/,
 			}
 			else
 			{
-				m_bInSection = false;
-			}
-		}
-		else
-		{
-			m_bInSection = false;
-		}
-		
-		return true;
-	}
-
-	case PTX_SectionHdrFtr:
-	{
-		_closeSpan();
-		_closeTag();
-		_closeSection();
-
-		PT_AttrPropIndex indexAP = pcr->getIndexAP();
-		const PP_AttrProp* pAP = NULL;
-		if (m_pDocument->getAttrProp(indexAP, &pAP) && pAP)
-		{
-			const XML_Char* pszSectionType = NULL;
-			pAP->getAttribute("type", pszSectionType);
-			if (
-				!pszSectionType
-				|| (0 == UT_strcmp(pszSectionType, "doc"))
-				)
-			{
-				_openSection(pcr->getIndexAP());
+#if 1
+			  // export headers & footers
 				m_bInSection = true;
-			}
-			else
-			{
-				m_bInSection = false;
+				_openSection(pcr->getIndexAP());
+#else
+				// don't export headers and footers
+				m_bInSection = false ;
+#endif
 			}
 		}
 		else
@@ -1991,21 +2024,48 @@ bool s_HTML_Listener::signal(UT_uint32 /* iSignal */)
 
 UT_Error IE_Exp_HTML::_writeDocument(void)
 {
+  bool err = true;
 	m_pListener = new s_HTML_Listener(getDoc(),this, m_bIs4);
 	if (!m_pListener)
 		return UT_IE_NOMEMORY;
-	if (!getDoc()->tellListener(static_cast<PL_Listener *>(m_pListener)))
-		return UT_ERROR;
-	delete m_pListener;
-
-	m_pListener = NULL;
+	if (getDocRange())
+	  err = getDoc()->tellListenerSubset(static_cast<PL_Listener *>(m_pListener),getDocRange());
+	else 
+	  err = getDoc()->tellListener(static_cast<PL_Listener *>(m_pListener)) ;
+	DELETEP(m_pListener);
 	
-	return ((m_error) ? UT_IE_COULDNOTWRITE : UT_OK);
+	if ( m_error == UT_OK && err == true )
+	  return UT_OK;
+	return UT_IE_COULDNOTWRITE;
 }
 
 /*****************************************************************/
 /*****************************************************************/
 
+/*!
+   removes the suffix from a string by searching backwards for the specified 
+   character delimiter. If the delimiter is not found, a copy of the original 
+   string is returned
+   
+   eg. _stripSuffix("/home/user/file.png, '.') returns "/home/user/file" 
+       _stripSuffix("/home/user/foo_bar, '_') returns /home/user/foo 
+       _stripSuffix("/home/user/file.png, '_') returns /home/user/file.png"
+*/
+char *s_HTML_Listener::_stripSuffix(const char* from, char delimiter)
+{
+    char * fremove_s = (char *)malloc(strlen(from)+1);
+    strcpy(fremove_s, from);   
+
+    char * p = fremove_s + strlen(fremove_s);
+    while ((p >= fremove_s) && (*p != delimiter))
+        p--;
+	
+    if (p >= fremove_s)
+	*p = '\0';
+    
+    return fremove_s;
+}
+
 void s_HTML_Listener::_handleDataItems(void)
 {
  	const char * szName;
@@ -2027,32 +2087,41 @@ void s_HTML_Listener::_handleDataItems(void)
 		if(loc > -1)
 		{
 			FILE *fp;
-			char fname [1024]; // EVIL EVIL bad hardcoded buffer size
+			UT_String fname; // EVIL EVIL bad hardcoded buffer size
 			
-			sprintf(fname, "%s_d", m_pie->getFileName());
-			int result = m_pDocument->getApp()->makeDirectory(fname, 0750);
+			UT_String_sprintf(fname, "%s_data", m_pie->getFileName());
+			int result = m_pDocument->getApp()->makeDirectory(fname.c_str(), 0750);
 			
 			if (!UT_strcmp(szMimeType, "image/svg-xml"))
-				sprintf(fname, "%s/%d.svg", fname, loc);
+				UT_String_sprintf(fname, "%s/%s_%d.svg", fname.c_str(), szName, loc);
 			if (!UT_strcmp(szMimeType, "text/mathml"))
-				sprintf(fname, "%s/%d.mathml", fname, loc);
+				UT_String_sprintf(fname, "%s/%s_%d.mathml", fname.c_str(), szName, loc);
 			else // PNG Image
-				sprintf(fname, "%s/%d.png", fname, loc);
-			
-			fp = fopen (fname, "wb+");
-			
-			if(!fp)
-				continue;
-			
-			int cnt = 0, len = pByteBuf->getLength();
-			
-			while (cnt < len)
-			{
-				cnt += fwrite (pByteBuf->getPointer(cnt), 
-							   sizeof(UT_Byte), len-cnt, fp);
+			{  
+				char * temp = _stripSuffix(UT_basename(szName), '_');
+				char * fstripped = _stripSuffix(temp, '.');
+				FREEP(temp);
+				UT_String_sprintf(fname, "%s/%s.png", fname.c_str(), fstripped);
+				FREEP(fstripped);
 			}
 			
-			fclose(fp);
+			if (!UT_isRegularFile(fname.c_str()))
+			{
+			    fp = fopen (fname.c_str(), "wb+");
+			
+			    if(!fp)
+				    continue;
+			
+			    int cnt = 0, len = pByteBuf->getLength();
+			
+			    while (cnt < len)
+			    {
+				    cnt += fwrite (pByteBuf->getPointer(cnt), 
+							     sizeof(UT_Byte), len-cnt, fp);
+			    }
+			
+			    fclose(fp);
+			}
 		}
 	}
 	
