@@ -81,7 +81,7 @@ ABI_FAR_CALL
 int abi_plugin_supports_version (UT_uint32 major, UT_uint32 minor, 
 								 UT_uint32 release)
 {
-	return isCurrentAbiVersion (major, minor, release) ? 1 : 0;
+  return 1;
 }
 
 #endif
