@@ -27,7 +27,7 @@ class s_XSL_FO_Listener;
 
 // The exporter/writer for the XML/XSL FO spec
 
-class IE_Exp_XSL_FO_Sniffer : public IE_ExpSniffer
+class ABI_EXPORT IE_Exp_XSL_FO_Sniffer : public IE_ExpSniffer
 {
 	friend class IE_Exp;
 
@@ -43,17 +43,18 @@ public:
 										IE_Exp ** ppie);
 };
 
-class IE_Exp_XSL_FO : public IE_Exp
+class ABI_EXPORT IE_Exp_XSL_FO : public IE_Exp
 {
 public:
 	IE_Exp_XSL_FO(PD_Document * pDocument);
 	virtual ~IE_Exp_XSL_FO();
 	
 protected:
-	virtual UT_Error	_writeDocument(void);
+	virtual UT_Error	_writeDocument();
 	
- private:
+private:
 	s_XSL_FO_Listener *	m_pListener;
+	UT_uint32 m_error;
 };
 
 #endif /* IE_EXP_XSL_FO_H */
