@@ -32,7 +32,6 @@
 
 
 #include "ap_Dialog_Id.h"
-#include "xap_App.h"
 #include "xap_DialogFactory.h"
 #include "xap_Dlg_Encoding.h"
 #include "ap_Prefs.h"
@@ -113,9 +112,10 @@ bool ImportStream::getRawChar(UT_UCSChar &ucs)
 	// Surrogates
 	UT_ASSERT(!(wc >= 0xD800 && wc <= 0xDFFF));
 	// Private Use Area
-	UT_ASSERT(!((wc >= 0xDB80 && wc <= 0xDBFF)||(wc >= 0xE000 && wc <= 0xF8FF)));
+	// No, Private User Area is not evil!  Commenting it out for now.
+	/* UT_ASSERT(!((wc >= 0xDB80 && wc <= 0xDBFF)||(wc >= 0xE000 && wc <= 0xF8FF))); */
 	// AbiWord control characters
-	UT_ASSERT(wc != UCS_FIELDSTART && wc != UCS_FIELDEND);
+	UT_ASSERT(wc != UCS_FIELDSTART && wc != UCS_FIELDEND && wc != UCS_BOOKMARKSTART && wc != UCS_BOOKMARKEND);
 	// Illegal characters
 	UT_ASSERT(wc != 0xFFFE && wc != 0xFFFF);
 
@@ -205,7 +205,15 @@ bool Inserter::insertBlock()
 		m_dPos++;
 	}
 	else
-		bRes = m_pDocument->appendStrux(PTX_Block, NULL);
+	  {
+	    // text gets applied in the Normal style
+	    const XML_Char * propsArray[3];
+	    propsArray[0] = "style";
+	    propsArray[1] = "Normal";
+	    propsArray[2] = 0;
+
+	    bRes = m_pDocument->appendStrux(PTX_Block, (const XML_Char **)propsArray);
+	  }
 
 	return bRes;
 }
@@ -244,10 +252,10 @@ bool Inserter::insertSpan(UT_GrowBuf &b)
  We also don't want to steal recognition when user wants to use
  the Encoded Text importer.
  */
-bool IE_Imp_Text_Sniffer::recognizeContents(const char * /* szBuf */,
+UT_Confidence_t IE_Imp_Text_Sniffer::recognizeContents(const char * /* szBuf */,
 											UT_uint32 /* iNumbytes */)
 {
-	return false;
+  return UT_CONFIDENCE_ZILCH; // anything can be a text document - we'll unfairly weight this down
 }
 
 /*!
@@ -402,9 +410,12 @@ IE_Imp_Text_Sniffer::UCS2_Endian IE_Imp_Text_Sniffer::_recognizeUCS2(const char
   Check filename extension for filetypes we support
  \param szSuffix Filename extension
  */
-bool IE_Imp_Text_Sniffer::recognizeSuffix(const char * szSuffix)
+UT_Confidence_t IE_Imp_Text_Sniffer::recognizeSuffix(const char * szSuffix)
 {
-	return (!UT_stricmp (szSuffix, ".txt") || !UT_stricmp(szSuffix, ".text"));
+  if (!UT_stricmp (szSuffix, ".txt") || !UT_stricmp(szSuffix, ".text") || !UT_stricmp (szSuffix, ".doc"))
+    return UT_CONFIDENCE_PERFECT;
+
+  return UT_CONFIDENCE_ZILCH;
 }
 
 UT_Error IE_Imp_Text_Sniffer::constructImporter(PD_Document * pDocument,
@@ -430,19 +441,21 @@ bool IE_Imp_Text_Sniffer::getDlgLabels(const char ** pszDesc,
 
  We don't attempt to recognize.  User must specifically choose Encoded Text.
  */
-bool IE_Imp_EncodedText_Sniffer::recognizeContents(const char * /* szBuf */,
+UT_Confidence_t IE_Imp_EncodedText_Sniffer::recognizeContents(const char * /* szBuf */,
 												   UT_uint32 /* iNumbytes */)
 {
-	return false;
+	return UT_CONFIDENCE_ZILCH;
 }
 
 /*!
   Check filename extension for filetypes we support
  \param szSuffix Filename extension
  */
-bool IE_Imp_EncodedText_Sniffer::recognizeSuffix(const char * szSuffix)
+UT_Confidence_t IE_Imp_EncodedText_Sniffer::recognizeSuffix(const char * szSuffix)
 {
-	return (!UT_stricmp (szSuffix, ".txt") || !UT_stricmp(szSuffix, ".text"));
+  if (!UT_stricmp (szSuffix, ".txt") || !UT_stricmp(szSuffix, ".text"))
+    return UT_CONFIDENCE_POOR;
+  return UT_CONFIDENCE_ZILCH;
 }
 
 UT_Error IE_Imp_EncodedText_Sniffer::constructImporter(PD_Document * pDocument,
@@ -609,10 +622,16 @@ UT_Error IE_Imp_Text::_constructStream(ImportStream *& pStream, FILE * fp)
  */
 UT_Error IE_Imp_Text::_writeHeader(FILE * /* fp */)
 {
-	X_ReturnNoMemIfError(getDoc()->appendStrux(PTX_Section, NULL));
-	X_ReturnNoMemIfError(getDoc()->appendStrux(PTX_Block, NULL));
+  // text gets applied in the Normal style
+  const XML_Char * propsArray[3];
+  propsArray[0] = "style";
+  propsArray[1] = "Normal";
+  propsArray[2] = 0;
 
-	return UT_OK;
+  X_ReturnNoMemIfError(getDoc()->appendStrux(PTX_Section, NULL));
+  X_ReturnNoMemIfError(getDoc()->appendStrux(PTX_Block, (const XML_Char**)propsArray));
+
+  return UT_OK;
 }
 
 /*!
