@@ -27,7 +27,7 @@ class PD_Document;
 
 // The importer/reader for XHTML 1.0
 
-class IE_Imp_XHTML_Sniffer : public IE_ImpSniffer
+class ABI_EXPORT IE_Imp_XHTML_Sniffer : public IE_ImpSniffer
 {
 	friend class IE_Imp;
 
@@ -35,9 +35,9 @@ public:
 	IE_Imp_XHTML_Sniffer() {}
 	virtual ~IE_Imp_XHTML_Sniffer() {}
 
-	virtual bool recognizeContents (const char * szBuf, 
+	virtual UT_Confidence_t recognizeContents (const char * szBuf, 
 									UT_uint32 iNumbytes);
-	virtual bool recognizeSuffix (const char * szSuffix);
+	virtual UT_Confidence_t recognizeSuffix (const char * szSuffix);
 	virtual bool getDlgLabels (const char ** szDesc,
 							   const char ** szSuffixList,
 							   IEFileType * ft);
@@ -46,23 +46,30 @@ public:
 
 };
 
-class IE_Imp_XHTML : public IE_Imp_XML
+class ABI_EXPORT IE_Imp_XHTML : public IE_Imp_XML
 {
 public:
     IE_Imp_XHTML(PD_Document * pDocument);
     ~IE_Imp_XHTML();
 
-    void			_startElement(const XML_Char *name, 
-								  const XML_Char **atts);
-    void			_endElement(const XML_Char *name);
+    virtual UT_Error	importFile(const char * szFilename);
 
-protected:
-    
+    void			startElement(const XML_Char *name, 
+								  const XML_Char **atts);
+    void			endElement(const XML_Char *name);
+
+    virtual void charData (const XML_Char * buffer, int length);
+
+private:
 	enum listType {L_NONE = 0, L_OL = 1, L_UL = 2 } m_listType;
 	UT_uint16	m_iListID;
+	bool        m_bFirstDiv;
 	UT_uint16	m_iNewListID;
 
 	UT_Stack	m_utsParents;
+	XML_Char *  m_szBookMarkName;
+
+	bool        m_addedPTXSection;
 };
 
 #endif /* IE_IMP_XHTML_H */
