@@ -24,6 +24,10 @@
 #include "data.h"
 #include "list.h"
 
+#ifdef DMALLOC
+#include <dmalloc.h>
+#endif
+
 static psiconv_color clone_color(psiconv_color color);
 static psiconv_font clone_font(psiconv_font font);
 static psiconv_border clone_border(psiconv_border border);
@@ -34,6 +38,9 @@ static void psiconv_free_in_line_layout_aux(void * layout);
 static void psiconv_free_paragraph_aux(void * paragraph);
 static void psiconv_free_paint_data_section_aux(void * section);
 static void psiconv_free_clipart_section_aux(void * section);
+static void psiconv_free_formula_aux(void *data);
+static void psiconv_free_sheet_worksheet_aux (void *data);
+static void psiconv_free_sheet_variable_aux(void * variable);
 
 static psiconv_word_styles_section psiconv_empty_word_styles_section(void);
 static psiconv_text_and_layout psiconv_empty_text_and_layout(void);
@@ -42,6 +49,11 @@ static psiconv_page_header psiconv_empty_page_header(void);
 static psiconv_page_layout_section psiconv_empty_page_layout_section(void);
 static psiconv_word_status_section psiconv_empty_word_status_section(void);
 static psiconv_word_f psiconv_empty_word_f(void);
+static psiconv_sheet_status_section psiconv_empty_sheet_status_section(void);
+static psiconv_formula_list psiconv_empty_formula_list(void);
+static psiconv_sheet_workbook_section 
+                                  psiconv_empty_sheet_workbook_section(void);
+static psiconv_sheet_f psiconv_empty_sheet_f(void);
 static psiconv_texted_f psiconv_empty_texted_f(void);
 static psiconv_paint_data_section psiconv_empty_paint_data_section(void);
 static psiconv_pictures psiconv_empty_pictures(void);
@@ -147,6 +159,7 @@ psiconv_paragraph_layout psiconv_basic_paragraph_layout(void)
     psiconv_bool_false,    /* keep_with_next      */
     psiconv_bool_false,    /* on_next_page        */
     psiconv_bool_false,    /* no_widow_protection */
+    psiconv_bool_false,    /* wrap_to_fit_cell    */
     0.0,                   /* left_margin         */
     &bullet,               /* bullet              */
     &no_border,            /* left_border         */
@@ -311,6 +324,34 @@ psiconv_word_style psiconv_get_style (psiconv_word_styles_section ss, int nr)
     return psiconv_list_get(ss->styles,0xff - nr);
 }
 
+psiconv_formula psiconv_get_formula (psiconv_formula_list ss, int nr)
+{
+  return psiconv_list_get(ss,psiconv_list_length(ss)-nr-1);
+}
+
+/* TODO: What if a cell is both in a default row and a default column?!? */
+psiconv_sheet_cell_layout psiconv_get_default_layout 
+                                       (psiconv_sheet_line_list row_defaults,
+                                        psiconv_sheet_line_list col_defaults,
+                                        psiconv_sheet_cell_layout cell_default,
+                                        int row,int col)
+{
+  int i;
+  psiconv_sheet_line line;
+  for (i = 0;i < psiconv_list_length(row_defaults);i++) {
+    line = psiconv_list_get(row_defaults,i);
+    if (line->position == row)
+      return line->layout;
+  }
+  for (i = 0;i < psiconv_list_length(col_defaults);i++) {
+    line = psiconv_list_get(col_defaults,i);
+    if (line->position == col)
+      return line->layout;
+  }
+  return cell_default;
+}
+
+
 void psiconv_free_color (psiconv_color color)
 {
   if (color)
@@ -403,6 +444,7 @@ void psiconv_free_word_styles_section(psiconv_word_styles_section styles)
     psiconv_free_word_style(styles->normal);
     if (styles->styles) 
       psiconv_list_free_el(styles->styles,psiconv_free_style_aux);
+    free(styles);
   }
 }
 
@@ -536,6 +578,218 @@ void psiconv_free_word_f(psiconv_word_f file)
   }
 }
 
+void psiconv_free_sheet_status_section(psiconv_sheet_status_section section)
+{
+  if (section) 
+    free(section);
+}
+
+void psiconv_free_sheet_numberformat(psiconv_sheet_numberformat numberformat)
+{
+  if (numberformat)
+    free(numberformat);
+}
+
+void psiconv_free_sheet_cell_layout(psiconv_sheet_cell_layout layout)
+{
+  psiconv_free_paragraph_layout(layout->paragraph);
+  psiconv_free_character_layout(layout->character);
+  psiconv_free_sheet_numberformat(layout->numberformat);
+}
+
+void psiconv_free_sheet_cell_aux(void *cell)
+{
+  psiconv_sheet_cell data = cell;
+
+  psiconv_free_sheet_cell_layout(data->layout);
+
+  if ((data->type == psiconv_cell_string) && (data->data.dat_string))
+    free(data->data.dat_string);
+}
+
+void psiconv_free_sheet_cell(psiconv_sheet_cell cell)
+{
+  if (cell) {
+    psiconv_free_sheet_cell_aux(cell);
+    free(cell);
+  }
+}
+
+void psiconv_free_sheet_cell_list(psiconv_sheet_cell_list list)
+{
+  if (list) 
+    psiconv_list_free_el(list,psiconv_free_sheet_cell_aux);
+}
+
+void psiconv_free_sheet_line_aux(void *line)
+{
+  psiconv_sheet_line data = line;
+
+  psiconv_free_sheet_cell_layout(data->layout);
+}
+
+void psiconv_free_sheet_line(psiconv_sheet_line line)
+{
+  if (line) {
+    psiconv_free_sheet_line_aux(line);
+    free(line);
+  }
+}
+
+
+void psiconv_free_sheet_line_list(psiconv_sheet_line_list list)
+{
+  if (list) 
+    psiconv_list_free_el(list,psiconv_free_sheet_line_aux);
+}
+
+void psiconv_free_sheet_grid_break_list(psiconv_sheet_grid_break_list list)
+{
+  if (list)
+    psiconv_list_free(list);
+}
+
+void psiconv_free_sheet_grid_size(psiconv_sheet_grid_size s)
+{
+  if (s)
+    free(s);
+}
+
+void psiconv_free_sheet_grid_size_list(psiconv_sheet_grid_size_list list)
+{
+  if (list)
+    psiconv_list_free(list);
+}
+
+void psiconv_free_sheet_grid_section(psiconv_sheet_grid_section sec)
+{
+  if (sec) {
+    psiconv_free_sheet_grid_size_list(sec->row_heights);
+    psiconv_free_sheet_grid_size_list(sec->column_heights);
+    psiconv_free_sheet_grid_break_list(sec->row_page_breaks);
+    psiconv_free_sheet_grid_break_list(sec->column_page_breaks);
+    free(sec);
+  }
+}
+
+void psiconv_free_sheet_worksheet_aux (void *data)
+{
+  psiconv_sheet_worksheet section = data;
+  psiconv_free_sheet_cell_layout(section->default_layout);
+  psiconv_free_sheet_cell_list(section->cells);
+  psiconv_free_sheet_line_list(section->row_default_layouts);
+  psiconv_free_sheet_line_list(section->col_default_layouts);
+  psiconv_free_sheet_grid_section(section->grid);
+}
+
+void psiconv_free_sheet_worksheet(psiconv_sheet_worksheet sheet)
+{
+  if (sheet) {
+    psiconv_free_sheet_worksheet_aux(sheet);
+    free(sheet);
+  }
+}
+
+void psiconv_free_sheet_worksheet_list(psiconv_sheet_worksheet_list list)
+{
+  if (list) 
+    psiconv_list_free_el(list,psiconv_free_sheet_worksheet_aux);
+}
+
+void psiconv_free_formula_aux(void *data)
+{
+  psiconv_formula formula;
+  formula = data;
+  if (formula->type == psiconv_formula_dat_string) 
+    free(formula->data.dat_string);
+  else if ((formula->type != psiconv_formula_dat_int) &&
+           (formula->type != psiconv_formula_dat_var) &&
+           (formula->type != psiconv_formula_dat_float) &&
+           (formula->type != psiconv_formula_dat_cellref) &&
+           (formula->type != psiconv_formula_dat_cellblock) &&
+           (formula->type != psiconv_formula_dat_vcellblock) &&
+           (formula->type != psiconv_formula_mark_opsep) &&
+           (formula->type != psiconv_formula_mark_opend) &&
+           (formula->type != psiconv_formula_mark_eof)  &&
+           (formula->type != psiconv_formula_unknown)) 
+    psiconv_free_formula_list(formula->data.fun_operands);
+}
+
+void psiconv_free_formula(psiconv_formula formula)
+{
+  if (formula) {
+    psiconv_free_formula_aux(formula);
+    free(formula);
+  }
+}
+
+void psiconv_free_formula_list(psiconv_formula_list list)
+{
+  if (list) 
+    psiconv_list_free_el(list,psiconv_free_formula_aux);
+}
+
+void psiconv_free_sheet_name_section(psiconv_sheet_name_section section)
+{
+  if (section) {
+    if(section->name)
+      free(section->name);
+    free(section);
+  }
+}
+
+void psiconv_free_sheet_info_section(psiconv_sheet_info_section section)
+{
+  if (section) {
+    free(section);
+  }
+}
+
+void psiconv_free_sheet_variable_aux(void * variable)
+{
+  psiconv_sheet_variable var = variable;
+  if (var->name)
+    free(var->name);
+  if (var->type == psiconv_var_string)
+    free(var->data.dat_string);
+}
+
+void psiconv_free_sheet_variable(psiconv_sheet_variable var)
+{
+  if (var) {
+    psiconv_free_sheet_variable_aux(var);
+    free(var);
+  }
+}
+
+void psiconv_free_sheet_variable_list(psiconv_sheet_variable_list list)
+{
+  if (list) 
+    psiconv_list_free_el(list,psiconv_free_sheet_variable_aux);
+}
+
+void psiconv_free_sheet_workbook_section(psiconv_sheet_workbook_section section)
+{
+  if (section) {
+    psiconv_free_formula_list(section->formulas);
+    psiconv_free_sheet_worksheet_list(section->worksheets);
+    psiconv_free_sheet_name_section(section->name);
+    psiconv_free_sheet_info_section(section->info);
+    psiconv_free_sheet_variable_list(section->variables);
+    free(section);
+  }
+}
+
+void psiconv_free_sheet_f(psiconv_sheet_f file)
+{
+  if (file) { 
+    psiconv_free_page_layout_section(file->page_sec);
+    psiconv_free_sheet_status_section(file->status_sec);
+    psiconv_free_sheet_workbook_section(file->workbook_sec);
+    free(file);
+  }
+}
+
 void psiconv_free_texted_f(psiconv_texted_f file)
 {
   if (file) {
@@ -640,6 +894,8 @@ void psiconv_free_file(psiconv_file file)
       psiconv_free_sketch_f((psiconv_sketch_f) file->file);
     else if (file->type == psiconv_clipart_file)
       psiconv_free_clipart_f((psiconv_clipart_f) file->file);
+    else if (file->type == psiconv_sheet_file)
+      psiconv_free_sheet_f((psiconv_sheet_f) file->file);
     free(file);
   }
 }
@@ -920,6 +1176,64 @@ ERROR1:
   return NULL;
 }
 
+psiconv_sheet_status_section psiconv_empty_sheet_status_section(void)
+{
+  psiconv_sheet_status_section result;
+  if (!(result = malloc(sizeof(*result))))
+    return NULL;
+  result->show_horizontal_scrollbar = result->show_vertical_scrollbar = 
+        psiconv_triple_auto;
+  result->show_graph = psiconv_bool_false;
+  result->show_top_sheet_toolbar = result->show_side_sheet_toolbar = 
+        result->show_top_graph_toolbar = result->show_side_graph_toolbar = 
+        psiconv_bool_true;
+  result->cursor_row = result->cursor_column = 0;
+  result->sheet_display_size = result->graph_display_size = 1000;
+  return result;
+}
+
+psiconv_formula_list psiconv_empty_formula_list(void)
+{
+  return psiconv_list_new(sizeof(struct psiconv_formula_s));
+}
+
+psiconv_sheet_workbook_section psiconv_empty_sheet_workbook_section(void)
+{
+  psiconv_sheet_workbook_section result;
+  if (!(result = malloc(sizeof(*result))))
+    goto ERROR1;
+  if (!(result->formulas = psiconv_empty_formula_list()))
+    goto ERROR2;
+  return result;
+ERROR2:
+  free(result);
+ERROR1:
+  return NULL;
+}
+
+
+psiconv_sheet_f psiconv_empty_sheet_f(void)
+{
+  psiconv_sheet_f result;
+  if (!(result = malloc(sizeof(*result))))
+    goto ERROR1;
+  if (!(result->page_sec = psiconv_empty_page_layout_section()))
+    goto ERROR2;
+  if (!(result->status_sec = psiconv_empty_sheet_status_section()))
+    goto ERROR3;
+  if (!(result->workbook_sec = psiconv_empty_sheet_workbook_section()))
+    goto ERROR4;
+  return result;
+ERROR4:
+  psiconv_free_sheet_status_section(result->status_sec);
+ERROR3:
+  psiconv_free_page_layout_section(result->page_sec);
+ERROR2:
+  free(result);
+ERROR1:
+  return NULL;
+}
+
 psiconv_texted_f psiconv_empty_texted_f(void)
 {
   psiconv_texted_f result;
@@ -1062,6 +1376,9 @@ psiconv_file psiconv_empty_file(psiconv_file_type_t type)
   if (type == psiconv_word_file) {
     if (!(result->file =  psiconv_empty_word_f()))
       goto ERROR;
+  } else if (type == psiconv_sheet_file) {
+    if (!(result->file =  psiconv_empty_sheet_f()))
+      goto ERROR;
   } else if (type == psiconv_texted_file) {
     if (!(result->file =  psiconv_empty_texted_f()))
       goto ERROR;
