@@ -215,8 +215,7 @@ wvGetDgContainer (DgContainer * item, MSOFBH * msofbh, wvStream * fd)
 		count +=
 		    wvGetSpgrContainer (&
 					(item->spgrcontainer
-					 [item->no_spgrcontainer - 1]),
-&amsofbh, fd);
+					 [item->no_spgrcontainer - 1]), &amsofbh, fd);
 		break;
 	    default:
 		count += wvEatmsofbt (&amsofbh, fd);
@@ -303,8 +302,7 @@ wvGetSpgrContainer (SpgrContainer * item, MSOFBH * msofbh, wvStream * fd)
 		count +=
 		    wvGetSpgrContainer (&
 					(item->spgrcontainer
-					 [item->no_spgrcontainer - 1]),
-&amsofbh, fd);
+					 [item->no_spgrcontainer - 1]), &amsofbh, fd);
 		break;
 	    default:
 		count += wvEatmsofbt (&amsofbh, fd);
@@ -487,19 +485,20 @@ wv0x01 (Blip * blip, wvStream * fd, U32 len)
     MSOFBH amsofbh;
     FSPContainer item;
     U32 count = 0;
-    char test[3];
+/*    char test[3];*/
     int ret = 0;
 
 
     if (fd == NULL)
 	return (0);
 
+  /*lvm007@aha.ru fix hack as outdated look picf*/
     /* 
        temp hack to test older included bmps in word 6 and 7,
        should be wrapped in a modern escher strucure before getting
        to here, and then handled as normal
      */
-    test[2] = '\0';
+    /*test[2] = '\0';
     test[0] = read_8ubit (fd);
     test[1] = read_8ubit (fd);
     wvStream_rewind (fd);
@@ -509,7 +508,7 @@ wv0x01 (Blip * blip, wvStream * fd, U32 len)
 	  blip->type = msoblipDIB;
 	  return (1);
       }
-
+	*/
     while (count < len)
       {
 	  wvTrace (("count is %x,len is %x\n", count, len));
@@ -684,10 +683,12 @@ wvGetMSOFBH (MSOFBH * amsofbh, wvStream * fd)
 U32
 wvEatmsofbt (MSOFBH * amsofbh, wvStream * fd)
 {
-    U32 i;
-    for (i = 0; i < amsofbh->cbLength; i++)
-	read_8ubit (fd);
-    return (amsofbh->cbLength);
+    /* Ries (rvt@dds.nl)
+       Changed read to bound to 32bit instead of 8bit 
+       MS prolly does 32 bit bounderaries ??? */
+    U32 check=(amsofbh->cbLength + 3) >> 2;
+    wvStream_offset(fd,check << 2);
+    return (check << 2);
 }
 
 void
