@@ -53,6 +53,9 @@
 /*
   Include declarations.
 */
+#ifdef HAVE_CONFIG_H
+#include "config.h"
+#endif
 #include "magick.h"
 #include "defines.h"
 
